@@ -20,54 +20,37 @@
 #include "best_practices/best_practices_validation.h"
 #include "best_practices/bp_state.h"
 #include "state_tracker/buffer_state.h"
+#include "state_tracker/last_bound_state.h"
 #include "state_tracker/render_pass_state.h"
+#include "state_tracker/pipeline_state.h"
 #include <bitset>
 
 // Generic function to handle validation for all CmdDraw* type functions
-bool BestPractices::ValidateCmdDrawType(VkCommandBuffer cmd_buffer, const Location& loc) const {
+bool BestPractices::ValidateCmdDrawType(const bp_state::CommandBufferSubState& cb_state, const Location& loc) const {
     bool skip = false;
-    const auto cb_state = GetRead<bp_state::CommandBuffer>(cmd_buffer);
-    if (const auto* pipe = cb_state->GetCurrentPipeline(VK_PIPELINE_BIND_POINT_GRAPHICS)) {
-        if (const auto rp_state = cb_state->active_render_pass.get()) {
-            for (uint32_t i = 0; i < rp_state->create_info.subpassCount; ++i) {
-                const auto& subpass = rp_state->create_info.pSubpasses[i];
-                const auto* ds_state = pipe->DepthStencilState();
-                const uint32_t depth_stencil_attachment =
-                    GetSubpassDepthStencilAttachmentIndex(ds_state, subpass.pDepthStencilAttachment);
-                const auto* raster_state = pipe->RasterizationState();
-                if ((depth_stencil_attachment == VK_ATTACHMENT_UNUSED) && raster_state &&
-                    raster_state->depthBiasEnable == VK_TRUE) {
-                    skip |= LogWarning("BestPractices-vkCmdDraw-DepthBiasNoAttachment", cb_state->Handle(), loc,
-                                       "depthBiasEnable == VK_TRUE without a depth-stencil attachment.");
-                }
-            }
-        }
-    }
-    skip |= ValidatePushConstants(cmd_buffer, loc);
+    skip |= ValidatePushConstants(cb_state, loc);
     return skip;
 }
 
-bool BestPractices::ValidateCmdDispatchType(VkCommandBuffer cmd_buffer, const Location& loc) const {
+bool BestPractices::ValidateCmdDispatchType(const bp_state::CommandBufferSubState& cb_state, const Location& loc) const {
     bool skip = false;
-    skip |= ValidatePushConstants(cmd_buffer, loc);
+    skip |= ValidatePushConstants(cb_state, loc);
     return skip;
 }
 
-bool BestPractices::ValidatePushConstants(VkCommandBuffer cmd_buffer, const Location& loc) const {
+bool BestPractices::ValidatePushConstants(const bp_state::CommandBufferSubState& cb_state, const Location& loc) const {
     using Range = vvl::range<uint32_t>;
 
     bool skip = false;
 
-    const auto cb_state = GetRead<bp_state::CommandBuffer>(cmd_buffer);
-
-    if (!cb_state->push_constant_ranges_layout) {
+    if (!cb_state.base.push_constant_ranges_layout) {
         return skip;
     }
 
-    for (const VkPushConstantRange& push_constant_range : *cb_state->push_constant_ranges_layout) {
+    for (const VkPushConstantRange& push_constant_range : *cb_state.base.push_constant_ranges_layout) {
         Range layout_range(push_constant_range.offset, push_constant_range.offset + push_constant_range.size);
         uint32_t size_not_set = push_constant_range.size;
-        for (const vvl::CommandBuffer::PushConstantData& filled_pcr : cb_state->push_constant_data_chunks) {
+        for (const PushConstantData& filled_pcr : cb_state.push_constant_data_chunks) {
             Range filled_range(filled_pcr.offset, filled_pcr.offset + static_cast<uint32_t>(filled_pcr.values.size()));
             Range intersection = layout_range & filled_range;
             if (intersection.valid()) {
@@ -78,7 +61,7 @@ bool BestPractices::ValidatePushConstants(VkCommandBuffer cmd_buffer, const Loca
             }
         }
         if (size_not_set > 0) {
-            skip |= LogWarning("BestPractices-PushConstants", cmd_buffer, loc,
+            skip |= LogWarning("BestPractices-PushConstants", cb_state.base.Handle(), loc,
                                "Pipeline uses a push constant range with offset %" PRIu32 " and size %" PRIu32 ", but %" PRIu32
                                " bytes were never set with vkCmdPushConstants.",
                                push_constant_range.offset, push_constant_range.size, size_not_set);
@@ -89,35 +72,13 @@ bool BestPractices::ValidatePushConstants(VkCommandBuffer cmd_buffer, const Loca
     return skip;
 }
 
-void BestPractices::RecordCmdDrawType(bp_state::CommandBuffer& cb_state, uint32_t draw_count) {
+void BestPractices::RecordCmdDrawType(bp_state::CommandBufferSubState& cb_state, uint32_t draw_count) {
     if (VendorCheckEnabled(kBPVendorArm)) {
         RecordCmdDrawTypeArm(cb_state, draw_count);
     }
-    if (VendorCheckEnabled(kBPVendorNVIDIA)) {
-        RecordCmdDrawTypeNVIDIA(cb_state);
-    }
-
-    if (cb_state.render_pass_state.drawTouchAttachments) {
-        for (auto& touch : cb_state.render_pass_state.nextDrawTouchesAttachments) {
-            RecordAttachmentAccess(cb_state, touch.framebufferAttachment, touch.aspects);
-        }
-        // No need to touch the same attachments over and over.
-        cb_state.render_pass_state.drawTouchAttachments = false;
-    }
-
-    const auto* pipeline_state = cb_state.GetCurrentPipeline(VK_PIPELINE_BIND_POINT_GRAPHICS);
-    if (pipeline_state && !pipeline_state->IsDynamic(CB_DYNAMIC_STATE_VERTEX_INPUT_EXT)) {
-        if (pipeline_state->vertex_input_state && !pipeline_state->vertex_input_state->bindings.empty()) {
-            cb_state.uses_vertex_buffer = true;
-        }
-    } else {
-        if (!cb_state.dynamic_state_value.vertex_bindings.empty()) {
-            cb_state.uses_vertex_buffer = true;
-        }
-    }
 }
 
-void BestPractices::RecordCmdDrawTypeArm(bp_state::CommandBuffer& cb_state, uint32_t draw_count) {
+void BestPractices::RecordCmdDrawTypeArm(bp_state::CommandBufferSubState& cb_state, uint32_t draw_count) {
     auto& render_pass_state = cb_state.render_pass_state;
     // Each TBDR vendor requires a depth pre-pass draw call to have a minimum number of vertices/indices before it counts towards
     // depth prepass warnings First find the lowest enabled draw count
@@ -133,24 +94,17 @@ void BestPractices::RecordCmdDrawTypeArm(bp_state::CommandBuffer& cb_state, uint
     }
 }
 
-void BestPractices::RecordCmdDrawTypeNVIDIA(bp_state::CommandBuffer& cb_state) {
-    assert(VendorCheckEnabled(kBPVendorNVIDIA));
-
-    if (cb_state.nv.depth_test_enable && cb_state.nv.zcull_direction != ZcullDirection::Unknown) {
-        RecordSetScopeZcullDirection(cb_state, cb_state.nv.zcull_direction);
-        RecordZcullDraw(cb_state);
-    }
-}
-
 bool BestPractices::PreCallValidateCmdDraw(VkCommandBuffer commandBuffer, uint32_t vertexCount, uint32_t instanceCount,
                                            uint32_t firstVertex, uint32_t firstInstance, const ErrorObject& error_obj) const {
     bool skip = false;
+    const auto cb_state = GetRead<vvl::CommandBuffer>(commandBuffer);
+    const auto& sub_state = bp_state::SubState(*cb_state);
+    skip |= ValidateCmdDrawType(sub_state, error_obj.location);
 
     if (instanceCount == 0) {
         skip |= LogWarning("BestPractices-vkCmdDraw-instance-count-zero", device, error_obj.location.dot(Field::instanceCount),
                            "is zero.");
     }
-    skip |= ValidateCmdDrawType(commandBuffer, error_obj.location);
 
     return skip;
 }
@@ -159,18 +113,19 @@ bool BestPractices::PreCallValidateCmdDrawIndexed(VkCommandBuffer commandBuffer,
                                                   uint32_t firstIndex, int32_t vertexOffset, uint32_t firstInstance,
                                                   const ErrorObject& error_obj) const {
     bool skip = false;
+    const auto cb_state = GetRead<vvl::CommandBuffer>(commandBuffer);
+    const auto& sub_state = bp_state::SubState(*cb_state);
+    skip |= ValidateCmdDrawType(sub_state, error_obj.location);
 
     if (instanceCount == 0) {
         skip |= LogWarning("BestPractices-vkCmdDrawIndexed-instance-count-zero", device,
                            error_obj.location.dot(Field::instanceCount), "is zero.");
     }
-    skip |= ValidateCmdDrawType(commandBuffer, error_obj.location);
 
     // Check if we reached the limit for small indexed draw calls.
     // Note that we cannot update the draw call count here, so we do it in PostCallRecordCmdDrawIndexed.
-    const auto cb_state = GetRead<bp_state::CommandBuffer>(commandBuffer);
     if ((indexCount * instanceCount) <= kSmallIndexedDrawcallIndices &&
-        (cb_state->small_indexed_draw_call_count == kMaxSmallIndexedDrawcalls - 1) &&
+        (sub_state.small_indexed_draw_call_count == kMaxSmallIndexedDrawcalls - 1) &&
         (VendorCheckEnabled(kBPVendorArm) || VendorCheckEnabled(kBPVendorIMG))) {
         skip |= LogPerformanceWarning("BestPractices-vkCmdDrawIndexed-many-small-indexed-drawcalls", device, error_obj.location,
                                       "%s %s: The command buffer contains many small indexed drawcalls "
@@ -181,7 +136,7 @@ bool BestPractices::PreCallValidateCmdDrawIndexed(VkCommandBuffer commandBuffer,
     }
 
     if (VendorCheckEnabled(kBPVendorArm)) {
-        skip |= ValidateIndexBufferArm(*cb_state, indexCount, instanceCount, firstIndex, vertexOffset, firstInstance,
+        skip |= ValidateIndexBufferArm(sub_state, indexCount, instanceCount, firstIndex, vertexOffset, firstInstance,
                                        error_obj.location);
     }
 
@@ -213,26 +168,26 @@ bool BestPractices::PostTransformLRUCacheModel::query_cache(uint32_t value) {
     return false;
 }
 
-bool BestPractices::ValidateIndexBufferArm(const bp_state::CommandBuffer& cb_state, uint32_t indexCount, uint32_t instanceCount,
-                                           uint32_t firstIndex, int32_t vertexOffset, uint32_t firstInstance,
-                                           const Location& loc) const {
+bool BestPractices::ValidateIndexBufferArm(const bp_state::CommandBufferSubState& cb_state, uint32_t indexCount,
+                                           uint32_t instanceCount, uint32_t firstIndex, int32_t vertexOffset,
+                                           uint32_t firstInstance, const Location& loc) const {
     bool skip = false;
 
     // check for sparse/underutilised index buffer, and post-transform cache thrashing
-    const auto ib_state = Get<vvl::Buffer>(cb_state.index_buffer_binding.buffer);
+    const auto ib_state = Get<vvl::Buffer>(cb_state.base.index_buffer_binding.buffer);
     // If the maintenance6 feature is enabled, buffer can be VK_NULL_HANDLE. If buffer is VK_NULL_HANDLE and the nullDescriptor
     // feature is enabled, every index fetched results in a value of zero.
     if (!ib_state) {
         return skip;
     }
 
-    const VkIndexType ib_type = cb_state.index_buffer_binding.index_type;
+    const VkIndexType ib_type = cb_state.base.index_buffer_binding.index_type;
     const auto ib_memory_state = ib_state->MemoryState();
     if (!ib_memory_state) return skip;
 
     const void* ib_mem = ib_memory_state->p_driver_data;
 
-    const auto& last_bound_state = cb_state.lastBound[ConvertToLvlBindPoint(VK_PIPELINE_BIND_POINT_GRAPHICS)];
+    const auto& last_bound_state = cb_state.base.GetLastBoundGraphics();
     const bool primitive_restart_enable = last_bound_state.IsPrimitiveRestartEnable();
 
     // no point checking index buffer if the memory is nonexistant/unmapped, or if there is no graphics pipeline bound to this CB
@@ -371,43 +326,42 @@ bool BestPractices::ValidateIndexBufferArm(const bp_state::CommandBuffer& cb_sta
 void BestPractices::PostCallRecordCmdDrawIndexed(VkCommandBuffer commandBuffer, uint32_t indexCount, uint32_t instanceCount,
                                                  uint32_t firstIndex, int32_t vertexOffset, uint32_t firstInstance,
                                                  const RecordObject& record_obj) {
-    BaseClass::PostCallRecordCmdDrawIndexed(commandBuffer, indexCount, instanceCount, firstIndex, vertexOffset, firstInstance,
-                                               record_obj);
-    auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-    RecordCmdDrawType(*cb_state, indexCount * instanceCount);
+    auto cb_state = GetWrite<vvl::CommandBuffer>(commandBuffer);
+    auto& sub_state = bp_state::SubState(*cb_state);
+    RecordCmdDrawType(sub_state, indexCount * instanceCount);
 
     if ((indexCount * instanceCount) <= kSmallIndexedDrawcallIndices) {
-        cb_state->small_indexed_draw_call_count++;
+        sub_state.small_indexed_draw_call_count++;
     }
-
-    ValidateBoundDescriptorSets(*cb_state, VK_PIPELINE_BIND_POINT_GRAPHICS, record_obj.location.function);
 }
 
 bool BestPractices::PreCallValidateCmdDrawIndirect(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
                                                    uint32_t drawCount, uint32_t stride, const ErrorObject& error_obj) const {
     bool skip = false;
+    const auto cb_state = GetRead<vvl::CommandBuffer>(commandBuffer);
+    const auto& sub_state = bp_state::SubState(*cb_state);
+    skip |= ValidateCmdDrawType(sub_state, error_obj.location);
 
     if (drawCount == 0) {
         skip |= LogWarning("BestPractices-vkCmdDrawIndirect-draw-count-zero", device, error_obj.location.dot(Field::drawCount),
                            "is zero.");
     }
 
-    skip |= ValidateCmdDrawType(commandBuffer, error_obj.location);
-
     return skip;
 }
 
 bool BestPractices::PreCallValidateCmdDrawIndexedIndirect(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
                                                           uint32_t drawCount, uint32_t stride, const ErrorObject& error_obj) const {
     bool skip = false;
+    const auto cb_state = GetRead<vvl::CommandBuffer>(commandBuffer);
+    const auto& sub_state = bp_state::SubState(*cb_state);
+    skip |= ValidateCmdDrawType(sub_state, error_obj.location);
 
     if (drawCount == 0) {
         skip |= LogWarning("BestPractices-vkCmdDrawIndexedIndirect-draw-count-zero", device,
                            error_obj.location.dot(Field::drawCount), "is zero.");
     }
 
-    skip |= ValidateCmdDrawType(commandBuffer, error_obj.location);
-
     return skip;
 }
 
@@ -415,17 +369,18 @@ bool BestPractices::PreCallValidateCmdDrawIndexedIndirectCount(VkCommandBuffer c
                                                                VkBuffer countBuffer, VkDeviceSize countBufferOffset,
                                                                uint32_t maxDrawCount, uint32_t stride,
                                                                const ErrorObject& error_obj) const {
-    return ValidateCmdDrawType(commandBuffer, error_obj.location);
+    const auto cb_state = GetRead<vvl::CommandBuffer>(commandBuffer);
+    const auto& sub_state = bp_state::SubState(*cb_state);
+    return ValidateCmdDrawType(sub_state, error_obj.location);
 }
 
 void BestPractices::PostCallRecordCmdDrawIndexedIndirectCount(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
                                                               VkBuffer countBuffer, VkDeviceSize countBufferOffset,
                                                               uint32_t maxDrawCount, uint32_t stride,
                                                               const RecordObject& record_obj) {
-    BaseClass::PostCallRecordCmdDrawIndexedIndirectCount(commandBuffer, buffer, offset, countBuffer, countBufferOffset,
-                                                            maxDrawCount, stride, record_obj);
-    const auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-    RecordCmdDrawType(*cb_state, 0);
+    const auto cb_state = GetWrite<vvl::CommandBuffer>(commandBuffer);
+    auto& sub_state = bp_state::SubState(*cb_state);
+    RecordCmdDrawType(sub_state, 0);
 }
 
 bool BestPractices::PreCallValidateCmdDrawIndexedIndirectCountAMD(VkCommandBuffer commandBuffer, VkBuffer buffer,
@@ -464,32 +419,34 @@ bool BestPractices::PreCallValidateCmdDrawIndirectByteCountEXT(VkCommandBuffer c
                                                                uint32_t firstInstance, VkBuffer counterBuffer,
                                                                VkDeviceSize counterBufferOffset, uint32_t counterOffset,
                                                                uint32_t vertexStride, const ErrorObject& error_obj) const {
-    return ValidateCmdDrawType(commandBuffer, error_obj.location);
+    const auto cb_state = GetRead<vvl::CommandBuffer>(commandBuffer);
+    const auto& sub_state = bp_state::SubState(*cb_state);
+    return ValidateCmdDrawType(sub_state, error_obj.location);
 }
 
 void BestPractices::PostCallRecordCmdDrawIndirectByteCountEXT(VkCommandBuffer commandBuffer, uint32_t instanceCount,
                                                               uint32_t firstInstance, VkBuffer counterBuffer,
                                                               VkDeviceSize counterBufferOffset, uint32_t counterOffset,
                                                               uint32_t vertexStride, const RecordObject& record_obj) {
-    BaseClass::PostCallRecordCmdDrawIndirectByteCountEXT(commandBuffer, instanceCount, firstInstance, counterBuffer,
-                                                            counterBufferOffset, counterOffset, vertexStride, record_obj);
-    const auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-    RecordCmdDrawType(*cb_state, 0);
+    const auto cb_state = GetWrite<vvl::CommandBuffer>(commandBuffer);
+    auto& sub_state = bp_state::SubState(*cb_state);
+    RecordCmdDrawType(sub_state, 0);
 }
 
 bool BestPractices::PreCallValidateCmdDrawIndirectCount(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
                                                         VkBuffer countBuffer, VkDeviceSize countBufferOffset, uint32_t maxDrawCount,
                                                         uint32_t stride, const ErrorObject& error_obj) const {
-    return ValidateCmdDrawType(commandBuffer, error_obj.location);
+    const auto cb_state = GetRead<vvl::CommandBuffer>(commandBuffer);
+    const auto& sub_state = bp_state::SubState(*cb_state);
+    return ValidateCmdDrawType(sub_state, error_obj.location);
 }
 
 void BestPractices::PostCallRecordCmdDrawIndirectCount(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
                                                        VkBuffer countBuffer, VkDeviceSize countBufferOffset, uint32_t maxDrawCount,
                                                        uint32_t stride, const RecordObject& record_obj) {
-    BaseClass::PostCallRecordCmdDrawIndirectCount(commandBuffer, buffer, offset, countBuffer, countBufferOffset, maxDrawCount,
-                                                     stride, record_obj);
-    const auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-    RecordCmdDrawType(*cb_state, 0);
+    const auto cb_state = GetWrite<vvl::CommandBuffer>(commandBuffer);
+    auto& sub_state = bp_state::SubState(*cb_state);
+    RecordCmdDrawType(sub_state, 0);
 }
 
 bool BestPractices::PreCallValidateCmdDrawIndirectCountAMD(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
@@ -526,90 +483,97 @@ bool BestPractices::PreCallValidateCmdDrawMeshTasksIndirectCountNV(VkCommandBuff
                                                                    VkDeviceSize offset, VkBuffer countBuffer,
                                                                    VkDeviceSize countBufferOffset, uint32_t maxDrawCount,
                                                                    uint32_t stride, const ErrorObject& error_obj) const {
-    return ValidateCmdDrawType(commandBuffer, error_obj.location);
+    const auto cb_state = GetRead<vvl::CommandBuffer>(commandBuffer);
+    const auto& sub_state = bp_state::SubState(*cb_state);
+    return ValidateCmdDrawType(sub_state, error_obj.location);
 }
 
 void BestPractices::PostCallRecordCmdDrawMeshTasksIndirectCountNV(VkCommandBuffer commandBuffer, VkBuffer buffer,
                                                                   VkDeviceSize offset, VkBuffer countBuffer,
                                                                   VkDeviceSize countBufferOffset, uint32_t maxDrawCount,
                                                                   uint32_t stride, const RecordObject& record_obj) {
-    BaseClass::PostCallRecordCmdDrawMeshTasksIndirectCountNV(commandBuffer, buffer, offset, countBuffer, countBufferOffset,
-                                                                maxDrawCount, stride, record_obj);
-    const auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-    RecordCmdDrawType(*cb_state, 0);
+    const auto cb_state = GetWrite<vvl::CommandBuffer>(commandBuffer);
+    auto& sub_state = bp_state::SubState(*cb_state);
+    RecordCmdDrawType(sub_state, 0);
 }
 
 bool BestPractices::PreCallValidateCmdDrawMeshTasksIndirectNV(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
                                                               uint32_t drawCount, uint32_t stride,
                                                               const ErrorObject& error_obj) const {
-    return ValidateCmdDrawType(commandBuffer, error_obj.location);
+    const auto cb_state = GetRead<vvl::CommandBuffer>(commandBuffer);
+    const auto& sub_state = bp_state::SubState(*cb_state);
+    return ValidateCmdDrawType(sub_state, error_obj.location);
 }
 
 void BestPractices::PostCallRecordCmdDrawMeshTasksIndirectNV(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
                                                              uint32_t drawCount, uint32_t stride, const RecordObject& record_obj) {
-    BaseClass::PostCallRecordCmdDrawMeshTasksIndirectNV(commandBuffer, buffer, offset, drawCount, stride, record_obj);
-    const auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-    RecordCmdDrawType(*cb_state, 0);
+    const auto cb_state = GetWrite<vvl::CommandBuffer>(commandBuffer);
+    auto& sub_state = bp_state::SubState(*cb_state);
+    RecordCmdDrawType(sub_state, 0);
 }
 
 bool BestPractices::PreCallValidateCmdDrawMeshTasksNV(VkCommandBuffer commandBuffer, uint32_t taskCount, uint32_t firstTask,
                                                       const ErrorObject& error_obj) const {
-    return ValidateCmdDrawType(commandBuffer, error_obj.location);
+    const auto cb_state = GetRead<vvl::CommandBuffer>(commandBuffer);
+    const auto& sub_state = bp_state::SubState(*cb_state);
+    return ValidateCmdDrawType(sub_state, error_obj.location);
 }
 
 void BestPractices::PostCallRecordCmdDrawMeshTasksNV(VkCommandBuffer commandBuffer, uint32_t taskCount, uint32_t firstTask,
                                                      const RecordObject& record_obj) {
-    BaseClass::PostCallRecordCmdDrawMeshTasksNV(commandBuffer, taskCount, firstTask, record_obj);
-    const auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-    RecordCmdDrawType(*cb_state, 0);
+    const auto cb_state = GetWrite<vvl::CommandBuffer>(commandBuffer);
+    auto& sub_state = bp_state::SubState(*cb_state);
+    RecordCmdDrawType(sub_state, 0);
 }
 
 bool BestPractices::PreCallValidateCmdDrawMultiIndexedEXT(VkCommandBuffer commandBuffer, uint32_t drawCount,
                                                           const VkMultiDrawIndexedInfoEXT* pIndexInfo, uint32_t instanceCount,
                                                           uint32_t firstInstance, uint32_t stride, const int32_t* pVertexOffset,
                                                           const ErrorObject& error_obj) const {
-    return ValidateCmdDrawType(commandBuffer, error_obj.location);
+    const auto cb_state = GetRead<vvl::CommandBuffer>(commandBuffer);
+    const auto& sub_state = bp_state::SubState(*cb_state);
+    return ValidateCmdDrawType(sub_state, error_obj.location);
 }
 
 void BestPractices::PostCallRecordCmdDrawMultiIndexedEXT(VkCommandBuffer commandBuffer, uint32_t drawCount,
                                                          const VkMultiDrawIndexedInfoEXT* pIndexInfo, uint32_t instanceCount,
                                                          uint32_t firstInstance, uint32_t stride, const int32_t* pVertexOffset,
                                                          const RecordObject& record_obj) {
-    BaseClass::PostCallRecordCmdDrawMultiIndexedEXT(commandBuffer, drawCount, pIndexInfo, instanceCount, firstInstance, stride,
-                                                       pVertexOffset, record_obj);
     uint32_t count = 0;
     for (uint32_t i = 0; i < drawCount; ++i) {
         count += pIndexInfo[i].indexCount;
     }
-    const auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-    RecordCmdDrawType(*cb_state, count);
+    const auto cb_state = GetWrite<vvl::CommandBuffer>(commandBuffer);
+    auto& sub_state = bp_state::SubState(*cb_state);
+    RecordCmdDrawType(sub_state, count);
 }
 
 bool BestPractices::PreCallValidateCmdDrawMultiEXT(VkCommandBuffer commandBuffer, uint32_t drawCount,
                                                    const VkMultiDrawInfoEXT* pVertexInfo, uint32_t instanceCount,
                                                    uint32_t firstInstance, uint32_t stride, const ErrorObject& error_obj) const {
-    return ValidateCmdDrawType(commandBuffer, error_obj.location);
+    const auto cb_state = GetRead<vvl::CommandBuffer>(commandBuffer);
+    const auto& sub_state = bp_state::SubState(*cb_state);
+    return ValidateCmdDrawType(sub_state, error_obj.location);
 }
 
 void BestPractices::PostCallRecordCmdDrawMultiEXT(VkCommandBuffer commandBuffer, uint32_t drawCount,
                                                   const VkMultiDrawInfoEXT* pVertexInfo, uint32_t instanceCount,
                                                   uint32_t firstInstance, uint32_t stride, const RecordObject& record_obj) {
-    BaseClass::PostCallRecordCmdDrawMultiEXT(commandBuffer, drawCount, pVertexInfo, instanceCount, firstInstance, stride,
-                                                record_obj);
     uint32_t count = 0;
     for (uint32_t i = 0; i < drawCount; ++i) {
         count += pVertexInfo[i].vertexCount;
     }
-    const auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-    RecordCmdDrawType(*cb_state, count);
+    const auto cb_state = GetWrite<vvl::CommandBuffer>(commandBuffer);
+    auto& sub_state = bp_state::SubState(*cb_state);
+    RecordCmdDrawType(sub_state, count);
 }
 
-void BestPractices::ValidateBoundDescriptorSets(bp_state::CommandBuffer& cb_state, VkPipelineBindPoint bind_point, Func command) {
-    auto lvl_bind_point = ConvertToLvlBindPoint(bind_point);
-    auto& last_bound = cb_state.lastBound[lvl_bind_point];
-
-    for (const auto& ds_slot : last_bound.ds_slots) {
-        if (!ds_slot.ds_state) continue;
+void BestPractices::UpdateBoundDescriptorSets(bp_state::CommandBufferSubState& cb_state, const LastBound& last_bound_state,
+                                              const Location& loc) {
+    for (const auto& ds_slot : last_bound_state.ds_slots) {
+        if (!ds_slot.ds_state) {
+            continue;
+        }
         for (const auto& binding : *ds_slot.ds_state) {
             // For bindless scenarios, we should not attempt to track descriptor set state.
             // It is highly uncertain which resources are actually bound.
@@ -644,9 +608,8 @@ void BestPractices::ValidateBoundDescriptorSets(bp_state::CommandBuffer& cb_stat
                         break;
                 }
 
-                if (image_view) {
-                    auto image_view_state = Get<vvl::ImageView>(image_view);
-                    QueueValidateImageView(cb_state.queue_submit_functions, command, image_view_state.get(),
+                if (auto image_view_state = Get<vvl::ImageView>(image_view)) {
+                    QueueValidateImageView(cb_state.queue_submit_functions, loc, *image_view_state,
                                            IMAGE_SUBRESOURCE_USAGE_BP::DESCRIPTOR_ACCESS);
                 }
             }
@@ -656,31 +619,31 @@ void BestPractices::ValidateBoundDescriptorSets(bp_state::CommandBuffer& cb_stat
 
 void BestPractices::PostCallRecordCmdDraw(VkCommandBuffer commandBuffer, uint32_t vertexCount, uint32_t instanceCount,
                                           uint32_t firstVertex, uint32_t firstInstance, const RecordObject& record_obj) {
-    BaseClass::PostCallRecordCmdDraw(commandBuffer, vertexCount, instanceCount, firstVertex, firstInstance, record_obj);
-    const auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-    ValidateBoundDescriptorSets(*cb_state, VK_PIPELINE_BIND_POINT_GRAPHICS, record_obj.location.function);
-    RecordCmdDrawType(*cb_state, vertexCount * instanceCount);
+    const auto cb_state = GetWrite<vvl::CommandBuffer>(commandBuffer);
+    auto& sub_state = bp_state::SubState(*cb_state);
+    RecordCmdDrawType(sub_state, vertexCount * instanceCount);
 }
 
 void BestPractices::PostCallRecordCmdDrawIndirect(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
                                                   uint32_t drawCount, uint32_t stride, const RecordObject& record_obj) {
-    BaseClass::PostCallRecordCmdDrawIndirect(commandBuffer, buffer, offset, drawCount, stride, record_obj);
-    const auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-    ValidateBoundDescriptorSets(*cb_state, VK_PIPELINE_BIND_POINT_GRAPHICS, record_obj.location.function);
-    RecordCmdDrawType(*cb_state, drawCount);
+    const auto cb_state = GetWrite<vvl::CommandBuffer>(commandBuffer);
+    auto& sub_state = bp_state::SubState(*cb_state);
+    RecordCmdDrawType(sub_state, drawCount);
 }
 
 void BestPractices::PostCallRecordCmdDrawIndexedIndirect(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
                                                          uint32_t drawCount, uint32_t stride, const RecordObject& record_obj) {
-    BaseClass::PostCallRecordCmdDrawIndexedIndirect(commandBuffer, buffer, offset, drawCount, stride, record_obj);
-    const auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-    ValidateBoundDescriptorSets(*cb_state, VK_PIPELINE_BIND_POINT_GRAPHICS, record_obj.location.function);
-    RecordCmdDrawType(*cb_state, drawCount);
+    const auto cb_state = GetWrite<vvl::CommandBuffer>(commandBuffer);
+    auto& sub_state = bp_state::SubState(*cb_state);
+    RecordCmdDrawType(sub_state, drawCount);
 }
 
 bool BestPractices::PreCallValidateCmdDispatch(VkCommandBuffer commandBuffer, uint32_t groupCountX, uint32_t groupCountY,
                                                uint32_t groupCountZ, const ErrorObject& error_obj) const {
     bool skip = false;
+    const auto cb_state = GetRead<vvl::CommandBuffer>(commandBuffer);
+    auto& sub_state = bp_state::SubState(*cb_state);
+    skip |= ValidateCmdDispatchType(sub_state, error_obj.location);
 
     if ((groupCountX == 0) || (groupCountY == 0) || (groupCountZ == 0)) {
         skip |= LogWarning("BestPractices-vkCmdDispatch-group-count-zero", device, error_obj.location,
@@ -688,7 +651,6 @@ bool BestPractices::PreCallValidateCmdDispatch(VkCommandBuffer commandBuffer, ui
                            ", groupCountZ = %" PRIu32 ").",
                            groupCountX, groupCountY, groupCountZ);
     }
-    skip |= ValidateCmdDispatchType(commandBuffer, error_obj.location);
 
     return skip;
 }
@@ -696,9 +658,9 @@ bool BestPractices::PreCallValidateCmdDispatch(VkCommandBuffer commandBuffer, ui
 bool BestPractices::PreCallValidateCmdDispatchIndirect(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
                                                        const ErrorObject& error_obj) const {
     bool skip = false;
-
-    skip |= ValidateCmdDispatchType(commandBuffer, error_obj.location);
-
+    const auto cb_state = GetRead<vvl::CommandBuffer>(commandBuffer);
+    auto& sub_state = bp_state::SubState(*cb_state);
+    skip |= ValidateCmdDispatchType(sub_state, error_obj.location);
     return skip;
 }
 
@@ -706,6 +668,9 @@ bool BestPractices::PreCallValidateCmdDispatchBase(VkCommandBuffer commandBuffer
                                                    uint32_t baseGroupZ, uint32_t groupCountX, uint32_t groupCountY,
                                                    uint32_t groupCountZ, const ErrorObject& error_obj) const {
     bool skip = false;
+    const auto cb_state = GetRead<vvl::CommandBuffer>(commandBuffer);
+    auto& sub_state = bp_state::SubState(*cb_state);
+    skip |= ValidateCmdDispatchType(sub_state, error_obj.location);
 
     if ((groupCountX == 0) || (groupCountY == 0) || (groupCountZ == 0)) {
         skip |= LogWarning("BestPractices-vkCmdDispatchBase-group-count-zero", device, error_obj.location,
@@ -713,8 +678,6 @@ bool BestPractices::PreCallValidateCmdDispatchBase(VkCommandBuffer commandBuffer
                            ", groupCountZ = %" PRIu32 ").",
                            groupCountX, groupCountY, groupCountZ);
     }
-    skip |= ValidateCmdDispatchType(commandBuffer, error_obj.location);
-
     return skip;
 }
 
@@ -724,28 +687,3 @@ bool BestPractices::PreCallValidateCmdDispatchBaseKHR(VkCommandBuffer commandBuf
     return PreCallValidateCmdDispatchBase(commandBuffer, baseGroupX, baseGroupY, baseGroupZ, groupCountX, groupCountY, groupCountZ,
                                           error_obj);
 }
-
-void BestPractices::PostCallRecordCmdDispatch(VkCommandBuffer commandBuffer, uint32_t x, uint32_t y, uint32_t z,
-                                              const RecordObject& record_obj) {
-    const auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-    ValidateBoundDescriptorSets(*cb_state, VK_PIPELINE_BIND_POINT_COMPUTE, record_obj.location.function);
-}
-
-void BestPractices::PostCallRecordCmdDispatchIndirect(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
-                                                      const RecordObject& record_obj) {
-    const auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-    ValidateBoundDescriptorSets(*cb_state, VK_PIPELINE_BIND_POINT_COMPUTE, record_obj.location.function);
-}
-
-bool BestPractices::PreCallValidateEndCommandBuffer(VkCommandBuffer commandBuffer, const ErrorObject& error_obj) const {
-    bool skip = false;
-    const auto cb_state = GetRead<bp_state::CommandBuffer>(commandBuffer);
-
-    if (!cb_state->current_vertex_buffer_binding_info.empty() && !cb_state->uses_vertex_buffer) {
-        skip |=
-            LogPerformanceWarning("BestPractices-vkEndCommandBuffer-VtxIndexOutOfBounds", cb_state->Handle(), error_obj.location,
-                                  "Vertex buffers was bound to %s but no draws had a pipeline that used the vertex buffer.",
-                                  FormatHandle(cb_state->Handle()).c_str());
-    }
-    return skip;
-}
