@@ -20,17 +20,21 @@
  ****************************************************************************/
 #include "chassis.h"
 
-#include <array>
 #include <cstring>
-#include <mutex>
 
 #include "chassis/dispatch_object.h"
+#include "generated/dispatch_vector.h"
 #include "chassis/validation_object.h"
 #include "layer_options.h"
 #include "state_tracker/descriptor_sets.h"
 #include "chassis/chassis_modification_state.h"
 #include "core_checks/core_validation.h"
 #include "profiling/profiling.h"
+#include "containers/small_vector.h"
+#include "utils/dispatch_utils.h"
+
+#define STRINGIFY(s) STRINGIFY_HELPER(s)
+#define STRINGIFY_HELPER(s) #s
 
 namespace vulkan_layer_chassis {
 
@@ -264,14 +268,21 @@ VKAPI_ATTR void VKAPI_CALL DestroyInstance(VkInstance instance, const VkAllocati
         }
         vo->PreCallValidateDestroyInstance(instance, pAllocator, error_obj);
     }
-
+    vvl::base::Instance* state_tracker = nullptr;
     RecordObject record_obj(vvl::Func::vkDestroyInstance);
     for (auto& vo : instance_dispatch->object_dispatch) {
         if (!vo) {
             continue;
         }
+        if (vo->container_type == LayerObjectTypeStateTracker) {
+            state_tracker = vo.get();
+            continue;
+        }
         vo->PreCallRecordDestroyInstance(instance, pAllocator, record_obj);
     }
+    if (state_tracker) {
+        state_tracker->PreCallRecordDestroyInstance(instance, pAllocator, record_obj);
+    }
 
     VVL_TracyCZoneEnd(tracy_zone_precall);
     VVL_TracyCZone(tracy_zone_dispatch, true);
@@ -283,8 +294,14 @@ VKAPI_ATTR void VKAPI_CALL DestroyInstance(VkInstance instance, const VkAllocati
         if (!vo) {
             continue;
         }
+        if (vo->container_type == LayerObjectTypeStateTracker) {
+            continue;
+        }
         vo->PostCallRecordDestroyInstance(instance, pAllocator, record_obj);
     }
+    if (state_tracker) {
+        state_tracker->PostCallRecordDestroyInstance(instance, pAllocator, record_obj);
+    }
 
     DeactivateInstanceDebugCallbacks(instance_dispatch->debug_report);
     vvl::dispatch::FreeData(key, instance);
@@ -345,8 +362,6 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateDevice(VkPhysicalDevice gpu, const VkDevice
         }
         vo->PreCallRecordCreateDevice(gpu, pCreateInfo, pAllocator, pDevice, record_obj, &modified_create_info);
     }
-    // Recalculate enabled_features based on any changes made
-    GetEnabledDeviceFeatures(modified_create_info.ptr(), &device_dispatch->enabled_features, device_dispatch->api_version);
 
     VkResult result = fpCreateDevice(gpu, reinterpret_cast<VkDeviceCreateInfo*>(&modified_create_info), pAllocator, pDevice);
     if (result != VK_SUCCESS) {
@@ -354,9 +369,7 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateDevice(VkPhysicalDevice gpu, const VkDevice
     }
     record_obj.result = result;
     device_dispatch->device = *pDevice;
-    // Save local info in device object
-    device_dispatch->extensions = DeviceExtensions(instance_dispatch->extensions, device_dispatch->api_version,
-                                                   reinterpret_cast<VkDeviceCreateInfo*>(&modified_create_info));
+
     layer_init_device_dispatch_table(*pDevice, &device_dispatch->device_dispatch_table, fpGetDeviceProcAddr);
 
     instance_dispatch->debug_report->device_created++;
@@ -407,15 +420,19 @@ VKAPI_ATTR void VKAPI_CALL DestroyDevice(VkDevice device, const VkAllocationCall
         }
         vo->PreCallValidateDestroyDevice(device, pAllocator, error_obj);
     }
-
     RecordObject record_obj(vvl::Func::vkDestroyDevice);
+
+    // Even though layer object types reference the base device state tracker,
+    // it needs to be destroyed first: it stores the various object maps,
+    // those need to be destroyed by the time layer objects destroy their
+    // own device state.
     for (auto& vo : device_dispatch->object_dispatch) {
         if (!vo) {
             continue;
         }
+
         vo->PreCallRecordDestroyDevice(device, pAllocator, record_obj);
     }
-
     // Before device is destroyed, allow aborted objects to clean up
     for (auto& vo : device_dispatch->aborted_object_dispatch) {
         if (!vo) {
@@ -430,12 +447,31 @@ VKAPI_ATTR void VKAPI_CALL DestroyDevice(VkDevice device, const VkAllocationCall
 
     device_dispatch->DestroyDevice(device, pAllocator);
 
+    // The teardown order is
+    // 1. All active validation objects
+    // 2. All aborted validation objects (ex. if GPU-AV had to turn off mid run)
+    // 3. State Tracker validation object
+    // 4. Cleanup leaked handles that user didn't destroy
+    vvl::base::Device* state_tracker = nullptr;
     for (auto& vo : device_dispatch->object_dispatch) {
         if (!vo) {
             continue;
         }
+        if (vo->container_type == LayerObjectTypeStateTracker) {
+            state_tracker = vo.get();
+            continue;
+        }
         vo->PostCallRecordDestroyDevice(device, pAllocator, record_obj);
     }
+    for (auto& vo : device_dispatch->aborted_object_dispatch) {
+        if (!vo) {
+            continue;
+        }
+        vo->PostCallRecordDestroyDevice(device, pAllocator, record_obj);
+    }
+    if (state_tracker) {
+        state_tracker->PostCallRecordDestroyDevice(device, pAllocator, record_obj);
+    }
 
     auto instance_dispatch = vvl::dispatch::GetData(device_dispatch->physical_device);
     instance_dispatch->debug_report->device_created--;
@@ -454,53 +490,59 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateGraphicsPipelines(VkDevice device, VkPipeli
     bool skip = false;
     ErrorObject error_obj(vvl::Func::vkCreateGraphicsPipelines, VulkanTypedHandle(device, kVulkanObjectTypeDevice));
 
-    PipelineStates pipeline_states[LayerObjectTypeMaxEnum];
+    PipelineStates pipeline_states;
     chassis::CreateGraphicsPipelines chassis_state(pCreateInfos);
 
     {
-        VVL_ZoneScopedN("PreCallValidate_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallValidate_CreateGraphicsPipelines");
         for (const auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
             }
             auto lock = vo->ReadLock();
             skip |= vo->PreCallValidateCreateGraphicsPipelines(device, pipelineCache, createInfoCount, pCreateInfos, pAllocator,
-                                                               pPipelines, error_obj, pipeline_states[vo->container_type],
-                                                               chassis_state);
+                                                               pPipelines, error_obj, pipeline_states, chassis_state);
             if (skip) return VK_ERROR_VALIDATION_FAILED_EXT;
         }
     }
 
     RecordObject record_obj(vvl::Func::vkCreateGraphicsPipelines);
     {
-        VVL_ZoneScopedN("PreCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallRecord_CreateGraphicsPipelines");
         for (auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
             }
             auto lock = vo->WriteLock();
             vo->PreCallRecordCreateGraphicsPipelines(device, pipelineCache, createInfoCount, pCreateInfos, pAllocator, pPipelines,
-                                                     record_obj, pipeline_states[vo->container_type], chassis_state);
+                                                     record_obj, pipeline_states, chassis_state);
         }
     }
 
     VkResult result;
     {
-        VVL_ZoneScopedN("Dispatch_" __FUNCTION__);
+        VVL_ZoneScopedN("Dispatch_CreateGraphicsPipelines");
         result = device_dispatch->CreateGraphicsPipelines(device, pipelineCache, createInfoCount, chassis_state.pCreateInfos,
                                                           pAllocator, pPipelines);
+
+        // If we have modified the pCreateInfos caused things to fail, revert to allow the app to continue
+        if (result < VK_SUCCESS && chassis_state.is_modified) {
+            chassis_state.is_modified = false;
+            result = device_dispatch->CreateGraphicsPipelines(device, pipelineCache, createInfoCount, pCreateInfos, pAllocator,
+                                                              pPipelines);
+        }
     }
     record_obj.result = result;
 
     {
-        VVL_ZoneScopedN("PostCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PostCallRecord_CreateGraphicsPipelines");
         for (auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
             }
             auto lock = vo->WriteLock();
             vo->PostCallRecordCreateGraphicsPipelines(device, pipelineCache, createInfoCount, pCreateInfos, pAllocator, pPipelines,
-                                                      record_obj, pipeline_states[vo->container_type], chassis_state);
+                                                      record_obj, pipeline_states, chassis_state);
         }
     }
     return result;
@@ -516,53 +558,58 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateComputePipelines(VkDevice device, VkPipelin
     bool skip = false;
     ErrorObject error_obj(vvl::Func::vkCreateComputePipelines, VulkanTypedHandle(device, kVulkanObjectTypeDevice));
 
-    PipelineStates pipeline_states[LayerObjectTypeMaxEnum];
+    PipelineStates pipeline_states;
     chassis::CreateComputePipelines chassis_state(pCreateInfos);
 
     {
-        VVL_ZoneScopedN("PreCallValidate_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallValidate_CreateComputePipelines");
         for (const auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
             }
             auto lock = vo->ReadLock();
             skip |= vo->PreCallValidateCreateComputePipelines(device, pipelineCache, createInfoCount, pCreateInfos, pAllocator,
-                                                              pPipelines, error_obj, pipeline_states[vo->container_type],
-                                                              chassis_state);
+                                                              pPipelines, error_obj, pipeline_states, chassis_state);
             if (skip) return VK_ERROR_VALIDATION_FAILED_EXT;
         }
     }
 
     RecordObject record_obj(vvl::Func::vkCreateComputePipelines);
     {
-        VVL_ZoneScopedN("PreCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallRecord_CreateComputePipelines");
         for (auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
             }
             auto lock = vo->WriteLock();
             vo->PreCallRecordCreateComputePipelines(device, pipelineCache, createInfoCount, pCreateInfos, pAllocator, pPipelines,
-                                                    record_obj, pipeline_states[vo->container_type], chassis_state);
+                                                    record_obj, pipeline_states, chassis_state);
         }
     }
 
     VkResult result;
     {
-        VVL_ZoneScopedN("Dispatch_" __FUNCTION__);
+        VVL_ZoneScopedN("Dispatch_CreateComputePipelines");
         result = device_dispatch->CreateComputePipelines(device, pipelineCache, createInfoCount, chassis_state.pCreateInfos,
                                                          pAllocator, pPipelines);
+        // If we have modified the pCreateInfos caused things to fail, revert to allow the app to continue
+        if (result < VK_SUCCESS && chassis_state.is_modified) {
+            chassis_state.is_modified = false;
+            result = device_dispatch->CreateComputePipelines(device, pipelineCache, createInfoCount, pCreateInfos, pAllocator,
+                                                             pPipelines);
+        }
     }
     record_obj.result = result;
 
     {
-        VVL_ZoneScopedN("PostCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PostCallRecord_CreateComputePipelines");
         for (auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
             }
             auto lock = vo->WriteLock();
             vo->PostCallRecordCreateComputePipelines(device, pipelineCache, createInfoCount, pCreateInfos, pAllocator, pPipelines,
-                                                     record_obj, pipeline_states[vo->container_type], chassis_state);
+                                                     record_obj, pipeline_states, chassis_state);
         }
     }
     return result;
@@ -575,8 +622,7 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateRayTracingPipelinesNV(VkDevice device, VkPi
     bool skip = false;
     ErrorObject error_obj(vvl::Func::vkCreateRayTracingPipelinesNV, VulkanTypedHandle(device, kVulkanObjectTypeDevice));
 
-    PipelineStates pipeline_states[LayerObjectTypeMaxEnum];
-    chassis::CreateRayTracingPipelinesNV chassis_state(pCreateInfos);
+    PipelineStates pipeline_states;
 
     for (const auto& vo : device_dispatch->object_dispatch) {
         if (!vo) {
@@ -584,8 +630,7 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateRayTracingPipelinesNV(VkDevice device, VkPi
         }
         auto lock = vo->ReadLock();
         skip |= vo->PreCallValidateCreateRayTracingPipelinesNV(device, pipelineCache, createInfoCount, pCreateInfos, pAllocator,
-                                                               pPipelines, error_obj, pipeline_states[vo->container_type],
-                                                               chassis_state);
+                                                               pPipelines, error_obj, pipeline_states);
         if (skip) return VK_ERROR_VALIDATION_FAILED_EXT;
     }
 
@@ -596,11 +641,12 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateRayTracingPipelinesNV(VkDevice device, VkPi
         }
         auto lock = vo->WriteLock();
         vo->PreCallRecordCreateRayTracingPipelinesNV(device, pipelineCache, createInfoCount, pCreateInfos, pAllocator, pPipelines,
-                                                     record_obj, pipeline_states[vo->container_type], chassis_state);
+                                                     record_obj, pipeline_states);
     }
 
-    VkResult result = device_dispatch->CreateRayTracingPipelinesNV(device, pipelineCache, createInfoCount,
-                                                                   chassis_state.pCreateInfos, pAllocator, pPipelines);
+    VkResult result;
+    result =
+        device_dispatch->CreateRayTracingPipelinesNV(device, pipelineCache, createInfoCount, pCreateInfos, pAllocator, pPipelines);
     record_obj.result = result;
 
     for (auto& vo : device_dispatch->object_dispatch) {
@@ -609,7 +655,7 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateRayTracingPipelinesNV(VkDevice device, VkPi
         }
         auto lock = vo->WriteLock();
         vo->PostCallRecordCreateRayTracingPipelinesNV(device, pipelineCache, createInfoCount, pCreateInfos, pAllocator, pPipelines,
-                                                      record_obj, pipeline_states[vo->container_type], chassis_state);
+                                                      record_obj, pipeline_states);
     }
     return result;
 }
@@ -624,11 +670,11 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateRayTracingPipelinesKHR(VkDevice device, VkD
     bool skip = false;
     ErrorObject error_obj(vvl::Func::vkCreateRayTracingPipelinesKHR, VulkanTypedHandle(device, kVulkanObjectTypeDevice));
 
-    PipelineStates pipeline_states[LayerObjectTypeMaxEnum];
+    PipelineStates pipeline_states;
     auto chassis_state = std::make_shared<chassis::CreateRayTracingPipelinesKHR>(pCreateInfos);
 
     {
-        VVL_ZoneScopedN("PreCallValidate_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallValidate_CreateRayTracingPipelinesKHR");
         for (const auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
@@ -636,43 +682,49 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateRayTracingPipelinesKHR(VkDevice device, VkD
             auto lock = vo->ReadLock();
             skip |= vo->PreCallValidateCreateRayTracingPipelinesKHR(device, deferredOperation, pipelineCache, createInfoCount,
                                                                     pCreateInfos, pAllocator, pPipelines, error_obj,
-                                                                    pipeline_states[vo->container_type], *chassis_state);
+                                                                    pipeline_states, *chassis_state);
             if (skip) return VK_ERROR_VALIDATION_FAILED_EXT;
         }
     }
 
     RecordObject record_obj(vvl::Func::vkCreateRayTracingPipelinesKHR);
     {
-        VVL_ZoneScopedN("PreCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallRecord_CreateRayTracingPipelinesKHR");
         for (auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
             }
             auto lock = vo->WriteLock();
             vo->PreCallRecordCreateRayTracingPipelinesKHR(device, deferredOperation, pipelineCache, createInfoCount, pCreateInfos,
-                                                          pAllocator, pPipelines, record_obj, pipeline_states[vo->container_type],
-                                                          *chassis_state);
+                                                          pAllocator, pPipelines, record_obj, pipeline_states, *chassis_state);
         }
     }
 
     VkResult result;
     {
-        VVL_ZoneScopedN("Dispatch_" __FUNCTION__);
+        VVL_ZoneScopedN("Dispatch_CreateRayTracingPipelinesKHR");
         result = device_dispatch->CreateRayTracingPipelinesKHR(device, deferredOperation, pipelineCache, createInfoCount,
                                                                chassis_state->pCreateInfos, pAllocator, pPipelines);
+
+        // If we have modified the pCreateInfos caused things to fail, revert to allow the app to continue
+        // Note VK_OPERATION_DEFERRED_KHR is not "failing" as the pipeline will still be created
+        if (result < VK_SUCCESS && chassis_state->is_modified) {
+            chassis_state->is_modified = false;
+            result = device_dispatch->CreateRayTracingPipelinesKHR(device, deferredOperation, pipelineCache, createInfoCount,
+                                                                   pCreateInfos, pAllocator, pPipelines);
+        }
     }
     record_obj.result = result;
 
     {
-        VVL_ZoneScopedN("PostCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PostCallRecord_CreateRayTracingPipelinesKHR");
         for (auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
             }
             auto lock = vo->WriteLock();
             vo->PostCallRecordCreateRayTracingPipelinesKHR(device, deferredOperation, pipelineCache, createInfoCount, pCreateInfos,
-                                                           pAllocator, pPipelines, record_obj, pipeline_states[vo->container_type],
-                                                           chassis_state);
+                                                           pAllocator, pPipelines, record_obj, pipeline_states, chassis_state);
         }
     }
     return result;
@@ -688,7 +740,7 @@ VKAPI_ATTR VkResult VKAPI_CALL CreatePipelineLayout(VkDevice device, const VkPip
     ErrorObject error_obj(vvl::Func::vkCreatePipelineLayout, VulkanTypedHandle(device, kVulkanObjectTypeDevice));
 
     {
-        VVL_ZoneScopedN("PreCallValidate_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallValidate_CreatePipelineLayout");
         for (const auto& vo : device_dispatch->intercept_vectors[InterceptIdPreCallValidateCreatePipelineLayout]) {
             if (!vo) {
                 continue;
@@ -704,7 +756,7 @@ VKAPI_ATTR VkResult VKAPI_CALL CreatePipelineLayout(VkDevice device, const VkPip
 
     RecordObject record_obj(vvl::Func::vkCreatePipelineLayout);
     {
-        VVL_ZoneScopedN("PreCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallRecord_CreatePipelineLayout");
         for (auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
@@ -716,13 +768,13 @@ VKAPI_ATTR VkResult VKAPI_CALL CreatePipelineLayout(VkDevice device, const VkPip
 
     VkResult result;
     {
-        VVL_ZoneScopedN("Dispatch_" __FUNCTION__);
+        VVL_ZoneScopedN("Dispatch_CreatePipelineLayout");
         result = device_dispatch->CreatePipelineLayout(device, &chassis_state.modified_create_info, pAllocator, pPipelineLayout);
     }
     record_obj.result = result;
 
     {
-        VVL_ZoneScopedN("PostCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PostCallRecord_CreatePipelineLayout");
         for (auto& vo : device_dispatch->intercept_vectors[InterceptIdPostCallRecordCreatePipelineLayout]) {
             if (!vo) {
                 continue;
@@ -734,6 +786,56 @@ VKAPI_ATTR VkResult VKAPI_CALL CreatePipelineLayout(VkDevice device, const VkPip
     return result;
 }
 
+VKAPI_ATTR VkResult VKAPI_CALL GetShaderBinaryDataEXT(VkDevice device, VkShaderEXT shader, size_t* pDataSize, void* pData) {
+    VVL_ZoneScoped;
+
+    auto device_dispatch = vvl::dispatch::GetData(device);
+    bool skip = false;
+    ErrorObject error_obj(vvl::Func::vkGetShaderBinaryDataEXT, VulkanTypedHandle(device, kVulkanObjectTypeDevice));
+    {
+        VVL_ZoneScopedN("PreCallValidate_vkGetShaderBinaryDataEXT");
+        for (const auto& vo : device_dispatch->intercept_vectors[InterceptIdPreCallValidateGetShaderBinaryDataEXT]) {
+            if (!vo) {
+                continue;
+            }
+            auto lock = vo->ReadLock();
+            skip |= vo->PreCallValidateGetShaderBinaryDataEXT(device, shader, pDataSize, pData, error_obj);
+            if (skip) return VK_ERROR_VALIDATION_FAILED_EXT;
+        }
+    }
+
+    chassis::ShaderBinaryData chassis_state{shader};
+
+    RecordObject record_obj(vvl::Func::vkGetShaderBinaryDataEXT);
+    {
+        VVL_ZoneScopedN("PreCallRecord_vkGetShaderBinaryDataEXT");
+        for (auto& vo : device_dispatch->object_dispatch) {
+            if (!vo) {
+                continue;
+            }
+            auto lock = vo->WriteLock();
+            vo->PreCallRecordGetShaderBinaryDataEXT(device, shader, pDataSize, pData, record_obj, chassis_state);
+        }
+    }
+    VkResult result;
+    {
+        VVL_ZoneScopedN("Dispatch_vkGetShaderBinaryDataEXT");
+        result = device_dispatch->GetShaderBinaryDataEXT(device, chassis_state.modified_shader_handle, pDataSize, pData);
+    }
+    record_obj.result = result;
+    {
+        VVL_ZoneScopedN("PostCallRecord_vkGetShaderBinaryDataEXT");
+        for (auto& vo : device_dispatch->intercept_vectors[InterceptIdPostCallRecordGetShaderBinaryDataEXT]) {
+            if (!vo) {
+                continue;
+            }
+            auto lock = vo->WriteLock();
+            vo->PostCallRecordGetShaderBinaryDataEXT(device, shader, pDataSize, pData, record_obj);
+        }
+    }
+    return result;
+}
+
 // This API needs some local stack data for performance reasons and also may modify a parameter
 VKAPI_ATTR VkResult VKAPI_CALL CreateShaderModule(VkDevice device, const VkShaderModuleCreateInfo* pCreateInfo,
                                                   const VkAllocationCallbacks* pAllocator, VkShaderModule* pShaderModule) {
@@ -744,7 +846,7 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateShaderModule(VkDevice device, const VkShade
     ErrorObject error_obj(vvl::Func::vkCreateShaderModule, VulkanTypedHandle(device, kVulkanObjectTypeDevice));
 
     {
-        VVL_ZoneScopedN("PreCallValidate_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallValidate_CreateShaderModule");
         for (const auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
@@ -759,7 +861,7 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateShaderModule(VkDevice device, const VkShade
 
     RecordObject record_obj(vvl::Func::vkCreateShaderModule);
     {
-        VVL_ZoneScopedN("PreCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallRecord_CreateShaderModule");
         for (auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
@@ -773,12 +875,12 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateShaderModule(VkDevice device, const VkShade
     if (chassis_state.skip) return VK_ERROR_VALIDATION_FAILED_EXT;
     VkResult result;
     {
-        VVL_ZoneScopedN("Dispatch_" __FUNCTION__);
+        VVL_ZoneScopedN("Dispatch_CreateShaderModule");
         result = device_dispatch->CreateShaderModule(device, pCreateInfo, pAllocator, pShaderModule);
     }
     record_obj.result = result;
     {
-        VVL_ZoneScopedN("PostCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PostCallRecord_CreateShaderModule");
         for (auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
@@ -802,7 +904,7 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateShadersEXT(VkDevice device, uint32_t create
     chassis::ShaderObject chassis_state(createInfoCount, pCreateInfos);
 
     {
-        VVL_ZoneScopedN("PreCallValidate_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallValidate_CreateShadersEXT");
         for (const auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
@@ -815,7 +917,7 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateShadersEXT(VkDevice device, uint32_t create
 
     RecordObject record_obj(vvl::Func::vkCreateShadersEXT);
     {
-        VVL_ZoneScopedN("PreCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallRecord_CreateShadersEXT");
         for (auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
@@ -831,13 +933,19 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateShadersEXT(VkDevice device, uint32_t create
 
     VkResult result;
     {
-        VVL_ZoneScopedN("Dispatch_" __FUNCTION__);
+        VVL_ZoneScopedN("Dispatch_CreateShadersEXT");
         result = device_dispatch->CreateShadersEXT(device, createInfoCount, chassis_state.pCreateInfos, pAllocator, pShaders);
+
+        // If we have modified the pCreateInfos caused things to fail, revert to allow the app to continue
+        if (result < VK_SUCCESS && chassis_state.is_modified) {
+            chassis_state.is_modified = false;
+            result = device_dispatch->CreateShadersEXT(device, createInfoCount, pCreateInfos, pAllocator, pShaders);
+        }
     }
     record_obj.result = result;
 
     {
-        VVL_ZoneScopedN("PostCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PostCallRecord_CreateShadersEXT");
         for (auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
@@ -858,25 +966,26 @@ VKAPI_ATTR VkResult VKAPI_CALL AllocateDescriptorSets(VkDevice device, const VkD
     bool skip = false;
     ErrorObject error_obj(vvl::Func::vkAllocateDescriptorSets, VulkanTypedHandle(device, kVulkanObjectTypeDevice));
 
-    vvl::AllocateDescriptorSetsData ads_state[LayerObjectTypeMaxEnum];
+    // Because this is a high frequency function call and want to save time and populate this struct once.
+    // Both CoreCheck and Best Practice need this information during PreCallValidate time.
+    // During State Tracker PreCallValidate (instead of PreCallRecord), we populate this so that everyone else can use it.
+    vvl::AllocateDescriptorSetsData ads_state;
 
     {
-        VVL_ZoneScopedN("PreCallValidate_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallValidate_AllocateDescriptorSets");
         for (const auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
             }
-            ads_state[vo->container_type].Init(pAllocateInfo->descriptorSetCount);
             auto lock = vo->ReadLock();
-            skip |= vo->PreCallValidateAllocateDescriptorSets(device, pAllocateInfo, pDescriptorSets, error_obj,
-                                                              ads_state[vo->container_type]);
+            skip |= vo->PreCallValidateAllocateDescriptorSets(device, pAllocateInfo, pDescriptorSets, error_obj, ads_state);
             if (skip) return VK_ERROR_VALIDATION_FAILED_EXT;
         }
     }
 
     RecordObject record_obj(vvl::Func::vkAllocateDescriptorSets);
     {
-        VVL_ZoneScopedN("PreCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallRecord_AllocateDescriptorSets");
         for (auto& vo : device_dispatch->intercept_vectors[InterceptIdPreCallRecordAllocateDescriptorSets]) {
             if (!vo) {
                 continue;
@@ -888,20 +997,19 @@ VKAPI_ATTR VkResult VKAPI_CALL AllocateDescriptorSets(VkDevice device, const VkD
 
     VkResult result;
     {
-        VVL_ZoneScopedN("Dispatch_" __FUNCTION__);
+        VVL_ZoneScopedN("Dispatch_AllocateDescriptorSets");
         result = device_dispatch->AllocateDescriptorSets(device, pAllocateInfo, pDescriptorSets);
     }
     record_obj.result = result;
 
     {
-        VVL_ZoneScopedN("PostCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PostCallRecord_AllocateDescriptorSets");
         for (auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
             }
             auto lock = vo->WriteLock();
-            vo->PostCallRecordAllocateDescriptorSets(device, pAllocateInfo, pDescriptorSets, record_obj,
-                                                     ads_state[vo->container_type]);
+            vo->PostCallRecordAllocateDescriptorSets(device, pAllocateInfo, pDescriptorSets, record_obj, ads_state);
         }
     }
     return result;
@@ -917,7 +1025,7 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateBuffer(VkDevice device, const VkBufferCreat
     ErrorObject error_obj(vvl::Func::vkCreateBuffer, VulkanTypedHandle(device, kVulkanObjectTypeDevice));
 
     {
-        VVL_ZoneScopedN("PreCallValidate_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallValidate_CreateBuffer");
         for (const auto& vo : device_dispatch->intercept_vectors[InterceptIdPreCallValidateCreateBuffer]) {
             if (!vo) {
                 continue;
@@ -933,7 +1041,7 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateBuffer(VkDevice device, const VkBufferCreat
 
     RecordObject record_obj(vvl::Func::vkCreateBuffer);
     {
-        VVL_ZoneScopedN("PreCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallRecord_CreateBuffer");
         for (auto& vo : device_dispatch->object_dispatch) {
             if (!vo) {
                 continue;
@@ -945,13 +1053,13 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateBuffer(VkDevice device, const VkBufferCreat
 
     VkResult result;
     {
-        VVL_ZoneScopedN("Dispatch_" __FUNCTION__);
+        VVL_ZoneScopedN("Dispatch_CreateBuffer");
         result = device_dispatch->CreateBuffer(device, &chassis_state.modified_create_info, pAllocator, pBuffer);
     }
     record_obj.result = result;
 
     {
-        VVL_ZoneScopedN("PostCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PostCallRecord_CreateBuffer");
         for (auto& vo : device_dispatch->intercept_vectors[InterceptIdPostCallRecordCreateBuffer]) {
             if (!vo) {
                 continue;
@@ -971,7 +1079,7 @@ VKAPI_ATTR VkResult VKAPI_CALL QueuePresentKHR(VkQueue queue, const VkPresentInf
     bool skip = false;
     ErrorObject error_obj(vvl::Func::vkQueuePresentKHR, VulkanTypedHandle(queue, kVulkanObjectTypeQueue));
     {
-        VVL_ZoneScopedN("PreCallValidate_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallValidate_QueuePresentKHR");
         for (const auto& vo : device_dispatch->intercept_vectors[InterceptIdPreCallValidateQueuePresentKHR]) {
             if (!vo) {
                 continue;
@@ -983,7 +1091,7 @@ VKAPI_ATTR VkResult VKAPI_CALL QueuePresentKHR(VkQueue queue, const VkPresentInf
     }
     RecordObject record_obj(vvl::Func::vkQueuePresentKHR);
     {
-        VVL_ZoneScopedN("PreCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallRecord_QueuePresentKHR");
         for (auto& vo : device_dispatch->intercept_vectors[InterceptIdPreCallRecordQueuePresentKHR]) {
             if (!vo) {
                 continue;
@@ -1005,7 +1113,7 @@ VKAPI_ATTR VkResult VKAPI_CALL QueuePresentKHR(VkQueue queue, const VkPresentInf
 
     VkResult result;
     {
-        VVL_ZoneScopedN("Dispatch_" __FUNCTION__);
+        VVL_ZoneScopedN("Dispatch_QueuePresentKHR");
         result = device_dispatch->QueuePresentKHR(queue, pPresentInfo);
     }
     VVL_TracyCFrameMark;
@@ -1014,7 +1122,7 @@ VKAPI_ATTR VkResult VKAPI_CALL QueuePresentKHR(VkQueue queue, const VkPresentInf
 #endif
     record_obj.result = result;
     {
-        VVL_ZoneScopedN("PostCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PostCallRecord_QueuePresentKHR");
         for (auto& vo : device_dispatch->intercept_vectors[InterceptIdPostCallRecordQueuePresentKHR]) {
             if (!vo) {
                 continue;
@@ -1041,7 +1149,7 @@ VKAPI_ATTR VkResult VKAPI_CALL BeginCommandBuffer(VkCommandBuffer commandBuffer,
                           &handle_data);
     handle_data.command_buffer.is_secondary = device_dispatch->IsSecondary(commandBuffer);
     {
-        VVL_ZoneScopedN("PreCallValidate_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallValidate_BeginCommandBuffer");
         for (const auto& vo : device_dispatch->intercept_vectors[InterceptIdPreCallValidateBeginCommandBuffer]) {
             if (!vo) {
                 continue;
@@ -1054,7 +1162,7 @@ VKAPI_ATTR VkResult VKAPI_CALL BeginCommandBuffer(VkCommandBuffer commandBuffer,
 
     RecordObject record_obj(vvl::Func::vkBeginCommandBuffer, &handle_data);
     {
-        VVL_ZoneScopedN("PreCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PreCallRecord_BeginCommandBuffer");
         for (auto& vo : device_dispatch->intercept_vectors[InterceptIdPreCallRecordBeginCommandBuffer]) {
             if (!vo) {
                 continue;
@@ -1066,13 +1174,13 @@ VKAPI_ATTR VkResult VKAPI_CALL BeginCommandBuffer(VkCommandBuffer commandBuffer,
 
     VkResult result;
     {
-        VVL_ZoneScopedN("Dispatch_" __FUNCTION__);
+        VVL_ZoneScopedN("Dispatch_BeginCommandBuffer");
         result = device_dispatch->BeginCommandBuffer(commandBuffer, pBeginInfo);
     }
     record_obj.result = result;
 
     {
-        VVL_ZoneScopedN("PostCallRecord_" __FUNCTION__);
+        VVL_ZoneScopedN("PostCallRecord_BeginCommandBuffer");
         for (auto& vo : device_dispatch->intercept_vectors[InterceptIdPostCallRecordBeginCommandBuffer]) {
             if (!vo) {
                 continue;
