@@ -20,35 +20,31 @@
  ****************************************************************************/
 #pragma once
 
-#include <atomic>
-#include <shared_mutex>
+// This is just a "simple" way to make sure everyone has access to things like PRIu32
 #include <cinttypes>
+
+// simple util that is everywhere and accepting this as spot to allow global access to it
+#include "utils/assert_utils.h"
+
+#include <shared_mutex>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <algorithm>
 #include <memory>
-#include <string_view>
 
 #include <vulkan/vulkan.h>
 #include <vulkan/vk_layer.h>
 #include <vulkan/vk_enum_string_helper.h>
 #include <vulkan/utility/vk_struct_helper.hpp>
 #include <vulkan/utility/vk_safe_struct.hpp>
-#include "utils/cast_utils.h"
 #include "layer_options.h"
-#include "containers/custom_containers.h"
 #include "error_message/logging.h"
 #include "error_message/error_location.h"
 #include "error_message/record_object.h"
-#include "error_message/log_message_type.h"
-#include "utils/vk_layer_extension_utils.h"
-#include "utils/vk_layer_utils.h"
 #include "generated/vk_dispatch_table_helper.h"
 #include "chassis/dispatch_object.h"
 #include "generated/vk_extension_helper.h"
-#include "gpuav/core/gpuav_settings.h"
-#include "sync/sync_settings.h"
+#include "utils/lock_utils.h"
 
 namespace chassis {
 struct CreateGraphicsPipelines;
@@ -57,6 +53,7 @@ struct CreateRayTracingPipelinesNV;
 struct CreateRayTracingPipelinesKHR;
 struct CreateShaderModule;
 struct ShaderObject;
+struct ShaderBinaryData;
 struct CreatePipelineLayout;
 struct CreateBuffer;
 }  // namespace chassis
@@ -66,6 +63,10 @@ struct AllocateDescriptorSetsData;
 class Pipeline;
 }  // namespace vvl
 
+struct GlobalSettings;
+struct GpuAVSettings;
+struct SyncValSettings;
+
 // Because of GPL, we currently create our Pipeline state objects before the PreCallValidate
 // Each chassis layer will need to track its own state
 using PipelineStates = std::vector<std::shared_ptr<vvl::Pipeline>>;
@@ -90,8 +91,8 @@ class Instance : public Logger {
     GpuAVSettings& gpuav_settings;
     const SyncValSettings& syncval_settings;
 
-    const CHECK_DISABLED& disabled;
-    const CHECK_ENABLED& enabled;
+    const ValidationDisabled& disabled;
+    const ValidationEnabled& enabled;
 
     VkInstance instance = VK_NULL_HANDLE;
     const LayerObjectTypeId container_type;
@@ -119,6 +120,12 @@ class Instance : public Logger {
     void CopyDispatchState() { instance = dispatch_instance_->instance; }
     VkInstance VkHandle() const { return instance; }
 
+#if defined(DEBUG_CAPTURE_KEYBOARD)
+    // keep thing as void pointer to simplify including headers
+    void* xlib_display = nullptr;
+    void* xcb_connection = nullptr;
+#endif
+
 #include "generated/validation_object_instance_methods.h"
 };
 
@@ -141,8 +148,8 @@ class Device : public Logger {
     GpuAVSettings& gpuav_settings;
     const SyncValSettings& syncval_settings;
 
-    const CHECK_DISABLED& disabled;
-    const CHECK_ENABLED& enabled;
+    const ValidationDisabled& disabled;
+    const ValidationEnabled& enabled;
 
     const VkInstance instance;
     const VkPhysicalDevice physical_device;
@@ -299,24 +306,21 @@ class Device : public Logger {
                                                             uint32_t createInfoCount,
                                                             const VkRayTracingPipelineCreateInfoNV* pCreateInfos,
                                                             const VkAllocationCallbacks* pAllocator, VkPipeline* pPipelines,
-                                                            const ErrorObject& error_obj, PipelineStates& pipeline_states,
-                                                            chassis::CreateRayTracingPipelinesNV& chassis_state) const {
+                                                            const ErrorObject& error_obj, PipelineStates& pipeline_states) const {
         return PreCallValidateCreateRayTracingPipelinesNV(device, pipelineCache, createInfoCount, pCreateInfos, pAllocator,
                                                           pPipelines, error_obj);
     }
     virtual void PreCallRecordCreateRayTracingPipelinesNV(VkDevice device, VkPipelineCache pipelineCache, uint32_t createInfoCount,
                                                           const VkRayTracingPipelineCreateInfoNV* pCreateInfos,
                                                           const VkAllocationCallbacks* pAllocator, VkPipeline* pPipelines,
-                                                          const RecordObject& record_obj, PipelineStates& pipeline_states,
-                                                          chassis::CreateRayTracingPipelinesNV& chassis_state) {
+                                                          const RecordObject& record_obj, PipelineStates& pipeline_states) {
         PreCallRecordCreateRayTracingPipelinesNV(device, pipelineCache, createInfoCount, pCreateInfos, pAllocator, pPipelines,
                                                  record_obj);
     }
     virtual void PostCallRecordCreateRayTracingPipelinesNV(VkDevice device, VkPipelineCache pipelineCache, uint32_t createInfoCount,
                                                            const VkRayTracingPipelineCreateInfoNV* pCreateInfos,
                                                            const VkAllocationCallbacks* pAllocator, VkPipeline* pPipelines,
-                                                           const RecordObject& record_obj, PipelineStates& pipeline_states,
-                                                           chassis::CreateRayTracingPipelinesNV& chassis_state) {
+                                                           const RecordObject& record_obj, PipelineStates& pipeline_states) {
         PostCallRecordCreateRayTracingPipelinesNV(device, pipelineCache, createInfoCount, pCreateInfos, pAllocator, pPipelines,
                                                   record_obj);
     }
@@ -380,6 +384,12 @@ class Device : public Logger {
         PostCallRecordCreateShadersEXT(device, createInfoCount, pCreateInfos, pAllocator, pShaders, record_obj);
     }
 
+    // Allow modification of a down-chain parameter for CreatePipelineLayout
+    virtual void PreCallRecordGetShaderBinaryDataEXT(VkDevice device, VkShaderEXT shader, size_t* pDataSize, void* pData,
+                                                     const RecordObject& record_obj, chassis::ShaderBinaryData& chassis_state) {
+        PreCallRecordGetShaderBinaryDataEXT(device, shader, pDataSize, pData, record_obj);
+    }
+
     // Allow AllocateDescriptorSets to use some local stack storage for performance purposes
     virtual bool PreCallValidateAllocateDescriptorSets(VkDevice device, const VkDescriptorSetAllocateInfo* pAllocateInfo,
                                                        VkDescriptorSet* pDescriptorSets, const ErrorObject& error_obj,
