@@ -21,13 +21,16 @@
 #include <vector>
 
 #include <vulkan/vk_enum_string_helper.h>
+#include "cc_sync_vuid_maps.h"
 #include "cc_synchronization.h"
 #include "core_validation.h"
+#include "core_checks/cc_state_tracker.h"
 #include "state_tracker/queue_state.h"
 #include "state_tracker/semaphore_state.h"
 #include "state_tracker/image_state.h"
 #include "state_tracker/buffer_state.h"
-#include "sync/sync_vuid_maps.h"
+#include "state_tracker/event_map.h"
+#include "state_tracker/cmd_buffer_state.h"
 
 // Holds common information between all command buffers being submitted
 struct CommandBufferSubmitState {
@@ -36,16 +39,15 @@ struct CommandBufferSubmitState {
     QFOTransferCBScoreboards<QFOImageTransferBarrier> qfo_image_scoreboards;
     QFOTransferCBScoreboards<QFOBufferTransferBarrier> qfo_buffer_scoreboards;
     std::vector<VkCommandBuffer> current_cmds;
-    GlobalImageLayoutMap global_image_layout_map;
     std::vector<std::string> cmdbuf_label_stack;
     std::string last_closed_cmdbuf_label;
     bool found_unbalanced_cmdbuf_label;
 
-    // The "local" prefix is about tracking state within a *single* queue submission
-    // (accross all command buffers of that submission), as opposed to globally
-    // tracking state accross *all* submissions to the same queue.
+    // The "local" prefix is about tracking state accross command buffers of a *single* QueueSubmit command.
+    // This does not accumulate state from the previous submissions.
     QueryMap local_query_to_state_map;
     EventMap local_event_signal_info;
+    vvl::unordered_map<const vvl::Image *, ImageLayoutMap> local_image_layout_state;
     vvl::unordered_map<VkVideoSessionKHR, vvl::VideoSessionDeviceState> local_video_session_state{};
 
     CommandBufferSubmitState(const CoreChecks &c, const vvl::Queue &q) : core(c), queue_state(q) {
@@ -58,7 +60,6 @@ struct CommandBufferSubmitState {
 
     bool Validate(const Location &loc, const vvl::CommandBuffer &cb_state, uint32_t perf_pass) {
         bool skip = false;
-        skip |= core.ValidateCmdBufImageLayouts(loc, cb_state, global_image_layout_map);
         const VkCommandBuffer cmd = cb_state.VkHandle();
         current_cmds.push_back(cmd);
         skip |= core.ValidatePrimaryCommandBufferState(
@@ -72,16 +73,13 @@ struct CommandBufferSubmitState {
             return true;
         }
 
-        // Call submit-time functions to validate or update local mirrors of state (to preserve const-ness at validate time)
-        for (auto &function : cb_state.queue_submit_functions) {
-            skip |= function(queue_state, cb_state);
-        }
-        for (auto &function : cb_state.event_updates) {
+        auto &cb_sub_state = core::SubState(cb_state);
+        for (auto &function : cb_sub_state.event_updates) {
             skip |= function(const_cast<vvl::CommandBuffer &>(cb_state), /*do_validate*/ true, local_event_signal_info,
                              queue_state.VkHandle(), loc);
         }
         VkQueryPool first_perf_query_pool = VK_NULL_HANDLE;
-        for (auto &function : cb_state.query_updates) {
+        for (auto &function : cb_sub_state.query_updates) {
             skip |= function(const_cast<vvl::CommandBuffer &>(cb_state), /*do_validate*/ true, first_perf_query_pool, perf_pass,
                              &local_query_to_state_map);
         }
@@ -196,13 +194,13 @@ bool CoreChecks::PreCallValidateQueueSubmit(VkQueue queue, uint32_t submitCount,
             }
 
             // Validate flags for dynamic rendering
-            if (suspended_render_pass_instance && cb_state->hasRenderPassInstance && !cb_state->resumesRenderPassInstance) {
+            if (suspended_render_pass_instance && cb_state->has_render_pass_instance && !cb_state->resumes_render_pass_instance) {
                 skip |= LogError("VUID-VkSubmitInfo-pCommandBuffers-06016", queue, submit_loc,
                                  "has a suspended render pass instance, but pCommandBuffers[%" PRIu32
                                  "] has its own render pass instance that does not resume it.",
                                  i);
             }
-            if (cb_state->resumesRenderPassInstance) {
+            if (cb_state->resumes_render_pass_instance) {
                 if (!suspended_render_pass_instance) {
                     const LogObjectList objlist(cb_state->Handle(), queue);
                     skip |= LogError("VUID-VkSubmitInfo-pCommandBuffers-06193", objlist, cb_loc,
@@ -210,7 +208,7 @@ bool CoreChecks::PreCallValidateQueueSubmit(VkQueue queue, uint32_t submitCount,
                 }
                 suspended_render_pass_instance = false;
             }
-            if (cb_state->suspendsRenderPassInstance) {
+            if (cb_state->suspends_render_pass_instance) {
                 suspended_render_pass_instance = true;
             }
         }
@@ -261,7 +259,7 @@ bool CoreChecks::ValidateRenderPassStripeSubmitInfo(VkQueue queue, const vvl::Co
 
     const VkRenderPassStripeSubmitInfoARM *rp_submit_info = vku::FindStructInPNextChain<VkRenderPassStripeSubmitInfoARM>(pNext);
     if (!rp_submit_info) {
-        if (cb_state.has_render_pass_striped && !cb_state.resumesRenderPassInstance) {
+        if (cb_state.has_render_pass_striped && !cb_state.resumes_render_pass_instance) {
             skip |= LogError("VUID-VkCommandBufferSubmitInfo-commandBuffer-09445", objlist, loc.dot(Field::pNext),
                              "missing VkRenderPassStripeSubmitInfoARM struct because command buffer contain begin info "
                              "with renderpass striped struct");
@@ -269,7 +267,7 @@ bool CoreChecks::ValidateRenderPassStripeSubmitInfo(VkQueue queue, const vvl::Co
         return skip;
     }
 
-    if (rp_submit_info->stripeSemaphoreInfoCount != cb_state.striped_count && !cb_state.resumesRenderPassInstance) {
+    if (rp_submit_info->stripeSemaphoreInfoCount != cb_state.striped_count && !cb_state.resumes_render_pass_instance) {
         skip |= LogError("VUID-VkCommandBufferSubmitInfo-pNext-09446", objlist,
                          loc.pNext(Struct::VkRenderPassStripeSubmitInfoARM, Field::stripeSemaphoreInfoCount),
                          "(%" PRIu32 ") must be equal to VkRenderPassStripeBeginInfoARM::stripeInfoCount (%" PRIu32 ").",
@@ -361,16 +359,16 @@ bool CoreChecks::ValidateQueueSubmit2(VkQueue queue, uint32_t submitCount, const
                                  submit_loc.Fields().c_str(), string_VkSubmitFlags(submit.flags).c_str());
             }
 
-            if (suspended_render_pass_instance && cb_state->hasRenderPassInstance && !cb_state->resumesRenderPassInstance) {
+            if (suspended_render_pass_instance && cb_state->has_render_pass_instance && !cb_state->resumes_render_pass_instance) {
                 skip |= LogError("VUID-VkSubmitInfo2-commandBuffer-06012", queue, submit_loc,
                                  "has a suspended render pass instance, but pCommandBuffers[%" PRIu32
                                  "] has its own render pass instance that does not resume it.",
                                  i);
             }
-            if (cb_state->suspendsRenderPassInstance) {
+            if (cb_state->suspends_render_pass_instance) {
                 suspended_render_pass_instance = true;
             }
-            if (cb_state->resumesRenderPassInstance) {
+            if (cb_state->resumes_render_pass_instance) {
                 if (!suspended_render_pass_instance) {
                     skip |= LogError("VUID-VkSubmitInfo2-commandBuffer-06192", queue, cb_loc,
                                      "resumes a render pass instance, but there is no suspended render pass instance.");
@@ -401,9 +399,9 @@ bool CoreChecks::PreCallValidateQueueSubmit2(VkQueue queue, uint32_t submitCount
 
 void CoreChecks::PostCallRecordQueueSubmit(VkQueue queue, uint32_t submitCount, const VkSubmitInfo *pSubmits, VkFence fence,
                                            const RecordObject &record_obj) {
-    BaseClass::PostCallRecordQueueSubmit(queue, submitCount, pSubmits, fence, record_obj);
-
-    if (record_obj.result != VK_SUCCESS) return;
+    if (record_obj.result != VK_SUCCESS) {
+        return;
+    }
     // The triply nested for duplicates that in the StateTracker, but avoids the need for two additional callbacks.
     for (uint32_t submit_idx = 0; submit_idx < submitCount; submit_idx++) {
         const VkSubmitInfo &submit = pSubmits[submit_idx];
@@ -411,11 +409,9 @@ void CoreChecks::PostCallRecordQueueSubmit(VkQueue queue, uint32_t submitCount,
             auto cb_state = GetWrite<vvl::CommandBuffer>(submit.pCommandBuffers[i]);
             ASSERT_AND_CONTINUE(cb_state);
 
-            for (auto *secondary_cmd_buffer : cb_state->linkedCommandBuffers) {
-                UpdateCmdBufImageLayouts(*secondary_cmd_buffer);
+            for (auto *secondary_cmd_buffer : cb_state->linked_command_buffers) {
                 RecordQueuedQFOTransfers(*secondary_cmd_buffer);
             }
-            UpdateCmdBufImageLayouts(*cb_state);
             RecordQueuedQFOTransfers(*cb_state);
         }
     }
@@ -423,7 +419,9 @@ void CoreChecks::PostCallRecordQueueSubmit(VkQueue queue, uint32_t submitCount,
 
 void CoreChecks::RecordQueueSubmit2(VkQueue queue, uint32_t submitCount, const VkSubmitInfo2 *pSubmits, VkFence fence,
                                     const RecordObject &record_obj) {
-    if (record_obj.result != VK_SUCCESS) return;
+    if (record_obj.result != VK_SUCCESS) {
+        return;
+    }
     // The triply nested for duplicates that in the StateTracker, but avoids the need for two additional callbacks.
     for (uint32_t submit_idx = 0; submit_idx < submitCount; submit_idx++) {
         const VkSubmitInfo2 &submit = pSubmits[submit_idx];
@@ -431,11 +429,9 @@ void CoreChecks::RecordQueueSubmit2(VkQueue queue, uint32_t submitCount, const V
             auto cb_state = GetWrite<vvl::CommandBuffer>(submit.pCommandBufferInfos[i].commandBuffer);
             ASSERT_AND_CONTINUE(cb_state);
 
-            for (auto *secondary_cmd_buffer : cb_state->linkedCommandBuffers) {
-                UpdateCmdBufImageLayouts(*secondary_cmd_buffer);
+            for (auto *secondary_cmd_buffer : cb_state->linked_command_buffers) {
                 RecordQueuedQFOTransfers(*secondary_cmd_buffer);
             }
-            UpdateCmdBufImageLayouts(*cb_state);
             RecordQueuedQFOTransfers(*cb_state);
         }
     }
@@ -448,7 +444,6 @@ void CoreChecks::PostCallRecordQueueSubmit2KHR(VkQueue queue, uint32_t submitCou
 
 void CoreChecks::PostCallRecordQueueSubmit2(VkQueue queue, uint32_t submitCount, const VkSubmitInfo2 *pSubmits, VkFence fence,
                                             const RecordObject &record_obj) {
-    BaseClass::PostCallRecordQueueSubmit2(queue, submitCount, pSubmits, fence, record_obj);
     RecordQueueSubmit2(queue, submitCount, pSubmits, fence, record_obj);
 }
 
@@ -479,15 +474,13 @@ bool CoreChecks::ValidImageBufferQueue(const vvl::CommandBuffer &cb_state, const
 // Secondary command buffers were previously validated in vkCmdExecuteCommands().
 bool CoreChecks::ValidateQueueFamilyIndices(const Location &loc, const vvl::CommandBuffer &cb_state,
                                             const vvl::Queue &queue_state) const {
-    using sync_vuid_maps::GetQueueSubmitVUID;
-    using sync_vuid_maps::SubmitError;
     bool skip = false;
     auto pool = cb_state.command_pool;
     ASSERT_AND_RETURN_SKIP(pool);
 
     if (pool->queueFamilyIndex != queue_state.queue_family_index) {
         const LogObjectList objlist(cb_state.Handle(), queue_state.Handle());
-        const auto &vuid = GetQueueSubmitVUID(loc, SubmitError::kCmdWrongQueueFamily);
+        const auto &vuid = GetQueueSubmitVUID(loc, vvl::SubmitError::kCmdWrongQueueFamily);
         skip |= LogError(vuid, objlist, loc,
                          "Primary command buffer %s created in queue family %d is being submitted on %s "
                          "from queue family %d.",
@@ -532,8 +525,8 @@ bool CoreChecks::ValidateCommandBufferState(const vvl::CommandBuffer &cb_state,
     }
 
     // Validate ONE_TIME_SUBMIT_BIT CB is not being submitted more than once
-    if (const uint64_t submissions = cb_state.submitCount + current_submit_count;
-        (cb_state.beginInfo.flags & VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT) && (submissions > 1)) {
+    if (const uint64_t submissions = cb_state.submit_count + current_submit_count;
+        (cb_state.begin_info_flags & VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT) && (submissions > 1)) {
         // VU being worked on https://gitlab.khronos.org/vulkan/vulkan/-/issues/2456
         skip |= LogError("UNASSIGNED-DrawState-CommandBufferSingleSubmitViolation", cb_state.Handle(), loc,
                          "%s recorded with VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT has been submitted %" PRIu64 " times.",
@@ -565,13 +558,10 @@ bool CoreChecks::ValidateCommandBufferState(const vvl::CommandBuffer &cb_state,
 
 bool CoreChecks::ValidateCommandBufferSimultaneousUse(const Location &loc, const vvl::CommandBuffer &cb_state,
                                                       int current_submit_count) const {
-    using sync_vuid_maps::GetQueueSubmitVUID;
-    using sync_vuid_maps::SubmitError;
-
     bool skip = false;
     if ((cb_state.InUse() || current_submit_count > 1) &&
-        !(cb_state.beginInfo.flags & VK_COMMAND_BUFFER_USAGE_SIMULTANEOUS_USE_BIT)) {
-        const auto &vuid = sync_vuid_maps::GetQueueSubmitVUID(loc, SubmitError::kCmdNotSimultaneous);
+        !(cb_state.begin_info_flags & VK_COMMAND_BUFFER_USAGE_SIMULTANEOUS_USE_BIT)) {
+        const auto &vuid = GetQueueSubmitVUID(loc, vvl::SubmitError::kCmdNotSimultaneous);
 
         skip |= LogError(vuid, device, loc, "%s is already in use and is not marked for simultaneous use.",
                          FormatHandle(cb_state).c_str());
@@ -583,36 +573,34 @@ bool CoreChecks::ValidatePrimaryCommandBufferState(
     const Location &loc, const vvl::CommandBuffer &cb_state, uint32_t current_submit_count,
     QFOTransferCBScoreboards<QFOImageTransferBarrier> *qfo_image_scoreboards,
     QFOTransferCBScoreboards<QFOBufferTransferBarrier> *qfo_buffer_scoreboards) const {
-    using sync_vuid_maps::GetQueueSubmitVUID;
-    using sync_vuid_maps::SubmitError;
 
     // Track in-use for resources off of primary and any secondary CBs
     bool skip = false;
 
     if (cb_state.IsSecondary()) {
-        const auto &vuid = GetQueueSubmitVUID(loc, SubmitError::kSecondaryCmdInSubmit);
+        const auto &vuid = GetQueueSubmitVUID(loc, vvl::SubmitError::kSecondaryCmdInSubmit);
         skip |= LogError(vuid, cb_state.Handle(), loc, "Command buffer %s must be allocated with VK_COMMAND_BUFFER_LEVEL_PRIMARY.",
                          FormatHandle(cb_state).c_str());
     } else {
-        for (const auto *sub_cb : cb_state.linkedCommandBuffers) {
+        for (const auto *sub_cb : cb_state.linked_command_buffers) {
             skip |= ValidateQueuedQFOTransfers(*sub_cb, qfo_image_scoreboards, qfo_buffer_scoreboards, loc);
             // TODO: replace with InvalidateCommandBuffers() at recording.
-            if ((sub_cb->primaryCommandBuffer != cb_state.VkHandle()) &&
-                !(sub_cb->beginInfo.flags & VK_COMMAND_BUFFER_USAGE_SIMULTANEOUS_USE_BIT)) {
-                const auto &vuid = GetQueueSubmitVUID(loc, SubmitError::kSecondaryCmdNotSimultaneous);
-                const LogObjectList objlist(device, cb_state.Handle(), sub_cb->Handle(), sub_cb->primaryCommandBuffer);
+            if ((sub_cb->primary_command_buffer != cb_state.VkHandle()) &&
+                !(sub_cb->begin_info_flags & VK_COMMAND_BUFFER_USAGE_SIMULTANEOUS_USE_BIT)) {
+                const auto &vuid = GetQueueSubmitVUID(loc, vvl::SubmitError::kSecondaryCmdNotSimultaneous);
+                const LogObjectList objlist(device, cb_state.Handle(), sub_cb->Handle(), sub_cb->primary_command_buffer);
                 skip |= LogError(vuid, objlist, loc,
                                  "%s was submitted with secondary %s but that buffer has subsequently been bound to "
                                  "primary %s and it does not have VK_COMMAND_BUFFER_USAGE_SIMULTANEOUS_USE_BIT set.",
                                  FormatHandle(cb_state).c_str(), FormatHandle(sub_cb->Handle()).c_str(),
-                                 FormatHandle(sub_cb->primaryCommandBuffer).c_str());
+                                 FormatHandle(sub_cb->primary_command_buffer).c_str());
             }
 
             if (sub_cb->state != CbState::Recorded) {
                 const char *const finished_cb_vuid = (loc.function == Func::vkQueueSubmit)
                                                          ? "VUID-vkQueueSubmit-pCommandBuffers-00072"
                                                          : "VUID-vkQueueSubmit2-commandBuffer-03876";
-                const LogObjectList objlist(device, cb_state.Handle(), sub_cb->Handle(), sub_cb->primaryCommandBuffer);
+                const LogObjectList objlist(device, cb_state.Handle(), sub_cb->Handle(), sub_cb->primary_command_buffer);
                 skip |= LogError(finished_cb_vuid, objlist, loc,
                                  "Secondary command buffer %s is not in a valid (pending or executable) state.",
                                  FormatHandle(sub_cb->Handle()).c_str());
