@@ -1,6 +1,6 @@
-/* Copyright (c) 2015-2024 The Khronos Group Inc.
- * Copyright (c) 2015-2024 Valve Corporation
- * Copyright (c) 2015-2024 LunarG, Inc.
+/* Copyright (c) 2015-2025 The Khronos Group Inc.
+ * Copyright (c) 2015-2025 Valve Corporation
+ * Copyright (c) 2015-2025 LunarG, Inc.
  * Copyright (C) 2015-2023 Google Inc.
  * Modifications Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights reserved.
  * Modifications Copyright (C) 2022 RasterGrid Kft.
@@ -20,6 +20,8 @@
  */
 
 #include "core_validation.h"
+#include <vulkan/utility/vk_format_utils.h>
+#include "utils/image_utils.h"
 
 // There is a table in the Vulkan spec to list all formats that implicitly require YCbCr conversion,
 // but some features/extensions can explicitly turn that restriction off
@@ -62,8 +64,8 @@ bool CoreChecks::PreCallValidateCreateSamplerYcbcrConversion(VkDevice device, co
         // only check for external format inside VK_FORMAT_UNDEFINED check to prevent unnecessary extra errors from no format
         // features being supported
         if (external_format != 0) {
-            auto it = ahb_ext_formats_map.find(external_format);
-            if (it != ahb_ext_formats_map.end()) {
+            auto it = device_state->ahb_ext_formats_map.find(external_format);
+            if (it != device_state->ahb_ext_formats_map.end()) {
                 format_features = it->second;
             }
         }
