@@ -1,6 +1,6 @@
-/* Copyright (c) 2021-2024 The Khronos Group Inc.
- * Copyright (c) 2021-2024 Valve Corporation
- * Copyright (c) 2021-2024 LunarG, Inc.
+/* Copyright (c) 2021-2025 The Khronos Group Inc.
+ * Copyright (c) 2021-2025 Valve Corporation
+ * Copyright (c) 2021-2025 LunarG, Inc.
  * Copyright (C) 2021-2022 Google Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,6 +16,7 @@
  * limitations under the License.
  */
 #include "error_location.h"
+#include "generated/error_location_helper.h"
 
 void Location::AppendFields(std::ostream& out) const {
     if (prev) {
@@ -63,6 +64,44 @@ std::string Location::Message() const {
     return message;
 }
 
+std::string PrintPNextChain(vvl::Struct in_struct, const void* in_pNext) {
+    std::stringstream out;
+
+    if (in_pNext) {
+        // We might not have a good way to pass in the original struct
+        if (in_struct == vvl::Struct::Empty) {
+            out << "pNext";
+        } else {
+            out << "pNext chain: " << vvl::String(in_struct) << "::pNext";
+        }
+
+        for (const VkBaseInStructure* current = static_cast<const VkBaseInStructure*>(in_pNext); current != nullptr;
+             current = current->pNext) {
+            // Special case for the 2 special loader structs
+            if (current->sType == VK_STRUCTURE_TYPE_LOADER_INSTANCE_CREATE_INFO) {
+                out << " -> [VK_STRUCTURE_TYPE_LOADER_INSTANCE_CREATE_INFO]";
+            } else if (current->sType == VK_STRUCTURE_TYPE_LOADER_DEVICE_CREATE_INFO) {
+                out << " -> [VK_STRUCTURE_TYPE_LOADER_DEVICE_CREATE_INFO]";
+            } else {
+                vvl::Struct next_struct = vvl::StypeToStruct(current->sType);
+                if (next_struct == vvl::Struct::Empty) {
+                    out << " -> [Unknown VkStructureType " << (int)current->sType << "]";
+                } else {
+                    out << " -> [" << vvl::String(next_struct) << "]";
+                }
+            }
+        }
+    } else {
+        if (in_struct == vvl::Struct::Empty) {
+            out << "pNext is NULL";
+        } else {
+            out << vvl::String(in_struct) << "::pNext is NULL";
+        }
+    }
+
+    return out.str();
+}
+
 namespace vvl {
 LocationCapture::LocationCapture(const Location& loc) { Capture(loc, 1); }
 
@@ -88,6 +127,19 @@ LocationCapture::LocationCapture(LocationCapture&& other)
     }
 }
 
+LocationCapture& LocationCapture::operator=(LocationCapture&& other) {
+    capture.clear();
+    capture.PushBackFrom(other.capture);
+    if (capture.empty()) {
+        return *this;
+    }
+    capture[0].prev = nullptr;
+    for (CaptureStore::size_type i = 1; i < capture.size(); i++) {
+        capture[i].prev = &capture[i - 1];
+    }
+    return *this;
+}
+
 const Location* LocationCapture::Capture(const Location& loc, CaptureStore::size_type depth) {
     const Location* prev_capture = nullptr;
     if (loc.prev) {
