@@ -1,7 +1,7 @@
-/* Copyright (c) 2015-2024 The Khronos Group Inc.
- * Copyright (c) 2015-2024 Valve Corporation
- * Copyright (c) 2015-2024 LunarG, Inc.
- * Copyright (C) 2015-2024 Google Inc.
+/* Copyright (c) 2015-2025 The Khronos Group Inc.
+ * Copyright (c) 2015-2025 Valve Corporation
+ * Copyright (c) 2015-2025 LunarG, Inc.
+ * Copyright (C) 2015-2025 Google Inc.
  * Modifications Copyright (C) 2020 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -26,34 +26,35 @@ using BoundMemoryRange = vvl::BindableMemoryTracker::BoundMemoryRange;
 using BoundRanges = vvl::BindableLinearMemoryTracker::BoundRanges;
 using DeviceMemoryState = vvl::BindableMemoryTracker::DeviceMemoryState;
 
+namespace vvl {
 // It is allowed to export memory into the handles of different types,
 // that's why we use set of flags (VkExternalMemoryHandleTypeFlags)
-static VkExternalMemoryHandleTypeFlags GetExportHandleTypes(const VkMemoryAllocateInfo *p_alloc_info) {
-    auto export_info = vku::FindStructInPNextChain<VkExportMemoryAllocateInfo>(p_alloc_info->pNext);
+static VkExternalMemoryHandleTypeFlags GetExportHandleTypes(const VkMemoryAllocateInfo &alloc_info) {
+    auto export_info = vku::FindStructInPNextChain<VkExportMemoryAllocateInfo>(alloc_info.pNext);
     return export_info ? export_info->handleTypes : 0;
 }
 
 // Import works with a single handle type, that's why VkExternalMemoryHandleTypeFlagBits type is used.
 // Since FlagBits-type cannot have a value of 0, we use std::optional to indicate the presense of an import operation.
-static std::optional<VkExternalMemoryHandleTypeFlagBits> GetImportHandleType(const VkMemoryAllocateInfo *p_alloc_info) {
+std::optional<VkExternalMemoryHandleTypeFlagBits> GetImportHandleType(const VkMemoryAllocateInfo &alloc_info) {
 #ifdef VK_USE_PLATFORM_WIN32_KHR
-    auto win32_import = vku::FindStructInPNextChain<VkImportMemoryWin32HandleInfoKHR>(p_alloc_info->pNext);
+    auto win32_import = vku::FindStructInPNextChain<VkImportMemoryWin32HandleInfoKHR>(alloc_info.pNext);
     if (win32_import) {
         return win32_import->handleType;
     }
 #endif
-    auto fd_import = vku::FindStructInPNextChain<VkImportMemoryFdInfoKHR>(p_alloc_info->pNext);
+    auto fd_import = vku::FindStructInPNextChain<VkImportMemoryFdInfoKHR>(alloc_info.pNext);
     if (fd_import) {
         return fd_import->handleType;
     }
-    auto host_pointer_import = vku::FindStructInPNextChain<VkImportMemoryHostPointerInfoEXT>(p_alloc_info->pNext);
+    auto host_pointer_import = vku::FindStructInPNextChain<VkImportMemoryHostPointerInfoEXT>(alloc_info.pNext);
     if (host_pointer_import) {
         return host_pointer_import->handleType;
     }
 #ifdef VK_USE_PLATFORM_ANDROID_KHR
     // AHB Import doesn't have handle in the pNext chain
     // It should be assumed that all imported AHB can only have the same, single handleType
-    auto ahb_import = vku::FindStructInPNextChain<VkImportAndroidHardwareBufferInfoANDROID>(p_alloc_info->pNext);
+    auto ahb_import = vku::FindStructInPNextChain<VkImportAndroidHardwareBufferInfoANDROID>(alloc_info.pNext);
     if ((ahb_import) && (ahb_import->buffer != nullptr)) {
         return VK_EXTERNAL_MEMORY_HANDLE_TYPE_ANDROID_HARDWARE_BUFFER_BIT_ANDROID;
     }
@@ -61,9 +62,9 @@ static std::optional<VkExternalMemoryHandleTypeFlagBits> GetImportHandleType(con
     return std::nullopt;
 }
 
-static bool IsMultiInstance(const VkMemoryAllocateInfo *p_alloc_info, const VkMemoryHeap &memory_heap,
+static bool IsMultiInstance(const VkMemoryAllocateInfo &alloc_info, const VkMemoryHeap &memory_heap,
                             uint32_t physical_device_count) {
-    auto alloc_flags = vku::FindStructInPNextChain<VkMemoryAllocateFlagsInfo>(p_alloc_info->pNext);
+    auto alloc_flags = vku::FindStructInPNextChain<VkMemoryAllocateFlagsInfo>(alloc_info.pNext);
     if (alloc_flags && (alloc_flags->flags & VK_MEMORY_ALLOCATE_DEVICE_MASK_BIT)) {
         auto dev_mask = alloc_flags->deviceMask;
         return ((dev_mask != 0) && (dev_mask & (dev_mask - 1))) != 0;
@@ -74,9 +75,9 @@ static bool IsMultiInstance(const VkMemoryAllocateInfo *p_alloc_info, const VkMe
 }
 
 #ifdef VK_USE_PLATFORM_METAL_EXT
-static bool GetMetalExport(const VkMemoryAllocateInfo *info) {
+static bool GetMetalExport(const VkMemoryAllocateInfo &alloc_info) {
     bool retval = false;
-    auto export_metal_object_info = vku::FindStructInPNextChain<VkExportMetalObjectCreateInfoEXT>(info->pNext);
+    auto export_metal_object_info = vku::FindStructInPNextChain<VkExportMetalObjectCreateInfoEXT>(alloc_info.pNext);
     while (export_metal_object_info) {
         if (export_metal_object_info->exportObjectType == VK_EXPORT_METAL_OBJECT_TYPE_METAL_BUFFER_BIT_EXT) {
             retval = true;
@@ -88,12 +89,11 @@ static bool GetMetalExport(const VkMemoryAllocateInfo *info) {
 }
 #endif  // VK_USE_PLATFORM_METAL_EXT
 
-namespace vvl {
-DeviceMemory::DeviceMemory(VkDeviceMemory handle, const VkMemoryAllocateInfo *allocate_info, uint64_t fake_address,
+DeviceMemory::DeviceMemory(VkDeviceMemory handle, const VkMemoryAllocateInfo *in_allocate_info, uint64_t fake_address,
                            const VkMemoryType &memory_type, const VkMemoryHeap &memory_heap,
                            std::optional<DedicatedBinding> &&dedicated_binding, uint32_t physical_device_count)
     : StateObject(handle, kVulkanObjectTypeDeviceMemory),
-      safe_allocate_info(allocate_info),
+      safe_allocate_info(in_allocate_info),
       allocate_info(*safe_allocate_info.ptr()),
       export_handle_types(GetExportHandleTypes(allocate_info)),
       import_handle_type(GetImportHandleType(allocate_info)),
@@ -109,12 +109,12 @@ DeviceMemory::DeviceMemory(VkDeviceMemory handle, const VkMemoryAllocateInfo *al
 }
 }  // namespace vvl
 
-void vvl::BindableLinearMemoryTracker::BindMemory(StateObject *parent, std::shared_ptr<vvl::DeviceMemory> &mem_state,
-                                             VkDeviceSize memory_offset, VkDeviceSize resource_offset, VkDeviceSize size) {
-    ASSERT_AND_RETURN(mem_state);
+void vvl::BindableLinearMemoryTracker::BindMemory(StateObject *parent, std::shared_ptr<vvl::DeviceMemory> &memory_state,
+                                                  VkDeviceSize memory_offset, VkDeviceSize resource_offset, VkDeviceSize size) {
+    ASSERT_AND_RETURN(memory_state);
 
-    mem_state->AddParent(parent);
-    binding_ = {mem_state, memory_offset, 0u};
+    memory_state->AddParent(parent);
+    binding_ = {memory_state, memory_offset, 0u};
 }
 
 DeviceMemoryState vvl::BindableLinearMemoryTracker::GetBoundMemoryStates() const {
@@ -177,9 +177,9 @@ bool vvl::BindableSparseMemoryTracker::HasFullRangeBound() const {
     return true;
 }
 
-void vvl::BindableSparseMemoryTracker::BindMemory(StateObject *parent, std::shared_ptr<vvl::DeviceMemory> &mem_state,
-                                             VkDeviceSize memory_offset, VkDeviceSize resource_offset, VkDeviceSize size) {
-    MEM_BINDING memory_data{mem_state, memory_offset, resource_offset};
+void vvl::BindableSparseMemoryTracker::BindMemory(StateObject *parent, std::shared_ptr<vvl::DeviceMemory> &memory_state,
+                                                  VkDeviceSize memory_offset, VkDeviceSize resource_offset, VkDeviceSize size) {
+    MemoryBinding memory_data{memory_state, memory_offset, resource_offset};
     BindingMap::value_type item{{resource_offset, resource_offset + size}, memory_data};
 
     auto guard = WriteLockGuard{binding_lock_};
@@ -269,16 +269,16 @@ BoundRanges vvl::BindableSparseMemoryTracker::GetBoundRanges(const BufferRange &
 }
 
 DeviceMemoryState vvl::BindableSparseMemoryTracker::GetBoundMemoryStates() const {
-    DeviceMemoryState dev_mem_states;
+    DeviceMemoryState dev_memory_states;
 
     {
         auto guard = ReadLockGuard{binding_lock_};
         for (auto &binding : binding_map_) {
-            if (binding.second.memory_state) dev_mem_states.emplace(binding.second.memory_state);
+            if (binding.second.memory_state) dev_memory_states.emplace(binding.second.memory_state);
         }
     }
 
-    return dev_mem_states;
+    return dev_memory_states;
 }
 
 
@@ -309,13 +309,14 @@ bool vvl::BindableMultiplanarMemoryTracker::HasFullRangeBound() const {
 }
 
 // resource_offset is the plane index
-void vvl::BindableMultiplanarMemoryTracker::BindMemory(StateObject *parent, std::shared_ptr<vvl::DeviceMemory> &mem_state,
-                                                  VkDeviceSize memory_offset, VkDeviceSize resource_offset, VkDeviceSize size) {
-    ASSERT_AND_RETURN(mem_state);
+void vvl::BindableMultiplanarMemoryTracker::BindMemory(StateObject *parent, std::shared_ptr<vvl::DeviceMemory> &memory_state,
+                                                       VkDeviceSize memory_offset, VkDeviceSize resource_offset,
+                                                       VkDeviceSize size) {
+    ASSERT_AND_RETURN(memory_state);
 
     assert(resource_offset < planes_.size());
-    mem_state->AddParent(parent);
-    planes_[static_cast<size_t>(resource_offset)].binding = {mem_state, memory_offset, 0u};
+    memory_state->AddParent(parent);
+    planes_[static_cast<size_t>(resource_offset)].binding = {memory_state, memory_offset, 0u};
 }
 
 // range needs to be between [0, planes_[0].size + planes_[1].size + planes_[2].size)
@@ -341,15 +342,15 @@ BoundMemoryRange vvl::BindableMultiplanarMemoryTracker::GetBoundMemoryRange(cons
 }
 
 DeviceMemoryState vvl::BindableMultiplanarMemoryTracker::GetBoundMemoryStates() const {
-    DeviceMemoryState dev_mem_states;
+    DeviceMemoryState dev_memory_states;
 
     for (unsigned i = 0u; i < planes_.size(); ++i) {
         if (planes_[i].binding.memory_state) {
-            dev_mem_states.insert(planes_[i].binding.memory_state);
+            dev_memory_states.insert(planes_[i].binding.memory_state);
         }
     }
 
-    return dev_mem_states;
+    return dev_memory_states;
 }
 
 std::pair<VkDeviceMemory, MemoryRange> vvl::Bindable::GetResourceMemoryOverlap(
@@ -378,7 +379,7 @@ std::pair<VkDeviceMemory, MemoryRange> vvl::Bindable::GetResourceMemoryOverlap(
 }
 
 VkDeviceSize vvl::Bindable::GetFakeBaseAddress() const {
-    assert(!sparse);  // not implemented yet
+    // TODO: Sparse resources are not implemented yet
     const auto *binding = Binding();
     return binding ? binding->memory_offset + binding->memory_state->fake_base_address : 0;
 }
