@@ -1,6 +1,6 @@
-/* Copyright (c) 2024 The Khronos Group Inc.
- * Copyright (c) 2024 Valve Corporation
- * Copyright (c) 2024 LunarG, Inc.
+/* Copyright (c) 2024-2025 The Khronos Group Inc.
+ * Copyright (c) 2024-2025 Valve Corporation
+ * Copyright (c) 2024-2025 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,6 +20,31 @@
 
 namespace vvl {
 
+void PhysicalDevice::SetCallState(Func func, CallState new_state) {
+    WriteLockGuard guard(call_state_lock_);
+    auto result = call_state_.emplace(func, new_state);
+    if (!result.second) {
+        if (result.first->second < new_state) {
+            result.first->second = new_state;
+        }
+    }
+}
+
+void PhysicalDevice::SetCallState(Func func, bool has_ptr) {
+    CallState new_state = has_ptr ? CallState::QueryDetails : CallState::QueryCount;
+    SetCallState(func, new_state);
+}
+
+CallState PhysicalDevice::GetCallState(Func func) const {
+    ReadLockGuard guard(call_state_lock_);
+    auto iter = call_state_.find(func);
+    return iter != call_state_.end() ? iter->second : CallState::Uncalled;
+}
+
+bool PhysicalDevice::WasUncalled(Func func) const { return GetCallState(func) == vvl::CallState::Uncalled; }
+
+bool PhysicalDevice::WasCalled(Func func) const { return GetCallState(func) != vvl::CallState::Uncalled; }
+
 const std::vector<VkQueueFamilyProperties> PhysicalDevice::GetQueueFamilyProps(VkPhysicalDevice phys_dev) {
     std::vector<VkQueueFamilyProperties> result;
     uint32_t count;
