@@ -1,5 +1,5 @@
-/* Copyright (c) 2023-2024 Nintendo
- * Copyright (c) 2023-2024 LunarG, Inc.
+/* Copyright (c) 2023-2025 Nintendo
+ * Copyright (c) 2023-2025 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -26,10 +26,13 @@
 #include "state_tracker/pipeline_layout_state.h"
 
 namespace vvl {
+
+class ShaderObjectSubState;
+
 // Represents a VkShaderEXT (VK_EXT_shader_object) handle
-struct ShaderObject : public StateObject {
-    ShaderObject(ValidationStateTracker &dev_data, const VkShaderCreateInfoEXT &create_info, VkShaderEXT shader_object,
-                 std::shared_ptr<spirv::Module> &spirv_module, uint32_t createInfoCount, VkShaderEXT *pShaders);
+struct ShaderObject : public StateObject, public SubStateManager<ShaderObjectSubState> {
+    ShaderObject(DeviceState &dev_data, const VkShaderCreateInfoEXT &create_info_i, VkShaderEXT shader_object,
+                 std::shared_ptr<spirv::Module> &spirv_module);
 
     const vku::safe_VkShaderCreateInfoEXT safe_create_info;
     const VkShaderCreateInfoEXT &create_info;
@@ -49,13 +52,21 @@ struct ShaderObject : public StateObject {
     const PushConstantRangesId push_constant_ranges;
     const std::vector<PipelineLayoutCompatId> set_compat_ids;
 
-    // TOOD Create a shader object inherited class
-    struct InstrumentationData {
-        bool was_instrumented = false;
-    } instrumentation_data;
-
     VkShaderEXT VkHandle() const { return handle_.Cast<VkShaderEXT>(); }
     bool IsGraphicsShaderState() const { return create_info.stage != VK_SHADER_STAGE_COMPUTE_BIT; };
     VkPrimitiveTopology GetTopology() const;
 };
+
+class ShaderObjectSubState {
+  public:
+    explicit ShaderObjectSubState(ShaderObject &obj) : base(obj) {}
+    ShaderObjectSubState(const ShaderObjectSubState &) = delete;
+    ShaderObjectSubState &operator=(const ShaderObjectSubState &) = delete;
+    virtual ~ShaderObjectSubState() {}
+    virtual void Destroy() {}
+    virtual void NotifyInvalidate(const StateObject::NodeList &invalid_nodes, bool unlink) {}
+
+    ShaderObject &base;
+};
+
 }  // namespace vvl
