@@ -94,6 +94,20 @@ bool VideoProfileDesc::InitProfile(VkVideoProfileInfoKHR const *profile) {
                 profile_.base.pNext = &profile_.decode_av1;
                 break;
             }
+            case VK_VIDEO_CODEC_OPERATION_DECODE_VP9_BIT_KHR: {
+                auto decode_vp9 = vku::FindStructInPNextChain<VkVideoDecodeVP9ProfileInfoKHR>(profile->pNext);
+                if (decode_vp9) {
+                    profile_.valid = true;
+                    profile_.decode_vp9 = *decode_vp9;
+                    profile_.decode_vp9.pNext = nullptr;
+                } else {
+                    profile_.valid = false;
+                    profile_.decode_vp9 = vku::InitStructHelper();
+                }
+                profile_.is_decode = true;
+                profile_.base.pNext = &profile_.decode_vp9;
+                break;
+            }
             case VK_VIDEO_CODEC_OPERATION_ENCODE_H264_BIT_KHR: {
                 auto encode_h264 = vku::FindStructInPNextChain<VkVideoEncodeH264ProfileInfoKHR>(profile->pNext);
                 if (encode_h264) {
@@ -193,13 +207,18 @@ void VideoProfileDesc::InitCapabilities(VkPhysicalDevice physical_device) {
             capabilities_.decode_av1 = vku::InitStructHelper();
             break;
 
+        case VK_VIDEO_CODEC_OPERATION_DECODE_VP9_BIT_KHR:
+            capabilities_.base.pNext = &capabilities_.decode;
+            capabilities_.decode = vku::InitStructHelper();
+            capabilities_.decode.pNext = &capabilities_.decode_vp9;
+            capabilities_.decode_vp9 = vku::InitStructHelper();
+            break;
+
         case VK_VIDEO_CODEC_OPERATION_ENCODE_H264_BIT_KHR:
             capabilities_.base.pNext = &capabilities_.encode;
             capabilities_.encode = vku::InitStructHelper();
             capabilities_.encode.pNext = &capabilities_.encode_h264;
             capabilities_.encode_h264 = vku::InitStructHelper();
-            capabilities_.encode_h264.pNext = &capabilities_.encode_ext.quantization_map;
-            capabilities_.encode_ext.quantization_map = vku::InitStructHelper();
             break;
 
         case VK_VIDEO_CODEC_OPERATION_ENCODE_H265_BIT_KHR:
@@ -207,8 +226,6 @@ void VideoProfileDesc::InitCapabilities(VkPhysicalDevice physical_device) {
             capabilities_.encode = vku::InitStructHelper();
             capabilities_.encode.pNext = &capabilities_.encode_h265;
             capabilities_.encode_h265 = vku::InitStructHelper();
-            capabilities_.encode_h265.pNext = &capabilities_.encode_ext.quantization_map;
-            capabilities_.encode_ext.quantization_map = vku::InitStructHelper();
             break;
 
         case VK_VIDEO_CODEC_OPERATION_ENCODE_AV1_BIT_KHR:
@@ -216,14 +233,20 @@ void VideoProfileDesc::InitCapabilities(VkPhysicalDevice physical_device) {
             capabilities_.encode = vku::InitStructHelper();
             capabilities_.encode.pNext = &capabilities_.encode_av1;
             capabilities_.encode_av1 = vku::InitStructHelper();
-            capabilities_.encode_av1.pNext = &capabilities_.encode_ext.quantization_map;
-            capabilities_.encode_ext.quantization_map = vku::InitStructHelper();
             break;
 
         default:
             return;
     }
 
+    if (IsEncode()) {
+        capabilities_.encode_ext.quantization_map = vku::InitStructHelper(capabilities_.base.pNext);
+        capabilities_.base.pNext = &capabilities_.encode_ext.quantization_map;
+
+        capabilities_.encode_ext.intra_refresh = vku::InitStructHelper(capabilities_.base.pNext);
+        capabilities_.base.pNext = &capabilities_.encode_ext.intra_refresh;
+    }
+
     VkResult result = DispatchGetPhysicalDeviceVideoCapabilitiesKHR(physical_device, &profile_.base, &capabilities_.base);
     if (result == VK_SUCCESS) {
         capabilities_.supported = true;
@@ -333,7 +356,7 @@ void VideoProfileDesc::Cache::Release(VideoProfileDesc const *desc) {
 VideoPictureResource::VideoPictureResource()
     : image_view_state(nullptr), image_state(nullptr), base_array_layer(0), range(), coded_offset(), coded_extent() {}
 
-VideoPictureResource::VideoPictureResource(const Device &dev_data, VkVideoPictureResourceInfoKHR const &res)
+VideoPictureResource::VideoPictureResource(const DeviceState &dev_data, VkVideoPictureResourceInfoKHR const &res)
     : image_view_state(dev_data.Get<ImageView>(res.imageViewBinding)),
       image_state(image_view_state ? image_view_state->image_state : nullptr),
       base_array_layer(res.baseArrayLayer),
@@ -396,6 +419,7 @@ VideoPictureID::VideoPictureID(VideoProfileDesc const &profile, VkVideoReference
 
         case VK_VIDEO_CODEC_OPERATION_DECODE_H265_BIT_KHR:
         case VK_VIDEO_CODEC_OPERATION_DECODE_AV1_BIT_KHR:
+        case VK_VIDEO_CODEC_OPERATION_DECODE_VP9_BIT_KHR:
         case VK_VIDEO_CODEC_OPERATION_ENCODE_H264_BIT_KHR:
         case VK_VIDEO_CODEC_OPERATION_ENCODE_H265_BIT_KHR:
         case VK_VIDEO_CODEC_OPERATION_ENCODE_AV1_BIT_KHR:
@@ -534,7 +558,7 @@ class RateControlStateMismatchRecorder {
     mutable std::string string_{};
 };
 
-bool VideoSessionDeviceState::ValidateRateControlState(const Device &dev_data, const VideoSession *vs_state,
+bool VideoSessionDeviceState::ValidateRateControlState(const Logger &log, const VideoSession *vs_state,
                                                        const vku::safe_VkVideoBeginCodingInfoKHR &begin_info,
                                                        const Location &loc) const {
     bool skip = false;
@@ -692,27 +716,31 @@ bool VideoSessionDeviceState::ValidateRateControlState(const Device &dev_data, c
 #undef CHECK_RC_LAYER_INFO
 
         if (mismatch_recorder.HasMismatch()) {
-            skip |= dev_data.LogError("VUID-vkCmdBeginVideoCodingKHR-pBeginInfo-08254", vs_state->Handle(), loc,
-                                      "The video encode rate control information specified when beginning the video coding scope "
-                                      "does not match the currently configured video encode rate control state for %s:\n%s",
-                                      dev_data.FormatHandle(vs_state->Handle()).c_str(), mismatch_recorder.c_str());
+            skip |= log.LogError("VUID-vkCmdBeginVideoCodingKHR-pBeginInfo-08254", vs_state->Handle(), loc,
+                                 "The video encode rate control information specified when beginning the video coding scope "
+                                 "does not match the currently configured video encode rate control state for %s:\n%s",
+                                 log.FormatHandle(vs_state->Handle()).c_str(), mismatch_recorder.c_str());
         }
 
     } else {
         if (encode_.rate_control_state.base.rateControlMode != VK_VIDEO_ENCODE_RATE_CONTROL_MODE_DEFAULT_KHR) {
-            skip |=
-                dev_data.LogError("VUID-vkCmdBeginVideoCodingKHR-pBeginInfo-08253", vs_state->Handle(), loc,
-                                  "No VkVideoEncodeRateControlInfoKHR structure was specified when beginning the "
-                                  "video coding scope but the currently set video encode rate control mode for %s is %s.",
-                                  dev_data.FormatHandle(vs_state->Handle()).c_str(),
-                                  string_VkVideoEncodeRateControlModeFlagBitsKHR(encode_.rate_control_state.base.rateControlMode));
+            skip |= log.LogError("VUID-vkCmdBeginVideoCodingKHR-pBeginInfo-08253", vs_state->Handle(), loc,
+                                 "No VkVideoEncodeRateControlInfoKHR structure was specified when beginning the "
+                                 "video coding scope but the currently set video encode rate control mode for %s is %s.",
+                                 log.FormatHandle(vs_state->Handle()).c_str(),
+                                 string_VkVideoEncodeRateControlModeFlagBitsKHR(encode_.rate_control_state.base.rateControlMode));
         }
     }
 
     return skip;
 }
 
-VideoSession::VideoSession(const Device &dev_data, VkVideoSessionKHR handle, VkVideoSessionCreateInfoKHR const *pCreateInfo,
+static VkVideoEncodeIntraRefreshModeFlagBitsKHR InitIntraRefreshMode(const VkVideoSessionCreateInfoKHR *ci) {
+    auto intra_refresh_info = vku::FindStructInPNextChain<VkVideoEncodeSessionIntraRefreshCreateInfoKHR>(ci->pNext);
+    return (intra_refresh_info) ? intra_refresh_info->intraRefreshMode : VK_VIDEO_ENCODE_INTRA_REFRESH_MODE_NONE_KHR;
+}
+
+VideoSession::VideoSession(const DeviceState &dev_data, VkVideoSessionKHR handle, VkVideoSessionCreateInfoKHR const *pCreateInfo,
                            std::shared_ptr<const VideoProfileDesc> &&profile_desc)
     : StateObject(handle, kVulkanObjectTypeVideoSessionKHR),
       safe_create_info(pCreateInfo),
@@ -722,10 +750,11 @@ VideoSession::VideoSession(const Device &dev_data, VkVideoSessionKHR handle, VkV
       memory_bindings_queried(0),
       memory_bindings_(GetMemoryBindings(dev_data, handle)),
       unbound_memory_binding_count_(static_cast<uint32_t>(memory_bindings_.size())),
+      intra_refresh_mode_(InitIntraRefreshMode(pCreateInfo)),
       device_state_mutex_(),
       device_state_(pCreateInfo->maxDpbSlots) {}
 
-VideoSession::MemoryBindingMap VideoSession::GetMemoryBindings(const Device &dev_data, VkVideoSessionKHR vs) {
+VideoSession::MemoryBindingMap VideoSession::GetMemoryBindings(const DeviceState &dev_data, VkVideoSessionKHR vs) {
     uint32_t memory_requirement_count;
     DispatchGetVideoSessionMemoryRequirementsKHR(dev_data.device, vs, &memory_requirement_count, nullptr);
 
@@ -757,6 +786,11 @@ bool VideoSession::ReferenceSetupRequested(VkVideoDecodeInfoKHR const &decode_in
             return pic_info != nullptr && pic_info->pStdPictureInfo != nullptr &&
                    pic_info->pStdPictureInfo->refresh_frame_flags != 0;
         }
+        case VK_VIDEO_CODEC_OPERATION_DECODE_VP9_BIT_KHR: {
+            auto pic_info = vku::FindStructInPNextChain<VkVideoDecodeVP9PictureInfoKHR>(decode_info.pNext);
+            return pic_info != nullptr && pic_info->pStdPictureInfo != nullptr &&
+                   pic_info->pStdPictureInfo->refresh_frame_flags != 0;
+        }
 
         default:
             return false;
@@ -842,6 +876,12 @@ VideoSessionParameters::VideoSessionParameters(VkVideoSessionParametersKHR handl
             break;
         }
 
+        case VK_VIDEO_CODEC_OPERATION_DECODE_VP9_BIT_KHR: {
+            // VP9 does not use video session parameters
+            assert(false);
+            break;
+        }
+
         case VK_VIDEO_CODEC_OPERATION_ENCODE_H264_BIT_KHR: {
             const auto encode_h264 =
                 vku::FindStructInPNextChain<VkVideoEncodeH264SessionParametersCreateInfoKHR>(create_info.pNext);
@@ -936,6 +976,12 @@ void VideoSessionParameters::Update(VkVideoSessionParametersUpdateInfoKHR const
             break;
         }
 
+        case VK_VIDEO_CODEC_OPERATION_DECODE_VP9_BIT_KHR: {
+            // VP9 does not use video session parameters
+            assert(false);
+            break;
+        }
+
         case VK_VIDEO_CODEC_OPERATION_ENCODE_H264_BIT_KHR: {
             auto add_info = vku::FindStructInPNextChain<VkVideoEncodeH264SessionParametersAddInfoKHR>(info->pNext);
             if (add_info) {
@@ -1015,4 +1061,203 @@ void VideoSessionParameters::AddEncodeH265(VkVideoEncodeH265SessionParametersAdd
     }
 }
 
+std::string string_VideoProfileDesc(const vvl::VideoProfileDesc &profile) {
+    std::stringstream ss;
+    const vvl::VideoProfileDesc::Profile &internal_profile = profile.GetProfile();
+
+    switch (internal_profile.base.videoCodecOperation) {
+        case VK_VIDEO_CODEC_OPERATION_DECODE_H264_BIT_KHR:
+            ss << "H.264 Decode";
+            break;
+        case VK_VIDEO_CODEC_OPERATION_DECODE_H265_BIT_KHR:
+            ss << "H.265 Decode";
+            break;
+        case VK_VIDEO_CODEC_OPERATION_DECODE_AV1_BIT_KHR:
+            ss << "AV1 Decode";
+            break;
+        case VK_VIDEO_CODEC_OPERATION_DECODE_VP9_BIT_KHR:
+            ss << "VP9 Decode";
+            break;
+        case VK_VIDEO_CODEC_OPERATION_ENCODE_H264_BIT_KHR:
+            ss << "H.264 Encode";
+            break;
+        case VK_VIDEO_CODEC_OPERATION_ENCODE_H265_BIT_KHR:
+            ss << "H.265 Encode";
+            break;
+        case VK_VIDEO_CODEC_OPERATION_ENCODE_AV1_BIT_KHR:
+            ss << "AV1 Encode";
+            break;
+        default:
+            assert(false);
+            ss << "<unknown video codec>";
+            break;
+    }
+
+    ss << " (";
+    switch (internal_profile.base.chromaSubsampling) {
+        case VK_VIDEO_CHROMA_SUBSAMPLING_MONOCHROME_BIT_KHR:
+            ss << "monochrome";
+            break;
+        case VK_VIDEO_CHROMA_SUBSAMPLING_420_BIT_KHR:
+            ss << "4:2:0";
+            break;
+        case VK_VIDEO_CHROMA_SUBSAMPLING_422_BIT_KHR:
+            ss << "4:2:2";
+            break;
+        case VK_VIDEO_CHROMA_SUBSAMPLING_444_BIT_KHR:
+            ss << "4:4:4";
+            break;
+        default:
+            assert(false);
+            ss << "<unknown chroma subsampling>";
+            break;
+    }
+
+    ss << " ";
+
+    auto string_video_component_bit_depth = [](const VkVideoComponentBitDepthFlagsKHR bit_depth) {
+        switch (bit_depth) {
+            case VK_VIDEO_COMPONENT_BIT_DEPTH_8_BIT_KHR:
+                return "8";
+            case VK_VIDEO_COMPONENT_BIT_DEPTH_10_BIT_KHR:
+                return "10";
+            case VK_VIDEO_COMPONENT_BIT_DEPTH_12_BIT_KHR:
+                return "12";
+            default:
+                assert(false);
+                return "<unknown bit depth>";
+        }
+    };
+
+    ss << string_video_component_bit_depth(internal_profile.base.lumaBitDepth);
+    if (internal_profile.base.lumaBitDepth != internal_profile.base.chromaBitDepth) {
+        ss << ":" << string_video_component_bit_depth(internal_profile.base.chromaBitDepth);
+    }
+    ss << "-bit) ";
+
+    auto string_std_video_h264_profile_idc = [](StdVideoH264ProfileIdc stdProfileIdc) {
+        switch (stdProfileIdc) {
+            case STD_VIDEO_H264_PROFILE_IDC_BASELINE:
+                return "Baseline";
+            case STD_VIDEO_H264_PROFILE_IDC_MAIN:
+                return "Main";
+            case STD_VIDEO_H264_PROFILE_IDC_HIGH:
+                return "High";
+            case STD_VIDEO_H264_PROFILE_IDC_HIGH_444_PREDICTIVE:
+                return "High 4:4:4 Predictive";
+            default:
+                assert(false);
+                return "<unknown H.264 profile indicator>";
+        }
+    };
+
+    auto string_video_decode_h264_picture_layout = [](VkVideoDecodeH264PictureLayoutFlagBitsKHR pictureLayout) {
+        switch (pictureLayout) {
+            case VK_VIDEO_DECODE_H264_PICTURE_LAYOUT_PROGRESSIVE_KHR:
+                return "progressive";
+            case VK_VIDEO_DECODE_H264_PICTURE_LAYOUT_INTERLACED_INTERLEAVED_LINES_BIT_KHR:
+                return "interlaced (interleaved lines)";
+            case VK_VIDEO_DECODE_H264_PICTURE_LAYOUT_INTERLACED_SEPARATE_PLANES_BIT_KHR:
+                return "interlaced (separate planes)";
+            default:
+                assert(false);
+                return "<unknown H.264 decode picture layout>";
+        }
+    };
+
+    auto string_std_video_h265_profile_idc = [](StdVideoH265ProfileIdc stdProfileIdc) {
+        switch (stdProfileIdc) {
+            case STD_VIDEO_H265_PROFILE_IDC_MAIN:
+                return "Main";
+            case STD_VIDEO_H265_PROFILE_IDC_MAIN_10:
+                return "Main 10";
+            case STD_VIDEO_H265_PROFILE_IDC_MAIN_STILL_PICTURE:
+                return "Main Still Picture";
+            case STD_VIDEO_H265_PROFILE_IDC_FORMAT_RANGE_EXTENSIONS:
+                return "Format range extensions";
+            case STD_VIDEO_H265_PROFILE_IDC_SCC_EXTENSIONS:
+                return "Screen content coding extensions";
+            default:
+                assert(false);
+                return "<unknown H.265 profile indicator>";
+        }
+    };
+
+    auto string_std_video_av1_profile_idc = [](StdVideoAV1Profile stdProfileIdc) {
+        switch (stdProfileIdc) {
+            case STD_VIDEO_AV1_PROFILE_MAIN:
+                return "Main";
+            case STD_VIDEO_AV1_PROFILE_HIGH:
+                return "High";
+            case STD_VIDEO_AV1_PROFILE_PROFESSIONAL:
+                return "Professional";
+            default:
+                assert(false);
+                return "<unknown AV1 profile>";
+        }
+    };
+
+    auto string_std_video_vp9_profile = [](StdVideoVP9Profile stdProfile) {
+        switch (stdProfile) {
+            case STD_VIDEO_VP9_PROFILE_0:
+                return "Profile 0";
+            case STD_VIDEO_VP9_PROFILE_1:
+                return "Profile 1";
+            case STD_VIDEO_VP9_PROFILE_2:
+                return "Profile 2";
+            case STD_VIDEO_VP9_PROFILE_3:
+                return "Profile 3";
+            default:
+                assert(false);
+                return "<unknown VP9 profile>";
+        }
+    };
+
+    switch (internal_profile.base.videoCodecOperation) {
+        case VK_VIDEO_CODEC_OPERATION_DECODE_H264_BIT_KHR:
+            ss << string_std_video_h264_profile_idc(internal_profile.decode_h264.stdProfileIdc) << " "
+               << string_video_decode_h264_picture_layout(internal_profile.decode_h264.pictureLayout);
+            break;
+        case VK_VIDEO_CODEC_OPERATION_ENCODE_H264_BIT_KHR:
+            ss << string_std_video_h264_profile_idc(internal_profile.encode_h264.stdProfileIdc);
+            break;
+        case VK_VIDEO_CODEC_OPERATION_DECODE_H265_BIT_KHR:
+            ss << string_std_video_h265_profile_idc(internal_profile.decode_h265.stdProfileIdc);
+            break;
+        case VK_VIDEO_CODEC_OPERATION_ENCODE_H265_BIT_KHR:
+            ss << string_std_video_h265_profile_idc(internal_profile.encode_h265.stdProfileIdc);
+            break;
+        case VK_VIDEO_CODEC_OPERATION_DECODE_AV1_BIT_KHR:
+            ss << string_std_video_av1_profile_idc(internal_profile.decode_av1.stdProfile)
+               << (internal_profile.decode_av1.filmGrainSupport ? " with film grain support" : " without film grain support");
+            break;
+        case VK_VIDEO_CODEC_OPERATION_ENCODE_AV1_BIT_KHR:
+            ss << string_std_video_av1_profile_idc(internal_profile.decode_av1.stdProfile);
+            break;
+        case VK_VIDEO_CODEC_OPERATION_DECODE_VP9_BIT_KHR:
+            ss << string_std_video_vp9_profile(internal_profile.decode_vp9.stdProfile);
+            break;
+        default:
+            assert(false);
+            ss << "<unknown video profile>";
+            break;
+    }
+
+    return ss.str();
+}
+
+std::string string_SupportedVideoProfiles(const SupportedVideoProfiles &profiles) {
+    std::stringstream ss;
+
+    if (profiles.size() != 0) {
+        for (const auto &profile : profiles) {
+            ss << "\t" << string_VideoProfileDesc(*profile) << "\n";
+        }
+    } else {
+        ss << "\tNone\n";
+    }
+
+    return ss.str();
+}
+
 }  // namespace vvl
