@@ -16,6 +16,7 @@
  * limitations under the License.
  */
 
+#include "error_message/error_location.h"
 #include "stateless/stateless_validation.h"
 
 namespace stateless {
@@ -68,6 +69,13 @@ static const uint8_t kUtF8ThreeByteMask = 0xF8;
 static const uint8_t kUtF8DataByteCode = 0x80;
 static const uint8_t kUtF8DataByteMask = 0xC0;
 
+typedef enum VkStringErrorFlagBits {
+    VK_STRING_ERROR_NONE = 0x00000000,
+    VK_STRING_ERROR_LENGTH = 0x00000001,
+    VK_STRING_ERROR_BAD_DATA = 0x00000002,
+} VkStringErrorFlagBits;
+typedef VkFlags VkStringErrorFlags;
+
 static VkStringErrorFlags ValidateVkString(const int max_length, const char *utf8) {
     VkStringErrorFlags result = VK_STRING_ERROR_NONE;
     int num_char_bytes = 0;
@@ -238,10 +246,10 @@ bool Context::ValidateStructPnext(const Location &loc, const void *next, size_t
 
         const Location pNext_loc = loc.dot(Field::pNext);
         if ((allowed_type_count == 0) && (GetCustomStypeInfo().empty())) {
-            std::string message = "must be NULL. ";
+            std::string message = "must be NULL.\n%s\n";
             message += disclaimer;
-            skip |=
-                log.LogError(pnext_vuid, error_obj.handle, pNext_loc, message.c_str(), header_version, pNext_loc.Fields().c_str());
+            skip |= log.LogError(pnext_vuid, error_obj.handle, pNext_loc, message.c_str(),
+                                 PrintPNextChain(Struct::Empty, next).c_str(), header_version, pNext_loc.Fields().c_str());
         } else {
             const VkStructureType *start = allowed_types;
             const VkStructureType *end = allowed_types + allowed_type_count;
@@ -249,13 +257,12 @@ bool Context::ValidateStructPnext(const Location &loc, const void *next, size_t
 
             while (current != nullptr) {
                 if ((loc.function != Func::vkCreateInstance || (current->sType != VK_STRUCTURE_TYPE_LOADER_INSTANCE_CREATE_INFO)) &&
-                        (loc.function != Func::vkCreateDevice || (current->sType != VK_STRUCTURE_TYPE_LOADER_DEVICE_CREATE_INFO))) {
-                    std::string type_name = string_VkStructureType(current->sType);
+                    (loc.function != Func::vkCreateDevice || (current->sType != VK_STRUCTURE_TYPE_LOADER_DEVICE_CREATE_INFO))) {
                     if (unique_stype_check.find(current->sType) != unique_stype_check.end() && !IsDuplicatePnext(current->sType)) {
                         // stype_vuid will only be null if there are no listed pNext and will hit disclaimer check
-                        skip |=
-                            log.LogError(stype_vuid, error_obj.handle, pNext_loc,
-                                    "chain contains duplicate structure types: %s appears multiple times.", type_name.c_str());
+                        skip |= log.LogError(stype_vuid, error_obj.handle, pNext_loc,
+                                             "chain contains duplicate structure types: %s appears multiple times.\n%s",
+                                             string_VkStructureName(current->sType), PrintPNextChain(loc.structure, next).c_str());
                     } else {
                         unique_stype_check.insert(current->sType);
                     }
@@ -270,17 +277,20 @@ bool Context::ValidateStructPnext(const Location &loc, const void *next, size_t
                     }
                     if (!custom) {
                         if (std::find(start, end, current->sType) == end) {
-                            // String returned by string_VkStructureType for an unrecognized type.
+                            const std::string type_name = string_VkStructureType(current->sType);
                             if (type_name.compare("Unhandled VkStructureType") == 0) {
-                                std::string message = "chain includes a structure with unknown VkStructureType (%" PRIu32 "). ";
+                                std::string message =
+                                    "chain includes a structure with unknown VkStructureType (%" PRIu32 ").\n%s\n";
                                 message += disclaimer;
                                 skip |= log.LogError(pnext_vuid, error_obj.handle, pNext_loc, message.c_str(), current->sType,
-                                        header_version, pNext_loc.Fields().c_str());
+                                                     PrintPNextChain(Struct::Empty, next).c_str(), header_version,
+                                                     pNext_loc.Fields().c_str());
                             } else {
-                                std::string message = "chain includes a structure with unexpected VkStructureType %s. ";
+                                std::string message = "chain includes a structure with unexpected VkStructureType %s.\n%s\n";
                                 message += disclaimer;
                                 skip |= log.LogError(pnext_vuid, error_obj.handle, pNext_loc, message.c_str(), type_name.c_str(),
-                                        header_version, pNext_loc.Fields().c_str());
+                                                     PrintPNextChain(Struct::Empty, next).c_str(), header_version,
+                                                     pNext_loc.Fields().c_str());
                             }
                         }
                         // Send Location without pNext field so the pNext() connector can be used
@@ -344,6 +354,14 @@ bool Context::ValidateReservedFlags(const Location &loc, VkFlags value, const ch
     return skip;
 }
 
+bool Context::ValidateReservedFlags(const Location &loc, VkFlags64 value, const char *vuid) const {
+    bool skip = false;
+    if (value != 0) {
+        skip |= log.LogError(vuid, error_obj.handle, loc, "is %" PRIu64 ", but must be 0.", value);
+    }
+    return skip;
+}
+
 // helper to implement validation of both 32 bit and 64 bit flags.
 template <typename FlagTypedef>
 bool Context::ValidateFlagsImplementation(const Location &loc, vvl::FlagBitmask flag_bitmask, FlagTypedef all_flags,
