@@ -15,11 +15,13 @@
  * limitations under the License.
  */
 
+#include <vulkan/utility/vk_format_utils.h>
 #include "sync/sync_renderpass.h"
 #include "sync/sync_validation.h"
 #include "sync/sync_op.h"
 #include "sync/sync_image.h"
 #include "state_tracker/render_pass_state.h"
+#include "state_tracker/pipeline_state.h"
 
 // Action for validating resolve operations
 class ValidateResolveAction {
@@ -167,11 +169,11 @@ bool RenderPassAccessContext::ValidateLayoutTransitions(const CommandBufferAcces
             const SyncValidator &sync_state = cb_context.GetSyncState();
             const Location loc(command);
 
-            const syncval_state::ImageViewState *attachment_view = attachment_views[transition.attachment].GetViewState();
+            const vvl::ImageView *attachment_view = attachment_views[transition.attachment].GetViewState();
             std::stringstream ss;
             ss << "in subpass " << subpass << " on attachment " << transition.attachment << " (";
             ss << sync_state.FormatHandle(attachment_view->Handle());
-            ss << ", " << sync_state.FormatHandle(attachment_view->GetImageState()->Handle());
+            ss << ", " << sync_state.FormatHandle(attachment_view->image_state->Handle());
             ss << ", oldLayout " << string_VkImageLayout(transition.old_layout);
             ss << ", newLayout " << string_VkImageLayout(transition.new_layout);
             ss << ")";
@@ -294,20 +296,21 @@ bool RenderPassAccessContext::ValidateStoreOperation(const CommandBufferAccessCo
             bool checked_stencil = false;
             if (is_color) {
                 hazard = CurrentContext().DetectHazard(view_gen, AttachmentViewGen::Gen::kRenderArea,
-                                                       SYNC_COLOR_ATTACHMENT_OUTPUT_COLOR_ATTACHMENT_WRITE, SyncOrdering::kRaster);
+                                                       SYNC_COLOR_ATTACHMENT_OUTPUT_COLOR_ATTACHMENT_WRITE, SyncOrdering::kRaster,
+                                                       SyncFlag::kStoreOp);
                 aspect = "color";
             } else {
                 const bool stencil_op_stores = ci.stencilStoreOp != VK_ATTACHMENT_STORE_OP_NONE;
                 if (has_depth && store_op_stores) {
                     hazard = CurrentContext().DetectHazard(view_gen, AttachmentViewGen::Gen::kDepthOnlyRenderArea,
                                                            SYNC_LATE_FRAGMENT_TESTS_DEPTH_STENCIL_ATTACHMENT_WRITE,
-                                                           SyncOrdering::kRaster);
+                                                           SyncOrdering::kRaster, SyncFlag::kStoreOp);
                     aspect = "depth";
                 }
                 if (!hazard.IsHazard() && has_stencil && stencil_op_stores) {
                     hazard = CurrentContext().DetectHazard(view_gen, AttachmentViewGen::Gen::kStencilOnlyRenderArea,
                                                            SYNC_LATE_FRAGMENT_TESTS_DEPTH_STENCIL_ATTACHMENT_WRITE,
-                                                           SyncOrdering::kRaster);
+                                                           SyncOrdering::kRaster, SyncFlag::kStoreOp);
                     aspect = "stencil";
                     checked_stencil = true;
                 }
@@ -459,18 +462,19 @@ void RenderPassAccessContext::UpdateAttachmentStoreAccess(const vvl::RenderPass
 
             if (is_color && store_op_stores) {
                 access_context.UpdateAccessState(view_gen, AttachmentViewGen::Gen::kRenderArea,
-                                                 SYNC_COLOR_ATTACHMENT_OUTPUT_COLOR_ATTACHMENT_WRITE, SyncOrdering::kRaster, tag);
+                                                 SYNC_COLOR_ATTACHMENT_OUTPUT_COLOR_ATTACHMENT_WRITE, SyncOrdering::kRaster, tag,
+                                                 SyncFlag::kStoreOp);
             } else {
                 if (has_depth && store_op_stores) {
                     access_context.UpdateAccessState(view_gen, AttachmentViewGen::Gen::kDepthOnlyRenderArea,
                                                      SYNC_LATE_FRAGMENT_TESTS_DEPTH_STENCIL_ATTACHMENT_WRITE, SyncOrdering::kRaster,
-                                                     tag);
+                                                     tag, SyncFlag::kStoreOp);
                 }
                 const bool stencil_op_stores = ci.stencilStoreOp != VK_ATTACHMENT_STORE_OP_NONE;
                 if (has_stencil && stencil_op_stores) {
                     access_context.UpdateAccessState(view_gen, AttachmentViewGen::Gen::kStencilOnlyRenderArea,
                                                      SYNC_LATE_FRAGMENT_TESTS_DEPTH_STENCIL_ATTACHMENT_WRITE, SyncOrdering::kRaster,
-                                                     tag);
+                                                     tag, SyncFlag::kStoreOp);
                 }
             }
         }
@@ -512,9 +516,8 @@ void RenderPassAccessContext::RecordLayoutTransitions(const vvl::RenderPass &rp_
 // TODO: SyncError reporting places in this function are not covered by the tests.
 bool RenderPassAccessContext::ValidateDrawSubpassAttachment(const CommandBufferAccessContext &cb_context, vvl::Func command) const {
     bool skip = false;
-    const auto lv_bind_point = ConvertToLvlBindPoint(VK_PIPELINE_BIND_POINT_GRAPHICS);
     const vvl::CommandBuffer &cmd_buffer = cb_context.GetCBState();
-    const auto &last_bound_state = cmd_buffer.lastBound[lv_bind_point];
+    const auto &last_bound_state = cmd_buffer.GetLastBoundGraphics();
     const auto *pipe = last_bound_state.pipeline_state;
 
     if (!pipe || pipe->RasterizationDisabled()) {
@@ -610,8 +613,7 @@ bool RenderPassAccessContext::ValidateDrawSubpassAttachment(const CommandBufferA
 }
 
 void RenderPassAccessContext::RecordDrawSubpassAttachment(const vvl::CommandBuffer &cmd_buffer, const ResourceUsageTag tag) {
-    const auto lv_bind_point = ConvertToLvlBindPoint(VK_PIPELINE_BIND_POINT_GRAPHICS);
-    const auto &last_bound_state = cmd_buffer.lastBound[lv_bind_point];
+    const auto &last_bound_state = cmd_buffer.GetLastBoundGraphics();
     const auto *pipe = last_bound_state.pipeline_state;
     if (!pipe || pipe->RasterizationDisabled()) return;
 
@@ -668,8 +670,7 @@ void RenderPassAccessContext::RecordDrawSubpassAttachment(const vvl::CommandBuff
     }
 }
 
-const syncval_state::ImageViewState *RenderPassAccessContext::GetClearAttachmentView(
-    const VkClearAttachment &clear_attachment) const {
+const vvl::ImageView *RenderPassAccessContext::GetClearAttachmentView(const VkClearAttachment &clear_attachment) const {
     const auto &subpass = rp_state_->create_info.pSubpasses[current_subpass_];
     uint32_t attachment_index = VK_ATTACHMENT_UNUSED;
     if (clear_attachment.aspectMask & VK_IMAGE_ASPECT_COLOR_BIT) {
@@ -760,7 +761,7 @@ bool RenderPassAccessContext::ValidateFinalSubpassLayoutTransitions(const Comman
             std::stringstream ss;
             ss << "on attachment " << transition.attachment << " (";
             ss << sync_state.FormatHandle(view_gen.GetViewState()->Handle());
-            ss << ", " << sync_state.FormatHandle(view_gen.GetViewState()->GetImageState()->Handle());
+            ss << ", " << sync_state.FormatHandle(view_gen.GetViewState()->image_state->Handle());
             ss << ", oldLayout " << string_VkImageLayout(transition.old_layout);
             ss << ", newLayout " << string_VkImageLayout(transition.new_layout);
             ss << ")";
@@ -804,21 +805,21 @@ void RenderPassAccessContext::RecordLoadOperations(const ResourceUsageTag tag) {
                 const SyncAccessIndex load_op = ColorLoadUsage(ci.loadOp);
                 if (load_op != SYNC_ACCESS_INDEX_NONE) {
                     subpass_context.UpdateAccessState(view_gen, AttachmentViewGen::Gen::kRenderArea, load_op,
-                                                      SyncOrdering::kColorAttachment, tag);
+                                                      SyncOrdering::kColorAttachment, tag, SyncFlag::kLoadOp);
                 }
             } else {
                 if (has_depth) {
                     const SyncAccessIndex load_op = DepthStencilLoadUsage(ci.loadOp);
                     if (load_op != SYNC_ACCESS_INDEX_NONE) {
                         subpass_context.UpdateAccessState(view_gen, AttachmentViewGen::Gen::kDepthOnlyRenderArea, load_op,
-                                                          SyncOrdering::kDepthStencilAttachment, tag);
+                                                          SyncOrdering::kDepthStencilAttachment, tag, SyncFlag::kLoadOp);
                     }
                 }
                 if (has_stencil) {
                     const SyncAccessIndex load_op = DepthStencilLoadUsage(ci.stencilLoadOp);
                     if (load_op != SYNC_ACCESS_INDEX_NONE) {
                         subpass_context.UpdateAccessState(view_gen, AttachmentViewGen::Gen::kStencilOnlyRenderArea, load_op,
-                                                          SyncOrdering::kDepthStencilAttachment, tag);
+                                                          SyncOrdering::kDepthStencilAttachment, tag, SyncFlag::kLoadOp);
                     }
                 }
             }
@@ -826,7 +827,7 @@ void RenderPassAccessContext::RecordLoadOperations(const ResourceUsageTag tag) {
     }
 }
 AttachmentViewGenVector RenderPassAccessContext::CreateAttachmentViewGen(
-    const VkRect2D &render_area, const std::vector<const syncval_state::ImageViewState *> &attachment_views) {
+    const VkRect2D &render_area, const std::vector<const vvl::ImageView *> &attachment_views) {
     AttachmentViewGenVector view_gens;
     VkExtent3D extent = CastTo3D(render_area.extent);
     VkOffset3D offset = CastTo3D(render_area.offset);
@@ -838,7 +839,7 @@ AttachmentViewGenVector RenderPassAccessContext::CreateAttachmentViewGen(
 }
 RenderPassAccessContext::RenderPassAccessContext(const vvl::RenderPass &rp_state, const VkRect2D &render_area,
                                                  VkQueueFlags queue_flags,
-                                                 const std::vector<const syncval_state::ImageViewState *> &attachment_views,
+                                                 const std::vector<const vvl::ImageView *> &attachment_views,
                                                  const AccessContext *external_context)
     : rp_state_(&rp_state), render_area_(render_area), current_subpass_(0U), attachment_views_() {
     // Add this for all subpasses here so that they exist during next subpass validation
@@ -929,9 +930,8 @@ syncval_state::DynamicRenderingInfo::DynamicRenderingInfo(const SyncValidator &s
     }
 }
 
-const syncval_state::ImageViewState *syncval_state::DynamicRenderingInfo::GetClearAttachmentView(
-    const VkClearAttachment &clear_attachment) const {
-    const syncval_state::ImageViewState *attachment_view = nullptr;
+const vvl::ImageView *syncval_state::DynamicRenderingInfo::GetClearAttachmentView(const VkClearAttachment &clear_attachment) const {
+    const vvl::ImageView *attachment_view = nullptr;
     if (clear_attachment.aspectMask & VK_IMAGE_ASPECT_COLOR_BIT) {
         if (clear_attachment.colorAttachment < info.colorAttachmentCount) {
             attachment_view = attachments[clear_attachment.colorAttachment].view.get();
@@ -949,26 +949,27 @@ syncval_state::DynamicRenderingInfo::Attachment::Attachment(const SyncValidator
                                                             const vku::safe_VkRenderingAttachmentInfo &attachment_info,
                                                             AttachmentType type_, const VkOffset3D &offset,
                                                             const VkExtent3D &extent)
-    : info(attachment_info), view(state.Get<ImageViewState>(attachment_info.imageView)), view_gen(), type(type_) {
+    : info(attachment_info), view(state.Get<vvl::ImageView>(attachment_info.imageView)), view_gen(), type(type_) {
     if (view) {
         if (type == AttachmentType::kColor) {
-            view_gen = view->MakeImageRangeGen(offset, extent);
+            view_gen = syncval_state::MakeImageRangeGen(*view, offset, extent);
         } else if (type == AttachmentType::kDepth) {
-            view_gen = view->MakeImageRangeGen(offset, extent, VK_IMAGE_ASPECT_DEPTH_BIT);
+            view_gen = syncval_state::MakeImageRangeGen(*view, offset, extent, VK_IMAGE_ASPECT_DEPTH_BIT);
         } else {
-            view_gen = view->MakeImageRangeGen(offset, extent, VK_IMAGE_ASPECT_STENCIL_BIT);
+            view_gen = syncval_state::MakeImageRangeGen(*view, offset, extent, VK_IMAGE_ASPECT_STENCIL_BIT);
         }
 
         if (info.resolveImageView != VK_NULL_HANDLE && (info.resolveMode != VK_RESOLVE_MODE_NONE)) {
-            resolve_view = state.Get<ImageViewState>(info.resolveImageView);
+            resolve_view = state.Get<vvl::ImageView>(info.resolveImageView);
             if (resolve_view) {
                 if (type == AttachmentType::kColor) {
-                    resolve_gen.emplace(resolve_view->MakeImageRangeGen(offset, extent));
+                    resolve_gen.emplace(syncval_state::MakeImageRangeGen(*resolve_view, offset, extent));
                 } else if (type == AttachmentType::kDepth) {
                     // Only the depth aspect
-                    resolve_gen.emplace(resolve_view->MakeImageRangeGen(offset, extent, VK_IMAGE_ASPECT_DEPTH_BIT));
+                    resolve_gen.emplace(syncval_state::MakeImageRangeGen(*resolve_view, offset, extent, VK_IMAGE_ASPECT_DEPTH_BIT));
                 } else {
-                    resolve_gen.emplace(resolve_view->MakeImageRangeGen(offset, extent, VK_IMAGE_ASPECT_STENCIL_BIT));
+                    resolve_gen.emplace(
+                        syncval_state::MakeImageRangeGen(*resolve_view, offset, extent, VK_IMAGE_ASPECT_STENCIL_BIT));
                 }
             }
         }
