@@ -1,5 +1,5 @@
-/* Copyright (c) 2023 Valve Corporation
- * Copyright (c) 2023 LunarG, Inc.
+/* Copyright (c) 2024 Valve Corporation
+ * Copyright (c) 2024 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,16 +16,26 @@
 
 #pragma once
 
-#include "vulkan/vulkan.h"
+#include <vulkan/vulkan_core.h>
 
 namespace rt {
+
+enum class BuildType { Device, Host };
+
 // Compute scratch buffer size the idiomatic way.
 // Note: range_infos must be an array of build_info.geometryCount elements
-VkDeviceSize ComputeScratchSize(const VkDevice device, const VkAccelerationStructureBuildGeometryInfoKHR &build_info,
+VkDeviceSize ComputeScratchSize(BuildType build_type, const VkDevice device,
+                                const VkAccelerationStructureBuildGeometryInfoKHR &build_info,
                                 const VkAccelerationStructureBuildRangeInfoKHR *range_infos);
 
 // Compute acceleration structure size the idiomatic way.
 // Note: range_infos must be an array of build_info.geometryCount elements
-VkDeviceSize ComputeAccelerationStructureSize(const VkDevice device, const VkAccelerationStructureBuildGeometryInfoKHR &build_info,
+VkDeviceSize ComputeAccelerationStructureSize(BuildType build_type, const VkDevice device,
+                                              const VkAccelerationStructureBuildGeometryInfoKHR &build_info,
                                               const VkAccelerationStructureBuildRangeInfoKHR *range_infos);
-}
\ No newline at end of file
+
+inline const VkAccelerationStructureGeometryKHR &GetGeometry(const VkAccelerationStructureBuildGeometryInfoKHR &info,
+                                                             uint32_t geometry_i) {
+    return info.pGeometries ? info.pGeometries[geometry_i] : *info.ppGeometries[geometry_i];
+}
+}  // namespace rt
