@@ -373,9 +373,6 @@ def RunGenerators(api: str, registry: str, grammar: str, directory: str, styleFi
         # Parse the specified registry XML into an ElementTree object
         tree = ElementTree.parse(registry)
 
-        # Filter out extensions that are not on the API list
-        [exts.remove(e) for exts in tree.findall('extensions') for e in exts.findall('extension') if (sup := e.get('supported')) is not None and all(api not in sup.split(',') for api in apiList)]
-
         # Load the XML tree into the registry object
         reg.loadElementTree(tree)
 
@@ -410,40 +407,8 @@ def main(argv):
     # The shaders requires glslangvalidator, so they are updated manually with generate_spirv when needed
     verify_exclude = [
         '.clang-format',
-        'validation_cmd_copy_buffer_to_image_comp.h',
-        'validation_cmd_copy_buffer_to_image_comp.cpp',
-        'validation_cmd_dispatch_comp.h',
-        'validation_cmd_dispatch_comp.cpp',
-        'validation_cmd_count_buffer_comp.h',
-        'validation_cmd_count_buffer_comp.cpp',
-        'validation_cmd_first_instance_comp.h',
-        'validation_cmd_first_instance_comp.cpp',
-        'validation_cmd_draw_indexed_comp.h',
-        'validation_cmd_draw_indexed_comp.cpp',
-        'validation_cmd_draw_indexed_indirect_index_buffer_comp.h',
-        'validation_cmd_draw_indexed_indirect_index_buffer_comp.cpp',
-        'validation_cmd_draw_indexed_indirect_vertex_buffer_comp.h',
-        'validation_cmd_draw_indexed_indirect_vertex_buffer_comp.cpp',
-        'validation_cmd_draw_mesh_indirect_comp.h',
-        'validation_cmd_draw_mesh_indirect_comp.cpp',
-        'validation_cmd_trace_rays_rgen.h',
-        'validation_cmd_trace_rays_rgen.cpp',
-        'instrumentation_buffer_device_address_comp.h',
-        'instrumentation_buffer_device_address_comp.cpp',
-        'instrumentation_descriptor_indexing_oob_bindless_comp.h',
-        'instrumentation_descriptor_indexing_oob_bindless_comp.cpp',
-        'instrumentation_descriptor_indexing_oob_non_bindless_comp.h',
-        'instrumentation_descriptor_indexing_oob_non_bindless_comp.cpp',
-        'instrumentation_descriptor_class_general_buffer_comp.h',
-        'instrumentation_descriptor_class_general_buffer_comp.cpp',
-        'instrumentation_descriptor_class_texel_buffer_comp.h',
-        'instrumentation_descriptor_class_texel_buffer_comp.cpp',
-        'instrumentation_ray_query_comp.h',
-        'instrumentation_ray_query_comp.cpp',
-        'instrumentation_post_process_descriptor_index_comp.h',
-        'instrumentation_post_process_descriptor_index_comp.cpp',
-        'instrumentation_vertex_attribute_fetch_oob_vert.cpp',
-        'instrumentation_vertex_attribute_fetch_oob_vert.h',
+        'gpuav_offline_spirv.h',
+        'gpuav_offline_spirv.cpp',
         'feature_requirements_helper.h', # https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/8969
         'feature_requirements_helper.cpp'
     ]
