@@ -1,6 +1,6 @@
 #!/usr/bin/python3 -i
 #
-# Copyright (c) 2023-2024 The Khronos Group Inc.
+# Copyright (c) 2023-2025 The Khronos Group Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,7 @@
 # limitations under the License.
 
 import os
-from generators.base_generator import BaseGenerator
+from base_generator import BaseGenerator
 
 # TODO - Get in vk.xml
 # This is a representation of all the dynamic state information
@@ -34,7 +34,8 @@ dynamic_state_map = {
         "dependency" : ["rasterizerDiscardEnable", "depthBiasEnable"]
     },
     "VK_DYNAMIC_STATE_BLEND_CONSTANTS" : {
-        "command" : ["vkCmdSetBlendConstants"]
+        "command" : ["vkCmdSetBlendConstants"],
+        "dependency" : ["rasterizerDiscardEnable"]
     },
     "VK_DYNAMIC_STATE_DEPTH_BOUNDS" : {
         "command" : ["vkCmdSetDepthBounds"],
@@ -78,7 +79,7 @@ dynamic_state_map = {
     },
     "VK_DYNAMIC_STATE_DEPTH_WRITE_ENABLE" : {
         "command" : ["vkCmdSetDepthWriteEnable"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable", "depthTestEnable"]
     },
     "VK_DYNAMIC_STATE_DEPTH_COMPARE_OP" : {
         "command" : ["vkCmdSetDepthCompareOp"],
@@ -160,7 +161,8 @@ dynamic_state_map = {
         "dependency" : ["rasterizerDiscardEnable", "logicOpEnable"]
     },
     "VK_DYNAMIC_STATE_COLOR_WRITE_ENABLE_EXT" : {
-        "command" : ["vkCmdSetColorWriteEnableEXT"]
+        "command" : ["vkCmdSetColorWriteEnableEXT"],
+        "dependency" : ["rasterizerDiscardEnable"]
     },
     "VK_DYNAMIC_STATE_TESSELLATION_DOMAIN_ORIGIN_EXT" : {
         "command" : ["vkCmdSetTessellationDomainOriginEXT"]
@@ -190,13 +192,16 @@ dynamic_state_map = {
         "dependency" : ["rasterizerDiscardEnable"]
     },
     "VK_DYNAMIC_STATE_COLOR_BLEND_ENABLE_EXT" : {
-        "command" : ["vkCmdSetColorBlendEnableEXT"]
+        "command" : ["vkCmdSetColorBlendEnableEXT"],
+        "dependency" : ["rasterizerDiscardEnable"]
     },
     "VK_DYNAMIC_STATE_COLOR_BLEND_EQUATION_EXT" : {
-        "command" : ["vkCmdSetColorBlendEquationEXT"]
+        "command" : ["vkCmdSetColorBlendEquationEXT"],
+        "dependency" : ["rasterizerDiscardEnable"]
     },
     "VK_DYNAMIC_STATE_COLOR_WRITE_MASK_EXT" : {
-        "command" : ["vkCmdSetColorWriteMaskEXT"]
+        "command" : ["vkCmdSetColorWriteMaskEXT"],
+        "dependency" : ["rasterizerDiscardEnable"]
     },
     "VK_DYNAMIC_STATE_RASTERIZATION_STREAM_EXT" : {
         "command" : ["vkCmdSetRasterizationStreamEXT"]
@@ -218,15 +223,18 @@ dynamic_state_map = {
     },
     "VK_DYNAMIC_STATE_COLOR_BLEND_ADVANCED_EXT" : {
         "command" : ["vkCmdSetColorBlendAdvancedEXT"],
+        "dependency" : ["rasterizerDiscardEnable"]
     },
     "VK_DYNAMIC_STATE_PROVOKING_VERTEX_MODE_EXT" : {
         "command" : ["vkCmdSetProvokingVertexModeEXT"],
     },
     "VK_DYNAMIC_STATE_LINE_RASTERIZATION_MODE_EXT" : {
         "command" : ["vkCmdSetLineRasterizationModeEXT"],
+        "dependency" : ["rasterizerDiscardEnable"]
     },
     "VK_DYNAMIC_STATE_LINE_STIPPLE_ENABLE_EXT" : {
         "command" : ["vkCmdSetLineStippleEnableEXT"],
+        "dependency" : ["rasterizerDiscardEnable"]
     },
     "VK_DYNAMIC_STATE_DEPTH_CLIP_NEGATIVE_ONE_TO_ONE_EXT" : {
         "command" : ["vkCmdSetDepthClipNegativeOneToOneEXT"],
@@ -291,8 +299,8 @@ class DynamicStateOutputGenerator(BaseGenerator):
 
             /***************************************************************************
             *
-            * Copyright (c) 2023-2024 Valve Corporation
-            * Copyright (c) 2023-2024 LunarG, Inc.
+            * Copyright (c) 2023-2025 Valve Corporation
+            * Copyright (c) 2023-2025 LunarG, Inc.
             *
             * Licensed under the Apache License, Version 2.0 (the "License");
             * you may not use this file except in compliance with the License.
