@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 2023-2024 The Khronos Group Inc.
- * Copyright (c) 2023-2024 Valve Corporation
- * Copyright (c) 2023-2024 LunarG, Inc.
+ * Copyright (c) 2023-2025 The Khronos Group Inc.
+ * Copyright (c) 2023-2025 Valve Corporation
+ * Copyright (c) 2023-2025 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -11,6 +11,8 @@
  */
 
 #include "descriptor_helper.h"
+#include <vulkan/utility/vk_struct_helper.hpp>
+#include "binding.h"
 
 OneOffDescriptorSet::OneOffDescriptorSet(vkt::Device *device, const std::vector<VkDescriptorSetLayoutBinding> &bindings,
                                          VkDescriptorSetLayoutCreateFlags layout_flags, void *layout_pnext,
@@ -68,7 +70,7 @@ OneOffDescriptorIndexingSet::OneOffDescriptorIndexingSet(vkt::Device *device, co
     ds_layout_ci.flags = layout_flags;
     ds_layout_ci.bindingCount = ds_layout_bindings.size();
     ds_layout_ci.pBindings = ds_layout_bindings.data();
-    layout_.init(*device, ds_layout_ci);
+    layout_.Init(*device, ds_layout_ci);
 
     VkDescriptorPoolCreateInfo pool_ci = vku::InitStructHelper(create_pool_pnext);
     pool_ci.flags = pool_flags;
@@ -87,9 +89,30 @@ OneOffDescriptorIndexingSet::OneOffDescriptorIndexingSet(vkt::Device *device, co
     }
 }
 
+OneOffDescriptorSet::OneOffDescriptorSet(OneOffDescriptorSet &&rhs) noexcept { *this = std::move(rhs); }
+
+OneOffDescriptorSet &OneOffDescriptorSet::operator=(OneOffDescriptorSet &&rhs) noexcept {
+    device_ = rhs.device_;
+    rhs.device_ = nullptr;
+
+    pool_ = rhs.pool_;
+    rhs.pool_ = VK_NULL_HANDLE;
+
+    layout_ = std::move(rhs.layout_);
+
+    set_ = rhs.set_;
+    rhs.set_ = VK_NULL_HANDLE;
+
+    resource_infos = std::move(rhs.resource_infos);
+    descriptor_writes = std::move(rhs.descriptor_writes);
+    return *this;
+}
+
 OneOffDescriptorSet::~OneOffDescriptorSet() {
-    // No need to destroy set-- it's going away with the pool.
-    vk::DestroyDescriptorPool(device_->handle(), pool_, nullptr);
+    if (pool_ != VK_NULL_HANDLE) {
+        // No need to destroy set-- it's going away with the pool.
+        vk::DestroyDescriptorPool(device_->handle(), pool_, nullptr);
+    }
 }
 
 bool OneOffDescriptorSet::Initialized() { return pool_ != VK_NULL_HANDLE && layout_.initialized() && set_ != VK_NULL_HANDLE; }
@@ -175,3 +198,53 @@ void OneOffDescriptorSet::UpdateDescriptorSets() {
     }
     vk::UpdateDescriptorSets(device_->handle(), descriptor_writes.size(), descriptor_writes.data(), 0, NULL);
 }
+
+namespace vkt {
+DescriptorGetInfo::DescriptorGetInfo(VkSampler *sampler) {
+    get_info.type = VK_DESCRIPTOR_TYPE_SAMPLER;
+    get_info.data.pSampler = sampler;
+}
+
+DescriptorGetInfo::DescriptorGetInfo(VkDeviceAddress address) {
+    get_info.type = VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_KHR;
+    get_info.data.accelerationStructure = address;
+}
+
+DescriptorGetInfo::DescriptorGetInfo(VkDescriptorType type, VkSampler sampler, VkImageView image_view, VkImageLayout image_layout) {
+    image_info = {sampler, image_view, image_layout};
+    get_info.type = type;
+    if (type == VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER) {
+        get_info.data.pCombinedImageSampler = &image_info;
+    } else if (type == VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT) {
+        get_info.data.pInputAttachmentImage = &image_info;
+    } else if (type == VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE) {
+        get_info.data.pSampledImage = &image_info;
+    } else if (type == VK_DESCRIPTOR_TYPE_STORAGE_IMAGE) {
+        get_info.data.pStorageImage = &image_info;
+    } else {
+        assert(false);
+    }
+}
+
+DescriptorGetInfo::DescriptorGetInfo(VkDescriptorType type, VkDeviceAddress address, VkDeviceSize range, VkFormat format) {
+    address_info = vku::InitStructHelper();
+    address_info.address = address;
+    address_info.range = range;
+    address_info.format = format;
+    get_info.type = type;
+    if (type == VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER) {
+        get_info.data.pUniformTexelBuffer = &address_info;
+    } else if (type == VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER) {
+        get_info.data.pStorageTexelBuffer = &address_info;
+    } else if (type == VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER) {
+        get_info.data.pUniformBuffer = &address_info;
+    } else if (type == VK_DESCRIPTOR_TYPE_STORAGE_BUFFER) {
+        get_info.data.pStorageBuffer = &address_info;
+    } else {
+        assert(false);
+    }
+}
+
+DescriptorGetInfo::DescriptorGetInfo(VkDescriptorType type, const vkt::Buffer &buffer, VkDeviceSize range, VkFormat format)
+    : DescriptorGetInfo(type, buffer.Address(), range, format) {}
+}  // namespace vkt
