@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2023-2024 Valve Corporation
- * Copyright (c) 2023-2024 LunarG, Inc.
+ * Copyright (c) 2023-2025 Valve Corporation
+ * Copyright (c) 2023-2025 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,11 +14,12 @@
 
 #include <vulkan/utility/vk_struct_helper.hpp>
 #include <generated/enum_flag_bits.h>
-
-#include "utils/vk_layer_utils.h"
+#include "containers/container_utils.h"
 
 // We are trying to query unsupported handle types, which means we will likley trigger *-handleType-parameter VUs
 void IgnoreHandleTypeError(ErrorMonitor *monitor) {
+    monitor->SetAllowedFailureMsg("VUID-VkPhysicalDeviceExternalFenceInfo-handleType-parameter");
+
     monitor->SetAllowedFailureMsg("VUID-VkFenceGetFdInfoKHR-handleType-parameter");
     monitor->SetAllowedFailureMsg("VUID-VkFenceGetWin32HandleInfoKHR-handleType-parameter");
     monitor->SetAllowedFailureMsg("VUID-VkImportFenceFdInfoKHR-handleType-parameter");
@@ -216,13 +217,17 @@ bool HandleTypeNeedsDedicatedAllocation(VkPhysicalDevice gpu, const VkImageCreat
     return (external_features & VK_EXTERNAL_MEMORY_FEATURE_DEDICATED_ONLY_BIT) != 0;
 }
 
-bool SemaphoreExportImportSupported(VkPhysicalDevice gpu, VkExternalSemaphoreHandleTypeFlagBits handle_type, void *p_next) {
+bool SemaphoreExportImportSupported(VkPhysicalDevice gpu, VkSemaphoreType semaphore_type,
+                                    VkExternalSemaphoreHandleTypeFlagBits handle_type) {
     constexpr auto export_import_flags =
         VK_EXTERNAL_SEMAPHORE_FEATURE_EXPORTABLE_BIT | VK_EXTERNAL_SEMAPHORE_FEATURE_IMPORTABLE_BIT;
 
-    VkPhysicalDeviceExternalSemaphoreInfo info = vku::InitStructHelper(p_next);
-    info.handleType = handle_type;
+    VkSemaphoreTypeCreateInfo type_info = vku::InitStructHelper();
+    type_info.semaphoreType = VK_SEMAPHORE_TYPE_TIMELINE;
+
+    VkPhysicalDeviceExternalSemaphoreInfo external_info = vku::InitStructHelper(&type_info);
+    external_info.handleType = handle_type;
     VkExternalSemaphoreProperties properties = vku::InitStructHelper();
-    vk::GetPhysicalDeviceExternalSemaphoreProperties(gpu, &info, &properties);
+    vk::GetPhysicalDeviceExternalSemaphoreProperties(gpu, &external_info, &properties);
     return (properties.externalSemaphoreFeatures & export_import_flags) == export_import_flags;
-}
\ No newline at end of file
+}
