@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 2023-2024 The Khronos Group Inc.
- * Copyright (c) 2023-2024 Valve Corporation
- * Copyright (c) 2023-2024 LunarG, Inc.
+ * Copyright (c) 2023-2025 The Khronos Group Inc.
+ * Copyright (c) 2023-2025 Valve Corporation
+ * Copyright (c) 2023-2025 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,7 +20,7 @@ CreatePipelineHelper::CreatePipelineHelper(VkLayerTest &test, void *pNext) : lay
     gp_ci_.pNext = pNext;
 
     // InitDescriptorSetInfo
-    dsl_bindings_ = {{0, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER, 1, VK_SHADER_STAGE_ALL, nullptr}};
+    dsl_bindings_.emplace_back(VkDescriptorSetLayoutBinding{0, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER, 1, VK_SHADER_STAGE_ALL, nullptr});
 
     // InitInputAndVertexInfo
     vi_ci_ = vku::InitStructHelper();
@@ -124,6 +124,11 @@ void CreatePipelineHelper::ResetShaderInfo(const char *vertex_shader_text, const
     shader_stages_ = {vs_->GetStageCreateInfo(), fs_->GetStageCreateInfo()};
 }
 
+void CreatePipelineHelper::VertexShaderOnly() {
+    shader_stages_.clear();
+    shader_stages_.push_back(vs_->GetStageCreateInfo());
+}
+
 void CreatePipelineHelper::Destroy() {
     if (pipeline_cache_ != VK_NULL_HANDLE) {
         vk::DestroyPipelineCache(device_->handle(), pipeline_cache_, nullptr);
@@ -288,15 +293,15 @@ VkResult CreatePipelineHelper::CreateGraphicsPipeline(bool do_late_bind, bool no
                                        &pipeline_);
 }
 
-CreateComputePipelineHelper::CreateComputePipelineHelper(VkLayerTest &test, void *pNext) : layer_test_(test) {
+CreateComputePipelineHelper::CreateComputePipelineHelper(vkt::Device& device, void *pNext) {
     // default VkDevice, can be overwritten if multi-device tests
-    device_ = layer_test_.DeviceObj();
+    device_ = &device;
 
     cp_ci_ = vku::InitStructHelper();
     cp_ci_.pNext = pNext;
 
     // InitDescriptorSetInfo
-    dsl_bindings_ = {{0, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER, 1, VK_SHADER_STAGE_ALL, nullptr}};
+    dsl_bindings_.emplace_back(VkDescriptorSetLayoutBinding{0, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER, 1, VK_SHADER_STAGE_ALL, nullptr});
 
     // InitPipelineLayoutInfo
     pipeline_layout_ci_ = vku::InitStructHelper();
@@ -309,19 +314,50 @@ CreateComputePipelineHelper::CreateComputePipelineHelper(VkLayerTest &test, void
     pc_ci_.initialDataSize = 0;
     pc_ci_.pInitialData = nullptr;
 
-    InitShaderInfo();
+    cs_ = VkShaderObj(device, kMinimalShaderGlsl, VK_SHADER_STAGE_COMPUTE_BIT);
 
     cp_ci_.flags = 0;
     cp_ci_.layout = VK_NULL_HANDLE;
 }
 
-CreateComputePipelineHelper::~CreateComputePipelineHelper() { Destroy(); }
+CreateComputePipelineHelper::CreateComputePipelineHelper(VkLayerTest &test, void *pNext)
+    : CreateComputePipelineHelper(*test.DeviceObj(), pNext) {}
 
-void CreateComputePipelineHelper::InitShaderInfo() {
-    cs_ = std::make_unique<VkShaderObj>(&layer_test_, kMinimalShaderGlsl, VK_SHADER_STAGE_COMPUTE_BIT);
-    // We shouldn't need a fragment shader but add it to be able to run on more devices
+CreateComputePipelineHelper::CreateComputePipelineHelper(CreateComputePipelineHelper &&rhs) noexcept { *this = std::move(rhs); }
+
+CreateComputePipelineHelper &CreateComputePipelineHelper::operator=(CreateComputePipelineHelper &&rhs) noexcept {
+    dsl_bindings_ = std::move(rhs.dsl_bindings_);
+    descriptor_set_ = std::move(rhs.descriptor_set_);
+
+    pipeline_layout_ci_ = rhs.pipeline_layout_ci_;
+    rhs.pipeline_layout_ci_ = {};
+
+    pipeline_layout_ = std::move(rhs.pipeline_layout_);
+
+    cp_ci_ = rhs.cp_ci_;
+    rhs.cp_ci_ = {};
+
+    pc_ci_ = rhs.pc_ci_;
+    rhs.pc_ci_ = {};
+
+    pipeline_cache_ = rhs.pipeline_cache_;
+    rhs.pipeline_cache_ = VK_NULL_HANDLE;
+
+    cs_ = std::move(rhs.cs_);
+
+    override_skip_ = rhs.override_skip_;
+    rhs.override_skip_ = false;
+
+    device_ = rhs.device_;
+    rhs.device_ = nullptr;
+
+    pipeline_ = rhs.pipeline_;
+    rhs.pipeline_ = VK_NULL_HANDLE;
+    return *this;
 }
 
+CreateComputePipelineHelper::~CreateComputePipelineHelper() noexcept { Destroy(); }
+
 void CreateComputePipelineHelper::InitPipelineCache() {
     if (pipeline_cache_ != VK_NULL_HANDLE) {
         vk::DestroyPipelineCache(device_->handle(), pipeline_cache_, nullptr);
@@ -346,21 +382,21 @@ void CreateComputePipelineHelper::LateBindPipelineInfo() {
     if (cp_ci_.layout == VK_NULL_HANDLE) {
         // Create a default descriptor and pipeline layout
         if (pipeline_layout_.handle() == VK_NULL_HANDLE) {
-            if (!descriptor_set_) {
+            if (!descriptor_set_.Initialized()) {
                 // User can pass in own bindings
-                descriptor_set_.reset(new OneOffDescriptorSet(device_, dsl_bindings_));
-                ASSERT_TRUE(descriptor_set_->Initialized());
+                descriptor_set_ = OneOffDescriptorSet(device_, dsl_bindings_);
+                ASSERT_TRUE(descriptor_set_.Initialized());
             }
 
             const std::vector<VkPushConstantRange> push_ranges(
                 pipeline_layout_ci_.pPushConstantRanges,
                 pipeline_layout_ci_.pPushConstantRanges + pipeline_layout_ci_.pushConstantRangeCount);
-            pipeline_layout_ = vkt::PipelineLayout(*device_, {&descriptor_set_->layout_}, push_ranges, pipeline_layout_ci_.flags);
+            pipeline_layout_ = vkt::PipelineLayout(*device_, {&descriptor_set_.layout_}, push_ranges, pipeline_layout_ci_.flags);
         }
 
         cp_ci_.layout = pipeline_layout_.handle();
     }
-    cp_ci_.stage = cs_.get()->GetStageCreateInfo();
+    cp_ci_.stage = cs_.GetStageCreateInfo();
 }
 
 VkResult CreateComputePipelineHelper::CreateComputePipeline(bool do_late_bind, bool no_cache) {
@@ -413,19 +449,19 @@ SimpleGPL::SimpleGPL(VkLayerTest &test, VkPipelineLayout layout, const char *ver
     frag_out_lib_.CreateGraphicsPipeline(false);
 
     VkPipeline libraries[4] = {
-        vertex_input_lib_.Handle(),
-        pre_raster_lib_.Handle(),
-        frag_shader_lib_.Handle(),
-        frag_out_lib_.Handle(),
+        vertex_input_lib_,
+        pre_raster_lib_,
+        frag_shader_lib_,
+        frag_out_lib_,
     };
 
     VkPipelineLibraryCreateInfoKHR link_info = vku::InitStructHelper();
-    link_info.libraryCount = size(libraries);
+    link_info.libraryCount = size32(libraries);
     link_info.pLibraries = libraries;
 
     VkGraphicsPipelineCreateInfo exe_pipe_ci = vku::InitStructHelper(&link_info);
     exe_pipe_ci.layout = layout;
-    pipe_.init(*device, exe_pipe_ci);
+    pipe_.Init(*device, exe_pipe_ci);
 }
 
 }  // namespace vkt
