@@ -10,8 +10,9 @@
  */
 
 #include "ray_tracing_objects.h"
+#include "containers/container_utils.h"
 
-#include "utils/vk_layer_utils.h"
+#include "utils/math_utils.h"
 
 // #define VVL_DEBUG_LOG_SBT
 #ifdef VVL_DEBUG_LOG_SBT
@@ -339,7 +340,7 @@ void AccelerationStructureKHR::Create() {
         if (buffer_init_no_mem_) {
             device_buffer_.InitNoMemory(*device_, ci);
         } else {
-            device_buffer_.init(*device_, ci, buffer_memory_property_flags_, &alloc_flags);
+            device_buffer_.Init(*device_, ci, buffer_memory_property_flags_, &alloc_flags);
         }
     }
     vk_info_.buffer = device_buffer_.handle();
@@ -545,7 +546,7 @@ void BuildGeometryInfoKHR::SetupBuild(bool is_on_device_build, bool use_ppGeomet
                 alloc_flags.flags = VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT;
 
                 if (scratch_size > 0) {
-                    device_scratch_->init(*device_, scratch_size + as_props.minAccelerationStructureScratchOffsetAlignment,
+                    device_scratch_->Init(*device_, scratch_size + as_props.minAccelerationStructureScratchOffsetAlignment,
                                           VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT | VK_BUFFER_USAGE_STORAGE_BUFFER_BIT |
                                               device_scratch_additional_flags_,
                                           VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT, &alloc_flags);
@@ -630,7 +631,7 @@ void BuildGeometryInfoKHR::VkCmdBuildAccelerationStructuresIndirectKHR(VkCommand
     VkMemoryAllocateFlagsInfo alloc_flags = vku::InitStructHelper();
     alloc_flags.flags = VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT;
 
-    indirect_buffer_->init(*device_, 1 * vk_info_.geometryCount * sizeof(VkAccelerationStructureBuildRangeInfoKHR),
+    indirect_buffer_->Init(*device_, 1 * vk_info_.geometryCount * sizeof(VkAccelerationStructureBuildRangeInfoKHR),
                            VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT | VK_BUFFER_USAGE_STORAGE_BUFFER_BIT, kHostVisibleMemProps,
                            &alloc_flags);
 
@@ -1106,7 +1107,7 @@ GeometryKHR GeometrySimpleOnDeviceAABBInfo(const vkt::Device &device, VkBufferUs
                                             VK_BUFFER_USAGE_ACCELERATION_STRUCTURE_STORAGE_BIT_KHR |
                                             VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT | additional_geometry_buffer_flags;
 
-    aabb_buffer.init(device, aabb_buffer_size, buffer_usage, kHostVisibleMemProps, &alloc_flags);
+    aabb_buffer.Init(device, aabb_buffer_size, buffer_usage, kHostVisibleMemProps, &alloc_flags);
 
     // Fill buffer with one AABB
     aabb_geometry.SetPrimitiveCount(static_cast<uint32_t>(aabbs.size()));
@@ -1289,8 +1290,8 @@ BuildGeometryInfoKHR BuildGeometryInfoSimpleOnHostBottomLevel(const vkt::Device
 }
 
 vkt::as::BuildGeometryInfoKHR BuildGeometryInfoSimpleOnDeviceTopLevel(const vkt::Device &device,
-                                                                      std::shared_ptr<BuildGeometryInfoKHR> on_device_blas) {
-    assert(on_device_blas->GetDstAS()->IsBuilt());
+                                                                      const vkt::as::AccelerationStructureKHR &on_device_blas) {
+    assert(on_device_blas.IsBuilt());
 
     BuildGeometryInfoKHR out_build_info(&device);
 
@@ -1300,7 +1301,7 @@ vkt::as::BuildGeometryInfoKHR BuildGeometryInfoSimpleOnDeviceTopLevel(const vkt:
 
     // Set geometry to one instance pointing to bottom level acceleration structure
     std::vector<GeometryKHR> geometries;
-    geometries.emplace_back(GeometrySimpleDeviceInstance(device, on_device_blas->GetDstAS()->handle()));
+    geometries.emplace_back(GeometrySimpleDeviceInstance(device, on_device_blas.handle()));
     out_build_info.SetGeometries(std::move(geometries));
     out_build_info.SetBuildRanges(out_build_info.GetBuildRangeInfosFromGeometries());
 
@@ -1354,9 +1355,8 @@ BuildGeometryInfoKHR BuildOnDeviceTopLevel(const vkt::Device &device, vkt::Queue
     // Create acceleration structure
     cmd_buffer.Begin();
     // Build Bottom Level Acceleration Structure
-    auto bot_level_accel_struct =
-        std::make_shared<vkt::as::BuildGeometryInfoKHR>(vkt::as::blueprint::BuildGeometryInfoSimpleOnDeviceBottomLevel(device));
-    bot_level_accel_struct->BuildCmdBuffer(cmd_buffer);
+    vkt::as::BuildGeometryInfoKHR bot_level_accel_struct = vkt::as::blueprint::BuildGeometryInfoSimpleOnDeviceBottomLevel(device);
+    bot_level_accel_struct.BuildCmdBuffer(cmd_buffer);
     cmd_buffer.End();
 
     queue.Submit(cmd_buffer);
@@ -1365,7 +1365,7 @@ BuildGeometryInfoKHR BuildOnDeviceTopLevel(const vkt::Device &device, vkt::Queue
     cmd_buffer.Begin();
     // Build Top Level Acceleration Structure
     vkt::as::BuildGeometryInfoKHR top_level_accel_struct =
-        vkt::as::blueprint::BuildGeometryInfoSimpleOnDeviceTopLevel(device, bot_level_accel_struct);
+        vkt::as::blueprint::BuildGeometryInfoSimpleOnDeviceTopLevel(device, *bot_level_accel_struct.GetDstAS());
     top_level_accel_struct.BuildCmdBuffer(cmd_buffer);
     cmd_buffer.End();
 
@@ -1381,7 +1381,8 @@ BuildGeometryInfoKHR BuildOnDeviceTopLevel(const vkt::Device &device, vkt::Queue
 
 namespace rt {
 
-Pipeline::Pipeline(VkLayerTest &test, vkt::Device *device) : test_(test), device_(device) {}
+Pipeline::Pipeline(VkLayerTest &test, vkt::Device *device)
+    : test_(test), device_(device), pipeline_layout_ci_(vku::InitStructHelper()) {}
 
 Pipeline::~Pipeline() {
     if (deferred_op_ != VK_NULL_HANDLE) {
@@ -1412,6 +1413,11 @@ void Pipeline::AddBinding(VkDescriptorType descriptor_type, uint32_t binding, ui
 
 void Pipeline::CreateDescriptorSet() { desc_set_ = std::make_unique<OneOffDescriptorSet>(device_, bindings_); }
 
+void Pipeline::SetPipelineSetLayouts(uint32_t set_layout_count, const VkDescriptorSetLayout *set_layouts) {
+    pipeline_layout_ci_.setLayoutCount = set_layout_count;
+    pipeline_layout_ci_.pSetLayouts = set_layouts;
+}
+
 void Pipeline::SetPushConstantRangeSize(uint32_t byte_size) { push_constant_range_size_ = byte_size; }
 
 void Pipeline::SetGlslRayGenShader(const char *glsl) {
@@ -1466,16 +1472,15 @@ void Pipeline::BuildPipeline() {
 
     // Create pipeline layout
     if (!pipeline_layout_.initialized()) {
-        VkPipelineLayoutCreateInfo pipeline_layout_ci = vku::InitStructHelper();
         if (push_constant_range_size_ > 0) {
-            pipeline_layout_ci.pushConstantRangeCount = 1;
-            pipeline_layout_ci.pPushConstantRanges = &push_constant_range;
+            pipeline_layout_ci_.pushConstantRangeCount = 1;
+            pipeline_layout_ci_.pPushConstantRanges = &push_constant_range;
         }
         if (desc_set_) {
-            pipeline_layout_ci.setLayoutCount = 1;
-            pipeline_layout_ci.pSetLayouts = &desc_set_->layout_.handle();
+            pipeline_layout_ci_.setLayoutCount = 1;
+            pipeline_layout_ci_.pSetLayouts = &desc_set_->layout_.handle();
         }
-        pipeline_layout_.init(*device_, pipeline_layout_ci);
+        pipeline_layout_.Init(*device_, pipeline_layout_ci_);
     }
 
     // Assemble shaders information (stages and groups)
@@ -1549,7 +1554,7 @@ void Pipeline::BuildPipeline() {
     vk_info_.layout = pipeline_layout_;
 
     if (deferred_op_ == VK_NULL_HANDLE) {
-        rt_pipeline_.init(*device_, vk_info_);
+        rt_pipeline_.Init(*device_, vk_info_);
     } else {
         rt_pipeline_.InitDeferred(*device_, vk_info_, deferred_op_);
 
@@ -1599,7 +1604,7 @@ void Pipeline::BuildSbt() {
         VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT | VK_BUFFER_USAGE_SHADER_BINDING_TABLE_BIT_KHR | VK_BUFFER_USAGE_TRANSFER_SRC_BIT;
     VkMemoryAllocateFlagsInfo alloc_flags = vku::InitStructHelper();
     alloc_flags.flags = VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT;
-    sbt_buffer_.init(*device_, sbt_buffer_info, kHostVisibleMemProps, &alloc_flags);
+    sbt_buffer_.Init(*device_, sbt_buffer_info, kHostVisibleMemProps, &alloc_flags);
 
 #ifdef VVL_DEBUG_LOG_SBT
     std::cout << "SBT buffer fill:\n";
@@ -1800,6 +1805,8 @@ void Pipeline::DeferBuild() {
     }
 }
 
+VkShaderObj &Pipeline::GetRayGenShader(uint32_t ray_gen_i) { return *ray_gen_shaders_[ray_gen_i]; }
+
 vkt::rt::TraceRaysSbt Pipeline::GetTraceRaysSbt(uint32_t ray_gen_shader_i /*= 0*/) {
     // As of now, no function support if not using any ray generation shader
     assert(!ray_gen_shaders_.empty());
