@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 2015-2024 The Khronos Group Inc.
- * Copyright (c) 2015-2024 Valve Corporation
- * Copyright (c) 2015-2024 LunarG, Inc.
+ * Copyright (c) 2015-2025 The Khronos Group Inc.
+ * Copyright (c) 2015-2025 Valve Corporation
+ * Copyright (c) 2015-2025 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,6 +22,7 @@
 #include <glslang/Public/ShaderLang.h>
 
 #include "render.h"
+// This include is light and easier to have here a single place for all tests to grab
 #include "shader_templates.h"
 
 class VkRenderFramework;
@@ -43,10 +44,20 @@ bool ASMtoSPV(const spv_target_env target_env, const uint32_t options, const cha
 // TODO - move all of VkShaderObj to vkt::ShaderModule
 class VkShaderObj : public vkt::ShaderModule {
   public:
+    VkShaderObj() = default;
+    VkShaderObj(VkShaderObj &&rhs) noexcept = default;
+    VkShaderObj &operator=(VkShaderObj &&rhs) noexcept = default;
+
     // optional arguments listed order of most likely to be changed manually by a test
+    VkShaderObj(vkt::Device &device, const char *source, VkShaderStageFlagBits stage, const spv_target_env env = SPV_ENV_VULKAN_1_0,
+                SpvSourceType source_type = SPV_SOURCE_GLSL, const VkSpecializationInfo *spec_info = nullptr,
+                char const *entry_point = "main", const void *pNext = nullptr);
+
+    // DEPRECATED
     VkShaderObj(VkRenderFramework *framework, const char *source, VkShaderStageFlagBits stage,
                 const spv_target_env env = SPV_ENV_VULKAN_1_0, SpvSourceType source_type = SPV_SOURCE_GLSL,
                 const VkSpecializationInfo *spec_info = nullptr, char const *entry_point = "main", const void *pNext = nullptr);
+
     VkPipelineShaderStageCreateInfo const &GetStageCreateInfo() const;
 
     bool InitFromGLSL(const void *pNext = nullptr);
@@ -56,18 +67,16 @@ class VkShaderObj : public vkt::ShaderModule {
 
     // These functions return a pointer to a newly created _and initialized_ VkShaderObj if initialization was successful.
     // Otherwise, {} is returned.
-    static std::unique_ptr<VkShaderObj> CreateFromGLSL(VkRenderFramework *framework, const char *source,
-                                                       VkShaderStageFlagBits stage, const spv_target_env = SPV_ENV_VULKAN_1_0,
-                                                       const VkSpecializationInfo *spec_info = nullptr,
-                                                       const char *entry_point = "main");
-    static std::unique_ptr<VkShaderObj> CreateFromASM(VkRenderFramework *framework, const char *source, VkShaderStageFlagBits stage,
-                                                      const spv_target_env spv_env = SPV_ENV_VULKAN_1_0,
-                                                      const VkSpecializationInfo *spec_info = nullptr,
-                                                      const char *entry_point = "main");
+    static VkShaderObj CreateFromGLSL(VkRenderFramework *framework, const char *source, VkShaderStageFlagBits stage,
+                                      const spv_target_env = SPV_ENV_VULKAN_1_0, const VkSpecializationInfo *spec_info = nullptr,
+                                      const char *entry_point = "main");
+    static VkShaderObj CreateFromASM(VkRenderFramework *framework, const char *source, VkShaderStageFlagBits stage,
+                                     const spv_target_env spv_env = SPV_ENV_VULKAN_1_0,
+                                     const VkSpecializationInfo *spec_info = nullptr, const char *entry_point = "main");
 
-  protected:
+  private:
+    vkt::Device *m_device = nullptr;
     VkPipelineShaderStageCreateInfo m_stage_info;
-    vkt::Device &m_device;
     const char *m_source;
     spv_target_env m_spv_env;
 };
