@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 2023 The Khronos Group Inc.
- * Copyright (c) 2023 Valve Corporation
- * Copyright (c) 2023 LunarG, Inc.
+ * Copyright (c) 2023-2025 The Khronos Group Inc.
+ * Copyright (c) 2023-2025 Valve Corporation
+ * Copyright (c) 2023-2025 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -29,13 +29,14 @@ void AndroidExternalResolveTest::InitBasicAndroidExternalResolve() {
     nullColorAttachmentWithExternalFormatResolve = external_format_resolve_props.nullColorAttachmentWithExternalFormatResolve;
 }
 
+class PositiveAndroidExternalResolve : public AndroidExternalResolveTest {};
+
 TEST_F(PositiveAndroidExternalResolve, NoResolve) {
     TEST_DESCRIPTION("Make sure enabling the feature doesn't break normal usage of API.");
     RETURN_IF_SKIP(InitBasicAndroidExternalResolve());
     InitRenderTarget();
 
     CreatePipelineHelper pipe(*this);
-    pipe.InitState();
     pipe.CreateGraphicsPipeline();
 }
 
@@ -78,15 +79,13 @@ TEST_F(PositiveAndroidExternalResolve, RenderPassAndFramebuffer) {
     image_ci.tiling = VK_IMAGE_TILING_OPTIMAL;
     image_ci.usage = VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT;
 
-    VkImageObj color_image(m_device);
-    color_image.Init(image_ci);
+    vkt::Image color_image(*m_device, image_ci, vkt::set_layout);
     vkt::ImageView color_view = color_image.CreateView();
 
     image_ci.pNext = &external_format;
     image_ci.format = VK_FORMAT_UNDEFINED;
     image_ci.usage = VK_IMAGE_USAGE_SAMPLED_BIT;
-    VkImageObj resolve_image(m_device);
-    resolve_image.Init(image_ci);
+    vkt::Image resolve_image(*m_device, image_ci, vkt::set_layout);
 
     VkSamplerYcbcrConversionCreateInfo sycci = vku::InitStructHelper(&external_format);
     sycci.format = VK_FORMAT_UNDEFINED;
@@ -97,25 +96,21 @@ TEST_F(PositiveAndroidExternalResolve, RenderPassAndFramebuffer) {
     vkt::SamplerYcbcrConversion ycbcr_conv(*m_device, sycci);
 
     VkSamplerYcbcrConversionInfo syci = vku::InitStructHelper();
-    syci.conversion = ycbcr_conv.handle();
+    syci.conversion = ycbcr_conv;
 
     VkImageViewCreateInfo ivci = vku::InitStructHelper(&syci);
-    ivci.image = resolve_image.handle();
+    ivci.image = resolve_image;
     ivci.viewType = VK_IMAGE_VIEW_TYPE_2D;
     ivci.format = VK_FORMAT_UNDEFINED;
     ivci.subresourceRange = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1};
     const vkt::ImageView resolve_view(*m_device, ivci);
 
-    VkImageView attachments[2];
-    attachments[0] = color_view.handle();
-    attachments[1] = resolve_view.handle();
+    VkImageView attachments[2] = {color_view, resolve_view};
 
-    vkt::Framebuffer framebuffer(*m_device, rp.Handle(), 2, attachments);
+    vkt::Framebuffer framebuffer(*m_device, rp, 2, attachments);
 
-    CreatePipelineHelper pipe(*this);
-    pipe.InitState();
-    pipe.gp_ci_.pNext = &external_format;
-    pipe.gp_ci_.renderPass = rp.Handle();
+    CreatePipelineHelper pipe(*this, &external_format);
+    pipe.gp_ci_.renderPass = rp;
     pipe.CreateGraphicsPipeline();
 }
 
@@ -159,16 +154,13 @@ TEST_F(PositiveAndroidExternalResolve, ImagelessFramebuffer) {
     image_ci.samples = VK_SAMPLE_COUNT_1_BIT;
     image_ci.tiling = VK_IMAGE_TILING_OPTIMAL;
     image_ci.usage = VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT;
-
-    VkImageObj color_image(m_device);
-    color_image.Init(image_ci);
+    vkt::Image color_image(*m_device, image_ci, vkt::set_layout);
     vkt::ImageView color_view = color_image.CreateView();
 
     image_ci.pNext = &external_format;
     image_ci.format = VK_FORMAT_UNDEFINED;
     image_ci.usage = VK_IMAGE_USAGE_SAMPLED_BIT | VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT;
-    VkImageObj resolve_image(m_device);
-    resolve_image.Init(image_ci);
+    vkt::Image resolve_image(*m_device, image_ci, vkt::set_layout);
 
     VkSamplerYcbcrConversionCreateInfo sycci = vku::InitStructHelper(&external_format);
     sycci.format = VK_FORMAT_UNDEFINED;
@@ -179,18 +171,16 @@ TEST_F(PositiveAndroidExternalResolve, ImagelessFramebuffer) {
     vkt::SamplerYcbcrConversion ycbcr_conv(*m_device, sycci);
 
     VkSamplerYcbcrConversionInfo syci = vku::InitStructHelper();
-    syci.conversion = ycbcr_conv.handle();
+    syci.conversion = ycbcr_conv;
 
     VkImageViewCreateInfo ivci = vku::InitStructHelper(&syci);
-    ivci.image = resolve_image.handle();
+    ivci.image = resolve_image;
     ivci.viewType = VK_IMAGE_VIEW_TYPE_2D;
     ivci.format = VK_FORMAT_UNDEFINED;
     ivci.subresourceRange = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1};
     const vkt::ImageView resolve_view(*m_device, ivci);
 
-    VkImageView attachments[2];
-    attachments[0] = color_view.handle();
-    attachments[1] = resolve_view.handle();
+    VkImageView attachments[2] = {color_view, resolve_view};
 
     VkFramebufferAttachmentImageInfo framebuffer_attachment_image_info[2];
     framebuffer_attachment_image_info[0] = vku::InitStructHelper();
@@ -215,7 +205,7 @@ TEST_F(PositiveAndroidExternalResolve, ImagelessFramebuffer) {
     fb_ci.height = 32;
     fb_ci.layers = 1;
     fb_ci.attachmentCount = 2;
-    fb_ci.renderPass = rp.Handle();
+    fb_ci.renderPass = rp;
     m_errorMonitor->SetAllowedFailureMsg("VUID-VkFramebufferCreateInfo-flags-03201");
     vkt::Framebuffer framebuffer(*m_device, fb_ci);
 
@@ -227,16 +217,16 @@ TEST_F(PositiveAndroidExternalResolve, ImagelessFramebuffer) {
     render_pass_attachment_bi.pAttachments = attachments;
 
     VkRenderPassBeginInfo render_pass_bi = vku::InitStructHelper(&render_pass_attachment_bi);
-    render_pass_bi.renderPass = rp.Handle();
-    render_pass_bi.framebuffer = framebuffer.handle();
+    render_pass_bi.renderPass = rp;
+    render_pass_bi.framebuffer = framebuffer;
     render_pass_bi.renderArea.extent = {1, 1};
     render_pass_bi.clearValueCount = 1;
     render_pass_bi.pClearValues = &clear_value;
 
-    m_commandBuffer->begin();
-    m_commandBuffer->BeginRenderPass(render_pass_bi);
-    m_commandBuffer->EndRenderPass();
-    m_commandBuffer->end();
+    m_command_buffer.Begin();
+    m_command_buffer.BeginRenderPass(render_pass_bi);
+    m_command_buffer.EndRenderPass();
+    m_command_buffer.End();
 }
 
 TEST_F(PositiveAndroidExternalResolve, DynamicRendering) {
@@ -267,17 +257,14 @@ TEST_F(PositiveAndroidExternalResolve, DynamicRendering) {
     image_ci.samples = VK_SAMPLE_COUNT_4_BIT;
     image_ci.tiling = VK_IMAGE_TILING_OPTIMAL;
     image_ci.usage = VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT;
-
-    VkImageObj color_image(m_device);
-    color_image.Init(image_ci);
+    vkt::Image color_image(*m_device, image_ci, vkt::set_layout);
     vkt::ImageView color_view = color_image.CreateView();
 
     image_ci.pNext = &external_format;
     image_ci.samples = VK_SAMPLE_COUNT_1_BIT;
     image_ci.format = VK_FORMAT_UNDEFINED;
     image_ci.usage = VK_IMAGE_USAGE_SAMPLED_BIT | VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT;
-    VkImageObj resolve_image(m_device);
-    resolve_image.Init(image_ci);
+    vkt::Image resolve_image(*m_device, image_ci, vkt::set_layout);
 
     VkSamplerYcbcrConversionCreateInfo sycci = vku::InitStructHelper(&external_format);
     sycci.format = VK_FORMAT_UNDEFINED;
@@ -288,34 +275,34 @@ TEST_F(PositiveAndroidExternalResolve, DynamicRendering) {
     vkt::SamplerYcbcrConversion ycbcr_conv(*m_device, sycci);
 
     VkSamplerYcbcrConversionInfo syci = vku::InitStructHelper();
-    syci.conversion = ycbcr_conv.handle();
+    syci.conversion = ycbcr_conv;
 
     VkImageViewCreateInfo ivci = vku::InitStructHelper(&syci);
-    ivci.image = resolve_image.handle();
+    ivci.image = resolve_image;
     ivci.viewType = VK_IMAGE_VIEW_TYPE_2D;
     ivci.format = VK_FORMAT_UNDEFINED;
     ivci.subresourceRange = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1};
     const vkt::ImageView resolve_view(*m_device, ivci);
 
-    VkRenderingAttachmentInfoKHR color_attachment = vku::InitStructHelper();
-    color_attachment.imageView = color_view.handle();
+    VkRenderingAttachmentInfo color_attachment = vku::InitStructHelper();
+    color_attachment.imageView = color_view;
     color_attachment.imageLayout = VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL;
-    color_attachment.resolveMode = VK_RESOLVE_MODE_EXTERNAL_FORMAT_DOWNSAMPLE_ANDROID;
-    color_attachment.resolveImageView = resolve_view.handle();
+    color_attachment.resolveMode = VK_RESOLVE_MODE_EXTERNAL_FORMAT_DOWNSAMPLE_BIT_ANDROID;
+    color_attachment.resolveImageView = resolve_view;
     color_attachment.resolveImageLayout = VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL;
 
-    VkRenderingInfoKHR begin_rendering_info = vku::InitStructHelper();
+    VkRenderingInfo begin_rendering_info = vku::InitStructHelper();
     begin_rendering_info.layerCount = 1;
     begin_rendering_info.renderArea = {{0, 0}, {32, 32}};
     begin_rendering_info.colorAttachmentCount = 1;
     begin_rendering_info.pColorAttachments = &color_attachment;
 
-    m_commandBuffer->begin();
+    m_command_buffer.Begin();
     m_errorMonitor->SetAllowedFailureMsg("VUID-VkRenderingAttachmentInfo-imageView-06865");
     m_errorMonitor->SetAllowedFailureMsg("VUID-VkRenderingAttachmentInfo-imageView-06129");
-    m_commandBuffer->BeginRendering(begin_rendering_info);
-    m_commandBuffer->EndRendering();
-    m_commandBuffer->end();
+    m_command_buffer.BeginRendering(begin_rendering_info);
+    m_command_buffer.EndRendering();
+    m_command_buffer.End();
 }
 
 TEST_F(PositiveAndroidExternalResolve, PipelineBarrier) {
@@ -358,16 +345,13 @@ TEST_F(PositiveAndroidExternalResolve, PipelineBarrier) {
     image_ci.samples = VK_SAMPLE_COUNT_1_BIT;
     image_ci.tiling = VK_IMAGE_TILING_OPTIMAL;
     image_ci.usage = VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT;
-
-    VkImageObj color_image(m_device);
-    color_image.Init(image_ci);
+    vkt::Image color_image(*m_device, image_ci, vkt::set_layout);
     vkt::ImageView color_view = color_image.CreateView();
 
     image_ci.pNext = &external_format;
     image_ci.format = VK_FORMAT_UNDEFINED;
     image_ci.usage = VK_IMAGE_USAGE_SAMPLED_BIT;
-    VkImageObj resolve_image(m_device);
-    resolve_image.Init(image_ci);
+    vkt::Image resolve_image(*m_device, image_ci, vkt::set_layout);
 
     VkSamplerYcbcrConversionCreateInfo sycci = vku::InitStructHelper(&external_format);
     sycci.format = VK_FORMAT_UNDEFINED;
@@ -378,29 +362,25 @@ TEST_F(PositiveAndroidExternalResolve, PipelineBarrier) {
     vkt::SamplerYcbcrConversion ycbcr_conv(*m_device, sycci);
 
     VkSamplerYcbcrConversionInfo syci = vku::InitStructHelper();
-    syci.conversion = ycbcr_conv.handle();
+    syci.conversion = ycbcr_conv;
 
     VkImageViewCreateInfo ivci = vku::InitStructHelper(&syci);
-    ivci.image = resolve_image.handle();
+    ivci.image = resolve_image;
     ivci.viewType = VK_IMAGE_VIEW_TYPE_2D;
     ivci.format = VK_FORMAT_UNDEFINED;
     ivci.subresourceRange = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1};
     const vkt::ImageView resolve_view(*m_device, ivci);
 
-    VkImageView attachments[2];
-    attachments[0] = color_view.handle();
-    attachments[1] = resolve_view.handle();
+    VkImageView attachments[2] = {color_view, resolve_view};
 
-    vkt::Framebuffer framebuffer(*m_device, rp.Handle(), 2, attachments);
+    vkt::Framebuffer framebuffer(*m_device, rp, 2, attachments);
 
-    CreatePipelineHelper pipe(*this);
-    pipe.InitState();
-    pipe.gp_ci_.pNext = &external_format;
-    pipe.gp_ci_.renderPass = rp.Handle();
+    CreatePipelineHelper pipe(*this, &external_format);
+    pipe.gp_ci_.renderPass = rp;
     pipe.CreateGraphicsPipeline();
 
-    m_commandBuffer->begin();
-    m_commandBuffer->BeginRenderPass(rp.Handle(), framebuffer.handle());
+    m_command_buffer.Begin();
+    m_command_buffer.BeginRenderPass(rp, framebuffer);
 
     VkImageMemoryBarrier image_barrier = vku::InitStructHelper();
     image_barrier.srcAccessMask = VK_ACCESS_COLOR_ATTACHMENT_WRITE_BIT;
@@ -408,16 +388,16 @@ TEST_F(PositiveAndroidExternalResolve, PipelineBarrier) {
     image_barrier.subresourceRange = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1};
     image_barrier.oldLayout = VK_IMAGE_LAYOUT_GENERAL;
     image_barrier.newLayout = VK_IMAGE_LAYOUT_GENERAL;
-    image_barrier.image = resolve_image.handle();
+    image_barrier.image = resolve_image;
     image_barrier.srcQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
     image_barrier.dstQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
 
-    vk::CmdPipelineBarrier(m_commandBuffer->handle(), VK_PIPELINE_STAGE_COLOR_ATTACHMENT_OUTPUT_BIT,
+    vk::CmdPipelineBarrier(m_command_buffer, VK_PIPELINE_STAGE_COLOR_ATTACHMENT_OUTPUT_BIT,
                            VK_PIPELINE_STAGE_COLOR_ATTACHMENT_OUTPUT_BIT, VK_DEPENDENCY_BY_REGION_BIT, 0, nullptr, 0, nullptr, 1,
                            &image_barrier);
 
-    m_commandBuffer->EndRenderPass();
-    m_commandBuffer->end();
+    m_command_buffer.EndRenderPass();
+    m_command_buffer.End();
 }
 
 #endif  // VK_USE_PLATFORM_ANDROID_KHR
