@@ -1143,7 +1143,7 @@ TEST_F(NegativeShaderInterface, VsFsTypeMismatchShaderObject) {
     SetDefaultDynamicStatesExclude();
     m_command_buffer.BindShaders(vertShader, fragShader);
     m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-OpEntryPoint-07754");
-    vk::CmdDraw(m_command_buffer.handle(), 4, 1, 0, 0);
+    vk::CmdDraw(m_command_buffer, 4, 1, 0, 0);
     m_errorMonitor->VerifyFound();
     m_command_buffer.EndRendering();
     m_command_buffer.End();
@@ -1182,7 +1182,7 @@ TEST_F(NegativeShaderInterface, VsFsTypeMismatchVectorSizeShaderObject) {
     SetDefaultDynamicStatesExclude();
     m_command_buffer.BindShaders(vertShader, fragShader);
     m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-maintenance4-06817");
-    vk::CmdDraw(m_command_buffer.handle(), 4, 1, 0, 0);
+    vk::CmdDraw(m_command_buffer, 4, 1, 0, 0);
     m_errorMonitor->VerifyFound();
     m_command_buffer.EndRendering();
     m_command_buffer.End();
@@ -1707,9 +1707,9 @@ TEST_F(NegativeShaderInterface, DISABLED_FragmentOutputNotWrittenDynamicRenderin
 
     m_command_buffer.Begin();
     m_command_buffer.BeginRenderingColor(GetDynamicRenderTarget(), GetRenderTargetArea());
-    vk::CmdBindPipeline(m_command_buffer.handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipe.Handle());
+    vk::CmdBindPipeline(m_command_buffer, VK_PIPELINE_BIND_POINT_GRAPHICS, pipe);
     m_errorMonitor->SetDesiredWarning("Undefined-Value-ShaderOutputNotProduced-DynamicRendering");
-    vk::CmdDraw(m_command_buffer.handle(), 3, 1, 0, 0);
+    vk::CmdDraw(m_command_buffer, 3, 1, 0, 0);
     m_errorMonitor->VerifyFound();
     m_command_buffer.EndRendering();
     m_command_buffer.End();
@@ -1733,9 +1733,9 @@ TEST_F(NegativeShaderInterface, DISABLED_FragmentOutputNotWrittenDynamicRenderin
     SetDefaultDynamicStatesExclude();
     m_command_buffer.BindShaders(vert_shader, frag_shader);
     VkColorComponentFlags color_write_mask = 0xf;  // all
-    vk::CmdSetColorWriteMaskEXT(m_command_buffer.handle(), 0, 1, &color_write_mask);
+    vk::CmdSetColorWriteMaskEXT(m_command_buffer, 0, 1, &color_write_mask);
     m_errorMonitor->SetDesiredWarning("Undefined-Value-ShaderOutputNotProduced-DynamicRendering");
-    vk::CmdDraw(m_command_buffer.handle(), 4, 1, 0, 0);
+    vk::CmdDraw(m_command_buffer, 4, 1, 0, 0);
     m_errorMonitor->VerifyFound();
     m_command_buffer.EndRendering();
     m_command_buffer.End();
@@ -1776,7 +1776,7 @@ TEST_F(NegativeShaderInterface, DISABLED_FragmentOutputNotWrittenArray) {
     pipe.shader_stages_[1] = fs.GetStageCreateInfo();
     pipe.cb_ci_.attachmentCount = 2;
     pipe.cb_ci_.pAttachments = color_blends;
-    pipe.gp_ci_.renderPass = rp.Handle();
+    pipe.gp_ci_.renderPass = rp;
     m_errorMonitor->SetDesiredWarning("Undefined-Value-ShaderOutputNotProduced");
     pipe.CreateGraphicsPipeline();
     m_errorMonitor->VerifyFound();
@@ -1829,9 +1829,9 @@ TEST_F(NegativeShaderInterface, DISABLED_FragmentOutputNotWrittenArrayDynamicRen
 
     m_command_buffer.Begin();
     m_command_buffer.BeginRendering(rendering_info);
-    vk::CmdBindPipeline(m_command_buffer.handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipe.Handle());
+    vk::CmdBindPipeline(m_command_buffer, VK_PIPELINE_BIND_POINT_GRAPHICS, pipe);
     m_errorMonitor->SetDesiredWarning("Undefined-Value-ShaderOutputNotProduced-DynamicRendering");
-    vk::CmdDraw(m_command_buffer.handle(), 3, 1, 0, 0);
+    vk::CmdDraw(m_command_buffer, 3, 1, 0, 0);
     m_errorMonitor->VerifyFound();
     m_command_buffer.EndRendering();
     m_command_buffer.End();
@@ -1889,9 +1889,9 @@ TEST_F(NegativeShaderInterface, FragmentOutputTypeMismatchDynamicRendering) {
 
     m_command_buffer.Begin();
     m_command_buffer.BeginRenderingColor(GetDynamicRenderTarget(), GetRenderTargetArea());
-    vk::CmdBindPipeline(m_command_buffer.handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipe.Handle());
+    vk::CmdBindPipeline(m_command_buffer, VK_PIPELINE_BIND_POINT_GRAPHICS, pipe);
     m_errorMonitor->SetDesiredWarning("Undefined-Value-ShaderFragmentOutputMismatch-DynamicRendering");
-    vk::CmdDraw(m_command_buffer.handle(), 3, 1, 0, 0);
+    vk::CmdDraw(m_command_buffer, 3, 1, 0, 0);
     m_errorMonitor->VerifyFound();
     m_command_buffer.EndRendering();
     m_command_buffer.End();
@@ -1916,10 +1916,10 @@ TEST_F(NegativeShaderInterface, FragmentOutputDynamicRenderingUnusedAttachments)
 
     VkFormat ds_format = FindSupportedDepthStencilFormat(Gpu());
 
-    vkt::Image color_image(*m_device, 32, 32, 1, VK_FORMAT_R8G8B8A8_UNORM, VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT);
+    vkt::Image color_image(*m_device, 32, 32, VK_FORMAT_R8G8B8A8_UNORM, VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT);
     vkt::ImageView color_image_view = color_image.CreateView();
 
-    vkt::Image ds_image(*m_device, 32, 32, 1, ds_format, VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT);
+    vkt::Image ds_image(*m_device, 32, 32, ds_format, VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT);
     const vkt::ImageView depth_view(*m_device, ds_image.BasicViewCreatInfo(VK_IMAGE_ASPECT_DEPTH_BIT));
 
     VkRenderingAttachmentInfo color_attachment = vku::InitStructHelper();
@@ -1928,7 +1928,7 @@ TEST_F(NegativeShaderInterface, FragmentOutputDynamicRenderingUnusedAttachments)
 
     VkRenderingAttachmentInfo depth_stencil_attachment = vku::InitStructHelper();
     depth_stencil_attachment.imageLayout = VK_IMAGE_LAYOUT_DEPTH_STENCIL_ATTACHMENT_OPTIMAL;
-    depth_stencil_attachment.imageView = depth_view.handle();
+    depth_stencil_attachment.imageView = depth_view;
 
     m_command_buffer.Begin();
     VkRenderingInfo begin_rendering_info = vku::InitStructHelper();
@@ -1938,9 +1938,9 @@ TEST_F(NegativeShaderInterface, FragmentOutputDynamicRenderingUnusedAttachments)
     begin_rendering_info.pDepthAttachment = &depth_stencil_attachment;
     begin_rendering_info.renderArea = {{0, 0}, {1, 1}};
     m_command_buffer.BeginRendering(begin_rendering_info);
-    vk::CmdBindPipeline(m_command_buffer.handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipe.Handle());
+    vk::CmdBindPipeline(m_command_buffer, VK_PIPELINE_BIND_POINT_GRAPHICS, pipe);
     m_errorMonitor->SetDesiredWarning("Undefined-Value-ShaderOutputNotConsumed-DynamicRendering");
-    vk::CmdDraw(m_command_buffer.handle(), 3, 1, 0, 0);
+    vk::CmdDraw(m_command_buffer, 3, 1, 0, 0);
     m_errorMonitor->VerifyFound();
     m_command_buffer.EndRendering();
     m_command_buffer.End();
@@ -2000,9 +2000,9 @@ TEST_F(NegativeShaderInterface, FragmentOutputNotConsumedDynamicRendering) {
 
     m_command_buffer.Begin();
     m_command_buffer.BeginRenderingColor(GetDynamicRenderTarget(), GetRenderTargetArea());
-    vk::CmdBindPipeline(m_command_buffer.handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipe.Handle());
+    vk::CmdBindPipeline(m_command_buffer, VK_PIPELINE_BIND_POINT_GRAPHICS, pipe);
     m_errorMonitor->SetDesiredWarning("Undefined-Value-ShaderOutputNotConsumed-DynamicRendering");
-    vk::CmdDraw(m_command_buffer.handle(), 3, 1, 0, 0);
+    vk::CmdDraw(m_command_buffer, 3, 1, 0, 0);
     m_errorMonitor->VerifyFound();
     m_command_buffer.EndRendering();
     m_command_buffer.End();
@@ -2039,8 +2039,7 @@ TEST_F(NegativeShaderInterface, InvalidStaticSpirv) {
                OpFunctionEnd
         )";
 
-    // VUID-StandaloneSpirv-Location-04919
-    m_errorMonitor->SetDesiredError("VUID-VkShaderModuleCreateInfo-pCode-08737");
+    m_errorMonitor->SetDesiredError("VUID-StandaloneSpirv-Location-04919");
     auto fs = VkShaderObj::CreateFromASM(this, spv_source, VK_SHADER_STAGE_FRAGMENT_BIT);
     m_errorMonitor->VerifyFound();
 }
@@ -2124,60 +2123,54 @@ TEST_F(NegativeShaderInterface, InvalidStaticSpirvMaintenance5Compute) {
     vkt::PipelineLayout layout(*m_device, {});
     CreateComputePipelineHelper pipe(*this);
     pipe.cp_ci_.stage = stage_ci;
-    pipe.cp_ci_.layout = layout.handle();
+    pipe.cp_ci_.layout = layout;
 
     m_errorMonitor->SetDesiredError("VUID-VkShaderModuleCreateInfo-pCode-08737");
     pipe.CreateComputePipeline(false);
     m_errorMonitor->VerifyFound();
 }
 
-// Re-enable when https://github.com/KhronosGroup/SPIRV-Tools/pull/6000 is merged
-TEST_F(NegativeShaderInterface, DISABLED_PhysicalStorageBuffer) {
-    TEST_DESCRIPTION("Regression shaders from https://github.com/KhronosGroup/Vulkan-ValidationLayers/pull/5349");
+TEST_F(NegativeShaderInterface, PhysicalStorageBuffer) {
+    TEST_DESCRIPTION("Details found in https://gitlab.khronos.org/spirv/SPIR-V/-/issues/779");
     SetTargetApiVersion(VK_API_VERSION_1_2);
     AddRequiredExtensions(VK_EXT_SCALAR_BLOCK_LAYOUT_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::bufferDeviceAddress);
-
     RETURN_IF_SKIP(Init());
     InitRenderTarget();
 
-    char const *vsSource = R"glsl(
-        #version 450
-        #extension GL_EXT_buffer_reference_uvec2 : enable
+    char const *vs_source = R"(
+               OpCapability Shader
+               OpCapability PhysicalStorageBufferAddresses
+               OpExtension "SPV_KHR_physical_storage_buffer"
+               OpMemoryModel PhysicalStorageBuffer64 GLSL450
+               OpEntryPoint Vertex %main "main" %outgoingPtr
+               OpDecorate %dataBuffer Block
+               OpMemberDecorate %dataBuffer 0 Offset 0
+               OpMemberDecorate %dataBuffer 1 Offset 4
+               OpDecorate %outgoingPtr Location 0
+               OpDecorate %outgoingPtr AliasedPointer
+       %void = OpTypeVoid
+          %3 = OpTypeFunction %void
+               OpTypeForwardPointer %_ptr_PhysicalStorageBuffer_dataBuffer PhysicalStorageBuffer
+        %int = OpTypeInt 32 1
+ %dataBuffer = OpTypeStruct %int %int
+%_ptr_PhysicalStorageBuffer_dataBuffer = OpTypePointer PhysicalStorageBuffer %dataBuffer
+%_ptr_Output__ptr_PhysicalStorageBuffer_dataBuffer = OpTypePointer Output %_ptr_PhysicalStorageBuffer_dataBuffer
+%outgoingPtr = OpVariable %_ptr_Output__ptr_PhysicalStorageBuffer_dataBuffer Output
+       %uint = OpTypeInt 32 0
+     %v2uint = OpTypeVector %uint 2
+     %uint_2 = OpConstant %uint 2
+         %14 = OpConstantComposite %v2uint %uint_2 %uint_2
+       %main = OpFunction %void None %3
+          %5 = OpLabel
+         %15 = OpBitcast %_ptr_PhysicalStorageBuffer_dataBuffer %14
+               OpStore %outgoingPtr %15
+               OpReturn
+               OpFunctionEnd
+    )";
 
-        layout(set=0, binding=0) layout(buffer_reference, std430) buffer dataBuffer {
-            highp int value1;
-            highp int value2;
-        };
-
-        layout(location=0) out dataBuffer outgoingPtr;
-        void main() {
-            outgoingPtr = dataBuffer(uvec2(2.0));
-        }
-    )glsl";
-
-    char const *fsSource = R"glsl(
-        #version 450
-        #extension GL_EXT_buffer_reference_uvec2 : enable
-
-        layout(set=0, binding=0) layout(buffer_reference, std430) buffer dataBuffer {
-            highp int value1;
-            highp int value2;
-        };
-
-        layout(location=0) in dataBuffer incomingPtr;
-        layout(location=0) out highp vec4 fragColor;
-        void main() {
-            highp ivec2 v = ivec2(incomingPtr.value1, incomingPtr.value2);
-            fragColor = vec4(float(v.x)/255.0,float(v.y)/255.0, float(v.x+v.y)/255.0,1.0);
-        }
-    )glsl";
-
-    m_errorMonitor->SetDesiredWarning("WARNING-PhysicalStorageBuffer-interface");
-    VkShaderObj vs(this, vsSource, VK_SHADER_STAGE_VERTEX_BIT);
-    m_errorMonitor->VerifyFound();
-    m_errorMonitor->SetDesiredWarning("WARNING-PhysicalStorageBuffer-interface");
-    VkShaderObj fs(this, fsSource, VK_SHADER_STAGE_FRAGMENT_BIT);
+    m_errorMonitor->SetDesiredWarning("VUID-StandaloneSpirv-Input-09557");
+    VkShaderObj vs(this, vs_source, VK_SHADER_STAGE_VERTEX_BIT, SPV_ENV_VULKAN_1_0, SPV_SOURCE_ASM);
     m_errorMonitor->VerifyFound();
 }
 
@@ -2224,7 +2217,7 @@ TEST_F(NegativeShaderInterface, DISABLED_MultipleFragmentAttachment) {
     pipe.shader_stages_[1] = fs.GetStageCreateInfo();
     pipe.cb_ci_.attachmentCount = 3;
     pipe.cb_ci_.pAttachments = color_blends;
-    pipe.gp_ci_.renderPass = rp.Handle();
+    pipe.gp_ci_.renderPass = rp;
     m_errorMonitor->SetDesiredWarning("Undefined-Value-ShaderOutputNotProduced");
     pipe.CreateGraphicsPipeline();
     m_errorMonitor->VerifyFound();
@@ -2282,9 +2275,9 @@ TEST_F(NegativeShaderInterface, DISABLED_MultipleFragmentAttachmentDynamicRender
 
     m_command_buffer.Begin();
     m_command_buffer.BeginRendering(rendering_info);
-    vk::CmdBindPipeline(m_command_buffer.handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipe.Handle());
+    vk::CmdBindPipeline(m_command_buffer, VK_PIPELINE_BIND_POINT_GRAPHICS, pipe);
     m_errorMonitor->SetDesiredWarning("Undefined-Value-ShaderOutputNotProduced-DynamicRendering");
-    vk::CmdDraw(m_command_buffer.handle(), 3, 1, 0, 0);
+    vk::CmdDraw(m_command_buffer, 3, 1, 0, 0);
     m_errorMonitor->VerifyFound();
     m_command_buffer.EndRendering();
     m_command_buffer.End();
@@ -2343,7 +2336,7 @@ TEST_F(NegativeShaderInterface, MissingInputAttachmentIndex) {
     CreatePipelineHelper pipe(*this);
     pipe.shader_stages_ = {pipe.vs_->GetStageCreateInfo(), fs.GetStageCreateInfo()};
     pipe.dsl_bindings_[0] = {0, VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT, 1, VK_SHADER_STAGE_FRAGMENT_BIT, nullptr};
-    pipe.gp_ci_.renderPass = rp.Handle();
+    pipe.gp_ci_.renderPass = rp;
     m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-None-09558");
     pipe.CreateGraphicsPipeline();
     m_errorMonitor->VerifyFound();
@@ -2467,164 +2460,171 @@ TEST_F(NegativeShaderInterface, MissingInputAttachmentIndexShaderObject) {
     m_errorMonitor->VerifyFound();
 }
 
-// Re-enable when https://github.com/KhronosGroup/SPIRV-Tools/pull/6000 is merged
-TEST_F(NegativeShaderInterface, DISABLED_PhysicalStorageBufferArray) {
+TEST_F(NegativeShaderInterface, PhysicalStorageBufferArray) {
+    TEST_DESCRIPTION("Details found in https://gitlab.khronos.org/spirv/SPIR-V/-/issues/779");
     SetTargetApiVersion(VK_API_VERSION_1_2);
     AddRequiredExtensions(VK_EXT_SCALAR_BLOCK_LAYOUT_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::bufferDeviceAddress);
     RETURN_IF_SKIP(Init())
     InitRenderTarget();
 
-    char const *vs_source = R"glsl(
-        #version 450
-        #extension GL_EXT_buffer_reference_uvec2 : enable
+    char const *vs_source = R"(
+               OpCapability Shader
+               OpCapability PhysicalStorageBufferAddresses
+               OpExtension "SPV_KHR_physical_storage_buffer"
+               OpMemoryModel PhysicalStorageBuffer64 GLSL450
+               OpEntryPoint Vertex %main "main" %outgoingPtr
+               OpDecorate %dataBuffer Block
+               OpMemberDecorate %dataBuffer 0 Offset 0
+               OpMemberDecorate %dataBuffer 1 Offset 4
+               OpDecorate %outgoingPtr Location 0
+               OpDecorate %outgoingPtr AliasedPointer
+       %void = OpTypeVoid
+          %3 = OpTypeFunction %void
+               OpTypeForwardPointer %_ptr_PhysicalStorageBuffer_dataBuffer PhysicalStorageBuffer
+        %int = OpTypeInt 32 1
+ %dataBuffer = OpTypeStruct %int %int
+%_ptr_PhysicalStorageBuffer_dataBuffer = OpTypePointer PhysicalStorageBuffer %dataBuffer
+       %uint = OpTypeInt 32 0
+     %uint_3 = OpConstant %uint 3
+%_arr__ptr_PhysicalStorageBuffer_dataBuffer_uint_3 = OpTypeArray %_ptr_PhysicalStorageBuffer_dataBuffer %uint_3
+%_ptr_Output__arr__ptr_PhysicalStorageBuffer_dataBuffer_uint_3 = OpTypePointer Output %_arr__ptr_PhysicalStorageBuffer_dataBuffer_uint_3
+%outgoingPtr = OpVariable %_ptr_Output__arr__ptr_PhysicalStorageBuffer_dataBuffer_uint_3 Output
+      %int_2 = OpConstant %int 2
+     %v2uint = OpTypeVector %uint 2
+     %uint_2 = OpConstant %uint 2
+         %17 = OpConstantComposite %v2uint %uint_2 %uint_2
+%_ptr_Output__ptr_PhysicalStorageBuffer_dataBuffer = OpTypePointer Output %_ptr_PhysicalStorageBuffer_dataBuffer
+       %main = OpFunction %void None %3
+          %5 = OpLabel
+         %18 = OpBitcast %_ptr_PhysicalStorageBuffer_dataBuffer %17
+         %20 = OpAccessChain %_ptr_Output__ptr_PhysicalStorageBuffer_dataBuffer %outgoingPtr %int_2
+               OpStore %20 %18
+               OpReturn
+               OpFunctionEnd
+    )";
 
-        layout(set=0, binding=0) layout(buffer_reference, std430) buffer dataBuffer {
-            int value1;
-            int value2;
-        };
-
-        layout(location=0) out dataBuffer outgoingPtr[3];
-        void main() {
-            outgoingPtr[2] = dataBuffer(uvec2(2.0));
-        }
-    )glsl";
-
-    char const *fs_source = R"glsl(
-        #version 450
-        #extension GL_EXT_buffer_reference_uvec2 : enable
-
-        layout(set=0, binding=0) layout(buffer_reference, std430) buffer dataBuffer {
-            int value1;
-            int value2;
-        };
-
-        layout(location=0) in dataBuffer incomingPtr[3];
-        layout(location=0) out vec4 fragColor;
-        void main() {
-            ivec2 v = ivec2(incomingPtr[1].value1, incomingPtr[2].value2);
-            fragColor = vec4(float(v.x)/255.0,float(v.y)/255.0, float(v.x+v.y)/255.0,1.0);
-        }
-    )glsl";
-
-    m_errorMonitor->SetDesiredWarning("WARNING-PhysicalStorageBuffer-interface");
-    VkShaderObj vs(this, vs_source, VK_SHADER_STAGE_VERTEX_BIT);
-    m_errorMonitor->VerifyFound();
-    m_errorMonitor->SetDesiredWarning("WARNING-PhysicalStorageBuffer-interface");
-    VkShaderObj fs(this, fs_source, VK_SHADER_STAGE_FRAGMENT_BIT);
+    m_errorMonitor->SetDesiredWarning("VUID-StandaloneSpirv-Input-09557");
+    VkShaderObj vs(this, vs_source, VK_SHADER_STAGE_VERTEX_BIT, SPV_ENV_VULKAN_1_0, SPV_SOURCE_ASM);
     m_errorMonitor->VerifyFound();
 }
 
-// Re-enable when https://github.com/KhronosGroup/SPIRV-Tools/pull/6000 is merged
-TEST_F(NegativeShaderInterface, DISABLED_PhysicalStorageBufferLinkedList) {
+TEST_F(NegativeShaderInterface, PhysicalStorageBufferLinkedList) {
+    TEST_DESCRIPTION("Details found in https://gitlab.khronos.org/spirv/SPIR-V/-/issues/779");
     SetTargetApiVersion(VK_API_VERSION_1_2);
     AddRequiredExtensions(VK_EXT_SCALAR_BLOCK_LAYOUT_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::bufferDeviceAddress);
     RETURN_IF_SKIP(Init())
     InitRenderTarget();
 
-    char const *vs_source = R"glsl(
-        #version 450
-        #extension GL_EXT_buffer_reference : enable
+    char const *fs_source = R"(
+               OpCapability Shader
+               OpCapability PhysicalStorageBufferAddresses
+               OpExtension "SPV_KHR_physical_storage_buffer"
+               OpMemoryModel PhysicalStorageBuffer64 GLSL450
+               OpEntryPoint Fragment %main "main" %fragColor %incomingPtr
+               OpExecutionMode %main OriginUpperLeft
+               OpDecorate %fragColor Location 0
+               OpDecorate %dataBuffer Block
+               OpMemberDecorate %dataBuffer 0 Offset 0
+               OpMemberDecorate %dataBuffer 1 Offset 16
+               OpDecorate %incomingPtr Location 0
+               OpDecorate %incomingPtr AliasedPointer
+       %void = OpTypeVoid
+          %3 = OpTypeFunction %void
+      %float = OpTypeFloat 32
+    %v4float = OpTypeVector %float 4
+%_ptr_Output_v4float = OpTypePointer Output %v4float
+  %fragColor = OpVariable %_ptr_Output_v4float Output
+               OpTypeForwardPointer %_ptr_PhysicalStorageBuffer_dataBuffer PhysicalStorageBuffer
+ %dataBuffer = OpTypeStruct %v4float %_ptr_PhysicalStorageBuffer_dataBuffer
+%_ptr_PhysicalStorageBuffer_dataBuffer = OpTypePointer PhysicalStorageBuffer %dataBuffer
+%_ptr_Input__ptr_PhysicalStorageBuffer_dataBuffer = OpTypePointer Input %_ptr_PhysicalStorageBuffer_dataBuffer
+%incomingPtr = OpVariable %_ptr_Input__ptr_PhysicalStorageBuffer_dataBuffer Input
+        %int = OpTypeInt 32 1
+      %int_0 = OpConstant %int 0
+%_ptr_PhysicalStorageBuffer_v4float = OpTypePointer PhysicalStorageBuffer %v4float
+       %main = OpFunction %void None %3
+          %5 = OpLabel
+         %14 = OpLoad %_ptr_PhysicalStorageBuffer_dataBuffer %incomingPtr
+         %18 = OpAccessChain %_ptr_PhysicalStorageBuffer_v4float %14 %int_0
+         %19 = OpLoad %v4float %18 Aligned 16
+               OpStore %fragColor %19
+               OpReturn
+               OpFunctionEnd
+    )";
 
-        layout(buffer_reference) buffer dataBuffer;
-        layout(set=0, binding=0) layout(buffer_reference, std430) buffer dataBuffer {
-            int value1;
-            dataBuffer next;
-        };
-
-        layout(location=0) out dataBuffer outgoingPtr;
-        void main() {
-            outgoingPtr.next.value1 = 3;
-        }
-    )glsl";
-
-    char const *fs_source = R"glsl(
-        #version 450
-        #extension GL_EXT_buffer_reference : enable
-
-        layout(buffer_reference) buffer dataBuffer;
-        layout(set=0, binding=0) layout(buffer_reference, std430) buffer dataBuffer {
-            int value1;
-            dataBuffer next;
-        };
-
-        layout(location=0) in dataBuffer incomingPtr;
-        layout(location=0) out vec4 fragColor;
-        void main() {
-            ivec2 v = ivec2(incomingPtr.value1, incomingPtr.next.value1);
-            fragColor = vec4(float(v.x)/255.0,float(v.y)/255.0, float(v.x+v.y)/255.0,1.0);
-        }
-    )glsl";
-
-    m_errorMonitor->SetDesiredWarning("WARNING-PhysicalStorageBuffer-interface");
-    VkShaderObj vs(this, vs_source, VK_SHADER_STAGE_VERTEX_BIT);
-    m_errorMonitor->VerifyFound();
-    m_errorMonitor->SetDesiredWarning("WARNING-PhysicalStorageBuffer-interface");
-    VkShaderObj fs(this, fs_source, VK_SHADER_STAGE_FRAGMENT_BIT);
+    m_errorMonitor->SetDesiredWarning("VUID-StandaloneSpirv-Input-09557");
+    VkShaderObj fs(this, fs_source, VK_SHADER_STAGE_FRAGMENT_BIT, SPV_ENV_VULKAN_1_0, SPV_SOURCE_ASM);
     m_errorMonitor->VerifyFound();
 }
 
-// Re-enable when https://github.com/KhronosGroup/SPIRV-Tools/pull/6000 is merged
-TEST_F(NegativeShaderInterface, DISABLED_PhysicalStorageBufferNested) {
+TEST_F(NegativeShaderInterface, PhysicalStorageBufferNested) {
+    TEST_DESCRIPTION("Details found in https://gitlab.khronos.org/spirv/SPIR-V/-/issues/779");
     SetTargetApiVersion(VK_API_VERSION_1_2);
     AddRequiredExtensions(VK_EXT_SCALAR_BLOCK_LAYOUT_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::bufferDeviceAddress);
     RETURN_IF_SKIP(Init())
     InitRenderTarget();
 
-    char const *vs_source = R"glsl(
-        #version 450
-        #extension GL_EXT_buffer_reference : enable
+    char const *fs_source = R"(
+               OpCapability Shader
+               OpCapability PhysicalStorageBufferAddresses
+               OpExtension "SPV_KHR_physical_storage_buffer"
+               OpMemoryModel PhysicalStorageBuffer64 GLSL450
+               OpEntryPoint Fragment %main "main" %fragColor %incomingPtr
+               OpExecutionMode %main OriginUpperLeft
+               OpDecorate %fragColor Location 0
+               OpDecorate %dataBuffer Block
+               OpMemberDecorate %dataBuffer 0 Offset 0
+               OpDecorate %t1 Block
+               OpMemberDecorate %t1 0 NonWritable
+               OpMemberDecorate %t1 0 Offset 0
+               OpDecorate %_runtimearr_v4float ArrayStride 16
+               OpDecorate %t2 Block
+               OpMemberDecorate %t2 0 NonWritable
+               OpMemberDecorate %t2 0 Offset 0
+               OpDecorate %incomingPtr Location 0
+               OpDecorate %incomingPtr AliasedPointer
+       %void = OpTypeVoid
+          %3 = OpTypeFunction %void
+      %float = OpTypeFloat 32
+    %v4float = OpTypeVector %float 4
+%_ptr_Output_v4float = OpTypePointer Output %v4float
+  %fragColor = OpVariable %_ptr_Output_v4float Output
+               OpTypeForwardPointer %_ptr_PhysicalStorageBuffer_dataBuffer PhysicalStorageBuffer
+               OpTypeForwardPointer %_ptr_PhysicalStorageBuffer_t1 PhysicalStorageBuffer
+ %dataBuffer = OpTypeStruct %_ptr_PhysicalStorageBuffer_t1
+               OpTypeForwardPointer %_ptr_PhysicalStorageBuffer_t2 PhysicalStorageBuffer
+         %t1 = OpTypeStruct %_ptr_PhysicalStorageBuffer_t2
+%_runtimearr_v4float = OpTypeRuntimeArray %v4float
+         %t2 = OpTypeStruct %_runtimearr_v4float
+%_ptr_PhysicalStorageBuffer_t2 = OpTypePointer PhysicalStorageBuffer %t2
+%_ptr_PhysicalStorageBuffer_t1 = OpTypePointer PhysicalStorageBuffer %t1
+%_ptr_PhysicalStorageBuffer_dataBuffer = OpTypePointer PhysicalStorageBuffer %dataBuffer
+%_ptr_Input__ptr_PhysicalStorageBuffer_dataBuffer = OpTypePointer Input %_ptr_PhysicalStorageBuffer_dataBuffer
+%incomingPtr = OpVariable %_ptr_Input__ptr_PhysicalStorageBuffer_dataBuffer Input
+        %int = OpTypeInt 32 1
+      %int_0 = OpConstant %int 0
+%_ptr_PhysicalStorageBuffer__ptr_PhysicalStorageBuffer_t1 = OpTypePointer PhysicalStorageBuffer %_ptr_PhysicalStorageBuffer_t1
+%_ptr_PhysicalStorageBuffer__ptr_PhysicalStorageBuffer_t2 = OpTypePointer PhysicalStorageBuffer %_ptr_PhysicalStorageBuffer_t2
+      %int_2 = OpConstant %int 2
+%_ptr_PhysicalStorageBuffer_v4float = OpTypePointer PhysicalStorageBuffer %v4float
+       %main = OpFunction %void None %3
+          %5 = OpLabel
+         %19 = OpLoad %_ptr_PhysicalStorageBuffer_dataBuffer %incomingPtr
+         %23 = OpAccessChain %_ptr_PhysicalStorageBuffer__ptr_PhysicalStorageBuffer_t1 %19 %int_0
+         %24 = OpLoad %_ptr_PhysicalStorageBuffer_t1 %23 Aligned 4
+         %26 = OpAccessChain %_ptr_PhysicalStorageBuffer__ptr_PhysicalStorageBuffer_t2 %24 %int_0
+         %27 = OpLoad %_ptr_PhysicalStorageBuffer_t2 %26 Aligned 4
+         %30 = OpAccessChain %_ptr_PhysicalStorageBuffer_v4float %27 %int_0 %int_2
+         %31 = OpLoad %v4float %30 Aligned 4
+               OpStore %fragColor %31
+               OpReturn
+               OpFunctionEnd
+    )";
 
-        layout(buffer_reference, buffer_reference_align = 4) readonly buffer t2 {
-            int values[];
-        };
-
-        layout(buffer_reference, buffer_reference_align = 4) readonly buffer t1 {
-            t2 c;
-        };
-
-        layout(set=0, binding=0) layout(buffer_reference, std430) buffer dataBuffer {
-            int value1;
-            t1 next;
-        };
-
-        layout(location=0) out dataBuffer outgoingPtr;
-        void main() {
-            outgoingPtr.next.c.values[3] = 3;
-        }
-    )glsl";
-
-    char const *fs_source = R"glsl(
-        #version 450
-        #extension GL_EXT_buffer_reference : enable
-
-        layout(buffer_reference, buffer_reference_align = 4) readonly buffer t2 {
-            int values[];
-        };
-
-        layout(buffer_reference, buffer_reference_align = 4) readonly buffer t1 {
-            t2 c;
-        };
-
-        layout(set=0, binding=0) layout(buffer_reference, std430) buffer dataBuffer {
-            int value1;
-            t1 next;
-        };
-
-        layout(location=0) in dataBuffer incomingPtr;
-        layout(location=0) out vec4 fragColor;
-        void main() {
-            ivec2 v = ivec2(incomingPtr.value1, incomingPtr.next.c.values[2]);
-            fragColor = vec4(float(v.x)/255.0,float(v.y)/255.0, float(v.x+v.y)/255.0,1.0);
-        }
-    )glsl";
-
-    m_errorMonitor->SetDesiredWarning("WARNING-PhysicalStorageBuffer-interface");
-    VkShaderObj vs(this, vs_source, VK_SHADER_STAGE_VERTEX_BIT);
-    m_errorMonitor->VerifyFound();
-    m_errorMonitor->SetDesiredWarning("WARNING-PhysicalStorageBuffer-interface");
-    VkShaderObj fs(this, fs_source, VK_SHADER_STAGE_FRAGMENT_BIT);
+    m_errorMonitor->SetDesiredWarning("VUID-StandaloneSpirv-Input-09557");
+    VkShaderObj fs(this, fs_source, VK_SHADER_STAGE_FRAGMENT_BIT, SPV_ENV_VULKAN_1_0, SPV_SOURCE_ASM);
     m_errorMonitor->VerifyFound();
 }
