@@ -176,7 +176,7 @@ TEST_F(NegativeSparseImage, ResidencyFlag) {
     image_memory_bind.extent = image_create_info.extent;
 
     VkSparseImageMemoryBindInfo image_memory_bind_info = {};
-    image_memory_bind_info.image = image.handle();
+    image_memory_bind_info.image = image;
     image_memory_bind_info.bindCount = 1;
     image_memory_bind_info.pBinds = &image_memory_bind;
 
@@ -336,7 +336,7 @@ TEST_F(NegativeSparseImage, QueueBindSparseMemoryType) {
     vkt::Buffer buffer(*m_device, buffer_create_info, vkt::no_mem);
 
     VkMemoryRequirements buffer_mem_reqs;
-    vk::GetBufferMemoryRequirements(device(), buffer.handle(), &buffer_mem_reqs);
+    vk::GetBufferMemoryRequirements(device(), buffer, &buffer_mem_reqs);
     const bool buffer_supports_all_mem_types = (buffer_mem_reqs.memoryTypeBits & mem_types_mask) == mem_types_mask;
     VkMemoryAllocateInfo buffer_mem_alloc = vku::InitStructHelper();
     buffer_mem_alloc.allocationSize = buffer_mem_reqs.size;
@@ -386,7 +386,7 @@ TEST_F(NegativeSparseImage, QueueBindSparseMemoryType) {
     vkt::Image image(*m_device, image_create_info, vkt::no_mem);
 
     VkMemoryRequirements image_mem_reqs;
-    vk::GetImageMemoryRequirements(device(), image.handle(), &image_mem_reqs);
+    vk::GetImageMemoryRequirements(device(), image, &image_mem_reqs);
     const bool image_supports_all_mem_types = (image_mem_reqs.memoryTypeBits & mem_types_mask) == mem_types_mask;
     VkMemoryAllocateInfo image_mem_alloc = vku::InitStructHelper();
     image_mem_alloc.allocationSize = image_mem_reqs.size;
@@ -420,19 +420,19 @@ TEST_F(NegativeSparseImage, QueueBindSparseMemoryType) {
     /// Specify memory bindings
     VkSparseMemoryBind buffer_memory_bind = {};
     buffer_memory_bind.size = buffer_mem_reqs.size;
-    buffer_memory_bind.memory = buffer_mem.handle();
+    buffer_memory_bind.memory = buffer_mem;
 
     VkSparseMemoryBind image_memory_bind = {};
     image_memory_bind.size = image_mem_reqs.size;
-    image_memory_bind.memory = image_mem.handle();
+    image_memory_bind.memory = image_mem;
 
     VkSparseBufferMemoryBindInfo buffer_memory_bind_info = {};
-    buffer_memory_bind_info.buffer = buffer.handle();
+    buffer_memory_bind_info.buffer = buffer;
     buffer_memory_bind_info.bindCount = 1;
     buffer_memory_bind_info.pBinds = &buffer_memory_bind;
 
     VkSparseImageOpaqueMemoryBindInfo image_opaque_memory_bind_info = {};
-    image_opaque_memory_bind_info.image = image.handle();
+    image_opaque_memory_bind_info.image = image;
     image_opaque_memory_bind_info.bindCount = 1;
     image_opaque_memory_bind_info.pBinds = &image_memory_bind;
 
@@ -536,13 +536,13 @@ TEST_F(NegativeSparseImage, QueueBindSparseMemoryType2) {
     vkt::Image image(*m_device, image_create_info, vkt::no_mem);
 
     VkMemoryRequirements buffer_mem_reqs;
-    vk::GetBufferMemoryRequirements(device(), buffer.handle(), &buffer_mem_reqs);
+    vk::GetBufferMemoryRequirements(device(), buffer, &buffer_mem_reqs);
     VkMemoryAllocateInfo buffer_mem_alloc = vku::InitStructHelper();
     buffer_mem_alloc.allocationSize = buffer_mem_reqs.size;
     buffer_mem_alloc.memoryTypeIndex = lazily_allocated_index;
 
     VkMemoryRequirements image_mem_reqs;
-    vk::GetImageMemoryRequirements(device(), image.handle(), &image_mem_reqs);
+    vk::GetImageMemoryRequirements(device(), image, &image_mem_reqs);
     VkMemoryAllocateInfo image_mem_alloc = vku::InitStructHelper();
     image_mem_alloc.allocationSize = image_mem_reqs.size;
     image_mem_alloc.memoryTypeIndex = lazily_allocated_index;
@@ -553,19 +553,19 @@ TEST_F(NegativeSparseImage, QueueBindSparseMemoryType2) {
 
     VkSparseMemoryBind buffer_memory_bind = {};
     buffer_memory_bind.size = buffer_mem_reqs.size;
-    buffer_memory_bind.memory = buffer_mem.handle();
+    buffer_memory_bind.memory = buffer_mem;
 
     VkSparseMemoryBind image_memory_bind = {};
     image_memory_bind.size = image_mem_reqs.size;
-    image_memory_bind.memory = image_mem.handle();
+    image_memory_bind.memory = image_mem;
 
     VkSparseBufferMemoryBindInfo buffer_memory_bind_info = {};
-    buffer_memory_bind_info.buffer = buffer.handle();
+    buffer_memory_bind_info.buffer = buffer;
     buffer_memory_bind_info.bindCount = 1;
     buffer_memory_bind_info.pBinds = &buffer_memory_bind;
 
     VkSparseImageOpaqueMemoryBindInfo image_opaque_memory_bind_info = {};
-    image_opaque_memory_bind_info.image = image.handle();
+    image_opaque_memory_bind_info.image = image;
     image_opaque_memory_bind_info.bindCount = 1;
     image_opaque_memory_bind_info.pBinds = &image_memory_bind;
 
@@ -646,7 +646,7 @@ TEST_F(NegativeSparseImage, QueueBindSparseMemoryType3) {
     VkExportMemoryAllocateInfo buffer_export_mem_alloc_info = vku::InitStructHelper();
     buffer_export_mem_alloc_info.handleTypes = GetCompatibleHandleTypes(Gpu(), buffer_create_info, buffer_exportable_type);
     VkMemoryRequirements buffer_mem_reqs{};
-    vk::GetBufferMemoryRequirements(device(), buffer.handle(), &buffer_mem_reqs);
+    vk::GetBufferMemoryRequirements(device(), buffer, &buffer_mem_reqs);
     const VkMemoryAllocateInfo buffer_mem_alloc = vkt::DeviceMemory::GetResourceAllocInfo(
         *m_device, buffer_mem_reqs, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT, &buffer_export_mem_alloc_info);
     vkt::DeviceMemory buffer_mem(*m_device, buffer_mem_alloc);
@@ -677,7 +677,7 @@ TEST_F(NegativeSparseImage, QueueBindSparseMemoryType3) {
     VkExportMemoryAllocateInfo image_export_mem_alloc_info = vku::InitStructHelper();
     image_export_mem_alloc_info.handleTypes = GetCompatibleHandleTypes(Gpu(), image_create_info, image_exportable_type);
     VkMemoryRequirements image_mem_reqs;
-    vk::GetImageMemoryRequirements(device(), image.handle(), &image_mem_reqs);
+    vk::GetImageMemoryRequirements(device(), image, &image_mem_reqs);
     const VkMemoryAllocateInfo image_mem_alloc = vkt::DeviceMemory::GetResourceAllocInfo(
         *m_device, image_mem_reqs, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT, &image_export_mem_alloc_info);
     vkt::DeviceMemory image_mem(*m_device, image_mem_alloc);
@@ -685,25 +685,25 @@ TEST_F(NegativeSparseImage, QueueBindSparseMemoryType3) {
     // Setup memory bindings
     VkSparseMemoryBind buffer_memory_bind = {};
     buffer_memory_bind.size = buffer_mem_reqs.size;
-    buffer_memory_bind.memory = buffer_mem.handle();
+    buffer_memory_bind.memory = buffer_mem;
 
     VkSparseBufferMemoryBindInfo buffer_memory_bind_info = {};
-    buffer_memory_bind_info.buffer = buffer.handle();
+    buffer_memory_bind_info.buffer = buffer;
     buffer_memory_bind_info.bindCount = 1;
     buffer_memory_bind_info.pBinds = &buffer_memory_bind;
 
     VkSparseMemoryBind image_memory_bind = {};
     image_memory_bind.size = image_mem_reqs.size;
-    image_memory_bind.memory = image_mem.handle();
+    image_memory_bind.memory = image_mem;
 
     VkSparseImageOpaqueMemoryBindInfo image_opaque_memory_bind_info = {};
-    image_opaque_memory_bind_info.image = image.handle();
+    image_opaque_memory_bind_info.image = image;
     image_opaque_memory_bind_info.bindCount = 1;
     image_opaque_memory_bind_info.pBinds = &image_memory_bind;
 
     VkSparseImageMemoryBind image_memory_bind_2 = {};
     image_memory_bind_2.extent = image_create_info.extent;
-    image_memory_bind_2.memory = image_mem.handle();
+    image_memory_bind_2.memory = image_mem;
     image_memory_bind_2.subresource.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
 
     VkSparseImageMemoryBindInfo image_memory_bind_info = {};
@@ -888,7 +888,7 @@ TEST_F(NegativeSparseImage, QueueBindSparseMemoryType4) {
     vkt::Image image(*m_device, image_create_info, vkt::no_mem);
 
     VkMemoryRequirements image_mem_reqs;
-    vk::GetImageMemoryRequirements(device(), image.handle(), &image_mem_reqs);
+    vk::GetImageMemoryRequirements(device(), image, &image_mem_reqs);
     const VkMemoryAllocateInfo image_mem_alloc =
         vkt::DeviceMemory::GetResourceAllocInfo(*m_device, image_mem_reqs, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT, &import_info2);
     vkt::DeviceMemory image_mem(*m_device, image_mem_alloc);
@@ -896,25 +896,25 @@ TEST_F(NegativeSparseImage, QueueBindSparseMemoryType4) {
     // Setup memory bindings
     VkSparseMemoryBind buffer_memory_bind = {};
     buffer_memory_bind.size = buffer1.MemoryRequirements().size;
-    buffer_memory_bind.memory = buffer_memory_imported.handle();
+    buffer_memory_bind.memory = buffer_memory_imported;
 
     VkSparseBufferMemoryBindInfo buffer_memory_bind_info = {};
-    buffer_memory_bind_info.buffer = buffer1.handle();
+    buffer_memory_bind_info.buffer = buffer1;
     buffer_memory_bind_info.bindCount = 1;
     buffer_memory_bind_info.pBinds = &buffer_memory_bind;
 
     VkSparseMemoryBind image_memory_bind = {};
     image_memory_bind.size = image_mem_reqs.size;
-    image_memory_bind.memory = image_mem.handle();
+    image_memory_bind.memory = image_mem;
 
     VkSparseImageOpaqueMemoryBindInfo image_opaque_memory_bind_info = {};
-    image_opaque_memory_bind_info.image = image.handle();
+    image_opaque_memory_bind_info.image = image;
     image_opaque_memory_bind_info.bindCount = 1;
     image_opaque_memory_bind_info.pBinds = &image_memory_bind;
 
     VkSparseImageMemoryBind image_memory_bind_2 = {};
     image_memory_bind_2.extent = image_create_info.extent;
-    image_memory_bind_2.memory = image_mem.handle();
+    image_memory_bind_2.memory = image_mem;
     image_memory_bind_2.subresource.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
 
     VkSparseImageMemoryBindInfo image_memory_bind_info = {};
@@ -976,43 +976,37 @@ TEST_F(NegativeSparseImage, ImageMemoryBind) {
         GTEST_SKIP() << "Required SPARSE_BINDING queue families not present";
     }
 
-    VkImageCreateInfo create_info = vkt::Image::CreateInfo();
+    VkImageCreateInfo create_info =
+        vkt::Image::ImageCreateInfo2D(1024, 1024, 1, 1, VK_FORMAT_B8G8R8A8_UNORM, VK_IMAGE_USAGE_SAMPLED_BIT);
     create_info.flags = VK_IMAGE_CREATE_SPARSE_BINDING_BIT | VK_IMAGE_CREATE_SPARSE_RESIDENCY_BIT;
     create_info.imageType = VK_IMAGE_TYPE_3D;
-    create_info.usage = VK_IMAGE_USAGE_SAMPLED_BIT;
-    create_info.format = VK_FORMAT_B8G8R8A8_UNORM;
-    create_info.extent.width = 1024;
-    create_info.extent.height = 1024;
-    create_info.extent.depth = 1;
-    create_info.arrayLayers = 1;
     vkt::Image image(*m_device, create_info, vkt::no_mem);
 
     VkMemoryRequirements image_mem_reqs;
-    vk::GetImageMemoryRequirements(m_device->handle(), image.handle(), &image_mem_reqs);
+    vk::GetImageMemoryRequirements(*m_device, image, &image_mem_reqs);
     const auto image_mem_alloc =
         vkt::DeviceMemory::GetResourceAllocInfo(*m_device, image_mem_reqs, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
 
-    vkt::DeviceMemory image_mem;
-    image_mem.init(*m_device, image_mem_alloc);
+    vkt::DeviceMemory image_mem(*m_device, image_mem_alloc);
 
     uint32_t requirements_count = 0u;
-    vk::GetImageSparseMemoryRequirements(m_device->handle(), image.handle(), &requirements_count, nullptr);
+    vk::GetImageSparseMemoryRequirements(*m_device, image, &requirements_count, nullptr);
 
     if (requirements_count == 0u) {
         GTEST_SKIP() << "No sparse image requirements for image format VK_FORMAT_B8G8R8A8_UNORM";
     }
 
     std::vector<VkSparseImageMemoryRequirements> sparse_reqs(requirements_count);
-    vk::GetImageSparseMemoryRequirements(m_device->handle(), image.handle(), &requirements_count, sparse_reqs.data());
+    vk::GetImageSparseMemoryRequirements(*m_device, image, &requirements_count, sparse_reqs.data());
 
     VkExtent3D granularity = sparse_reqs[0].formatProperties.imageGranularity;
     VkSparseImageMemoryBind image_bind{};
     image_bind.subresource.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
-    image_bind.memory = image_mem.handle();
+    image_bind.memory = image_mem;
     image_bind.extent = granularity;
 
     VkSparseImageMemoryBindInfo image_bind_info{};
-    image_bind_info.image = image.handle();
+    image_bind_info.image = image;
     image_bind_info.bindCount = 1u;
     image_bind_info.pBinds = &image_bind;
 
@@ -1097,30 +1091,24 @@ TEST_F(NegativeSparseImage, ImageMemoryBindInvalidMemory) {
         GTEST_SKIP() << "Required SPARSE_BINDING queue families not present";
     }
 
-    VkImageCreateInfo create_info = vkt::Image::CreateInfo();
+    VkImageCreateInfo create_info =
+        vkt::Image::ImageCreateInfo2D(1024, 1024, 1, 1, VK_FORMAT_B8G8R8A8_UNORM, VK_IMAGE_USAGE_SAMPLED_BIT);
     create_info.flags = VK_IMAGE_CREATE_SPARSE_BINDING_BIT | VK_IMAGE_CREATE_SPARSE_RESIDENCY_BIT;
     create_info.imageType = VK_IMAGE_TYPE_3D;
-    create_info.usage = VK_IMAGE_USAGE_SAMPLED_BIT;
-    create_info.format = VK_FORMAT_B8G8R8A8_UNORM;
-    create_info.extent.width = 1024;
-    create_info.extent.height = 1024;
-    create_info.extent.depth = 1;
-    create_info.arrayLayers = 1;
     vkt::Image image(*m_device, create_info, vkt::no_mem);
 
     VkMemoryRequirements image_mem_reqs;
-    vk::GetImageMemoryRequirements(m_device->handle(), image.handle(), &image_mem_reqs);
+    vk::GetImageMemoryRequirements(*m_device, image, &image_mem_reqs);
     const auto image_mem_alloc =
         vkt::DeviceMemory::GetResourceAllocInfo(*m_device, image_mem_reqs, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
 
-    vkt::DeviceMemory image_mem;
-    image_mem.init(*m_device, image_mem_alloc);
+    vkt::DeviceMemory image_mem(*m_device, image_mem_alloc);
 
     VkImageCreateInfo invalid_create_info = create_info;
     vkt::Image invalid_image(*m_device, invalid_create_info, vkt::no_mem);
 
     VkMemoryRequirements invalid_image_mem_reqs;
-    vk::GetImageMemoryRequirements(m_device->handle(), invalid_image.handle(), &invalid_image_mem_reqs);
+    vk::GetImageMemoryRequirements(*m_device, invalid_image, &invalid_image_mem_reqs);
 
     // Make sure that the same memory type is not chosen.
     invalid_image_mem_reqs.memoryTypeBits = ~image_mem_reqs.memoryTypeBits;
@@ -1133,28 +1121,27 @@ TEST_F(NegativeSparseImage, ImageMemoryBindInvalidMemory) {
         GTEST_SKIP() << "Could not find required memory type";
     }
 
-    vkt::DeviceMemory invalid_image_mem;
-    invalid_image_mem.init(*m_device, invalid_image_mem_alloc);
+    vkt::DeviceMemory invalid_image_mem(*m_device, invalid_image_mem_alloc);
 
     uint32_t requirements_count = 0u;
-    vk::GetImageSparseMemoryRequirements(m_device->handle(), image.handle(), &requirements_count, nullptr);
+    vk::GetImageSparseMemoryRequirements(*m_device, image, &requirements_count, nullptr);
 
     if (requirements_count == 0u) {
         GTEST_SKIP() << "No sparse image requirements for image format VK_FORMAT_B8G8R8A8_UNORM";
     }
 
     std::vector<VkSparseImageMemoryRequirements> sparse_reqs(requirements_count);
-    vk::GetImageSparseMemoryRequirements(m_device->handle(), image.handle(), &requirements_count, sparse_reqs.data());
+    vk::GetImageSparseMemoryRequirements(*m_device, image, &requirements_count, sparse_reqs.data());
 
     VkExtent3D granularity = sparse_reqs[0].formatProperties.imageGranularity;
     VkSparseImageMemoryBind image_bind{};
     image_bind.subresource.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
-    image_bind.memory = image_mem.handle();
+    image_bind.memory = image_mem;
     image_bind.memoryOffset = 0;
     image_bind.extent = granularity;
 
     VkSparseImageMemoryBindInfo image_bind_info{};
-    image_bind_info.image = image.handle();
+    image_bind_info.image = image;
     image_bind_info.bindCount = 1u;
     image_bind_info.pBinds = &image_bind;
 
@@ -1165,7 +1152,7 @@ TEST_F(NegativeSparseImage, ImageMemoryBindInvalidMemory) {
     VkQueue sparse_queue = m_device->QueuesWithSparseCapability()[0]->handle();
 
     // Force invalid device memory
-    image_bind.memory = invalid_image_mem.handle();
+    image_bind.memory = invalid_image_mem;
     m_errorMonitor->SetDesiredError("VUID-VkSparseImageMemoryBind-memory-01105");
     vk::QueueBindSparse(sparse_queue, 1, &bind_info, VK_NULL_HANDLE);
     m_errorMonitor->VerifyFound();
@@ -1182,19 +1169,14 @@ TEST_F(NegativeSparseImage, ImageMemoryBindInvalidAlignment) {
         GTEST_SKIP() << "Required SPARSE_BINDING queue families not present";
     }
 
-    VkImageCreateInfo create_info = vkt::Image::CreateInfo();
+    VkImageCreateInfo create_info =
+        vkt::Image::ImageCreateInfo2D(1024, 1024, 1, 1, VK_FORMAT_B8G8R8A8_UNORM, VK_IMAGE_USAGE_SAMPLED_BIT);
     create_info.flags = VK_IMAGE_CREATE_SPARSE_BINDING_BIT | VK_IMAGE_CREATE_SPARSE_RESIDENCY_BIT;
     create_info.imageType = VK_IMAGE_TYPE_3D;
-    create_info.usage = VK_IMAGE_USAGE_SAMPLED_BIT;
-    create_info.format = VK_FORMAT_B8G8R8A8_UNORM;
-    create_info.extent.width = 1024;
-    create_info.extent.height = 1024;
-    create_info.extent.depth = 1;
-    create_info.arrayLayers = 1;
     vkt::Image image(*m_device, create_info, vkt::no_mem);
 
     VkMemoryRequirements image_mem_reqs;
-    vk::GetImageMemoryRequirements(m_device->handle(), image.handle(), &image_mem_reqs);
+    vk::GetImageMemoryRequirements(*m_device, image, &image_mem_reqs);
     if (image_mem_reqs.alignment == 1) {
         GTEST_SKIP() << "Need image memory required alignment to be more than 1";
     }
@@ -1202,14 +1184,13 @@ TEST_F(NegativeSparseImage, ImageMemoryBindInvalidAlignment) {
     const auto image_mem_alloc =
         vkt::DeviceMemory::GetResourceAllocInfo(*m_device, image_mem_reqs, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
 
-    vkt::DeviceMemory image_mem;
-    image_mem.init(*m_device, image_mem_alloc);
+    vkt::DeviceMemory image_mem(*m_device, image_mem_alloc);
 
     VkImageCreateInfo invalid_create_info = create_info;
     vkt::Image invalid_image(*m_device, invalid_create_info, vkt::no_mem);
 
     VkMemoryRequirements invalid_image_mem_reqs;
-    vk::GetImageMemoryRequirements(m_device->handle(), invalid_image.handle(), &invalid_image_mem_reqs);
+    vk::GetImageMemoryRequirements(*m_device, invalid_image, &invalid_image_mem_reqs);
 
     // Make sure that the same memory type is not chosen.
     invalid_image_mem_reqs.memoryTypeBits = ~image_mem_reqs.memoryTypeBits;
@@ -1222,28 +1203,27 @@ TEST_F(NegativeSparseImage, ImageMemoryBindInvalidAlignment) {
         GTEST_SKIP() << "Could not find required memory type";
     }
 
-    vkt::DeviceMemory invalid_image_mem;
-    invalid_image_mem.init(*m_device, invalid_image_mem_alloc);
+    vkt::DeviceMemory invalid_image_mem(*m_device, invalid_image_mem_alloc);
 
     uint32_t requirements_count = 0u;
-    vk::GetImageSparseMemoryRequirements(m_device->handle(), image.handle(), &requirements_count, nullptr);
+    vk::GetImageSparseMemoryRequirements(*m_device, image, &requirements_count, nullptr);
 
     if (requirements_count == 0u) {
         GTEST_SKIP() << "No sparse image requirements for image format VK_FORMAT_B8G8R8A8_UNORM";
     }
 
     std::vector<VkSparseImageMemoryRequirements> sparse_reqs(requirements_count);
-    vk::GetImageSparseMemoryRequirements(m_device->handle(), image.handle(), &requirements_count, sparse_reqs.data());
+    vk::GetImageSparseMemoryRequirements(*m_device, image, &requirements_count, sparse_reqs.data());
 
     VkExtent3D granularity = sparse_reqs[0].formatProperties.imageGranularity;
     VkSparseImageMemoryBind image_bind{};
     image_bind.subresource.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
-    image_bind.memory = image_mem.handle();
+    image_bind.memory = image_mem;
     image_bind.memoryOffset = 0;
     image_bind.extent = granularity;
 
     VkSparseImageMemoryBindInfo image_bind_info{};
-    image_bind_info.image = image.handle();
+    image_bind_info.image = image;
     image_bind_info.bindCount = 1u;
     image_bind_info.pBinds = &image_bind;
 
@@ -1273,43 +1253,37 @@ TEST_F(NegativeSparseImage, ImageMemoryBindInvalidExtent) {
         GTEST_SKIP() << "Required SPARSE_BINDING queue families not present";
     }
 
-    VkImageCreateInfo create_info = vkt::Image::CreateInfo();
+    VkImageCreateInfo create_info =
+        vkt::Image::ImageCreateInfo2D(1024, 1024, 1, 1, VK_FORMAT_B8G8R8A8_UNORM, VK_IMAGE_USAGE_SAMPLED_BIT);
     create_info.flags = VK_IMAGE_CREATE_SPARSE_BINDING_BIT | VK_IMAGE_CREATE_SPARSE_RESIDENCY_BIT;
     create_info.imageType = VK_IMAGE_TYPE_3D;
-    create_info.usage = VK_IMAGE_USAGE_SAMPLED_BIT;
-    create_info.format = VK_FORMAT_B8G8R8A8_UNORM;
-    create_info.extent.width = 1024;
-    create_info.extent.height = 1024;
-    create_info.extent.depth = 1;
-    create_info.arrayLayers = 1;
     vkt::Image image(*m_device, create_info, vkt::no_mem);
 
     VkMemoryRequirements image_mem_reqs;
-    vk::GetImageMemoryRequirements(m_device->handle(), image.handle(), &image_mem_reqs);
+    vk::GetImageMemoryRequirements(*m_device, image, &image_mem_reqs);
     const auto image_mem_alloc =
         vkt::DeviceMemory::GetResourceAllocInfo(*m_device, image_mem_reqs, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
 
-    vkt::DeviceMemory image_mem;
-    image_mem.init(*m_device, image_mem_alloc);
+    vkt::DeviceMemory image_mem(*m_device, image_mem_alloc);
 
     uint32_t requirements_count = 0u;
-    vk::GetImageSparseMemoryRequirements(m_device->handle(), image.handle(), &requirements_count, nullptr);
+    vk::GetImageSparseMemoryRequirements(*m_device, image, &requirements_count, nullptr);
 
     if (requirements_count == 0u) {
         GTEST_SKIP() << "No sparse image requirements for image format VK_FORMAT_B8G8R8A8_UNORM";
     }
 
     std::vector<VkSparseImageMemoryRequirements> sparse_reqs(requirements_count);
-    vk::GetImageSparseMemoryRequirements(m_device->handle(), image.handle(), &requirements_count, sparse_reqs.data());
+    vk::GetImageSparseMemoryRequirements(*m_device, image, &requirements_count, sparse_reqs.data());
 
     VkExtent3D granularity = sparse_reqs[0].formatProperties.imageGranularity;
     VkSparseImageMemoryBind image_bind{};
     image_bind.subresource.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
-    image_bind.memory = image_mem.handle();
+    image_bind.memory = image_mem;
     image_bind.extent = granularity;
 
     VkSparseImageMemoryBindInfo image_bind_info{};
-    image_bind_info.image = image.handle();
+    image_bind_info.image = image;
     image_bind_info.bindCount = 1u;
     image_bind_info.pBinds = &image_bind;
 
