@@ -1,6 +1,6 @@
-/* Copyright (c) 2024 The Khronos Group Inc.
- * Copyright (c) 2024 Valve Corporation
- * Copyright (c) 2024 LunarG, Inc.
+/* Copyright (c) 2025 The Khronos Group Inc.
+ * Copyright (c) 2025 Valve Corporation
+ * Copyright (c) 2025 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
+#include "../framework/sync_val_tests.h"
 #include <thread>
-#include "../framework/layer_validation_tests.h"
 
 struct PositiveSyncValWsi : public VkSyncValTest {};
 
@@ -58,12 +58,8 @@ TEST_F(PositiveSyncValWsi, PresentAfterSubmit2AutomaticVisibility) {
     layout_transition.subresourceRange.baseArrayLayer = 0;
     layout_transition.subresourceRange.layerCount = 1;
 
-    VkDependencyInfo dep_info = vku::InitStructHelper();
-    dep_info.imageMemoryBarrierCount = 1;
-    dep_info.pImageMemoryBarriers = &layout_transition;
-
     m_command_buffer.Begin();
-    vk::CmdPipelineBarrier2(m_command_buffer, &dep_info);
+    m_command_buffer.Barrier(layout_transition);
     m_command_buffer.End();
 
     m_default_queue->Submit2(m_command_buffer, vkt::Wait(acquire_semaphore, VK_PIPELINE_STAGE_2_COLOR_ATTACHMENT_OUTPUT_BIT),
@@ -136,12 +132,8 @@ TEST_F(PositiveSyncValWsi, PresentAfterSubmitNoneDstStage) {
     layout_transition.image = swapchain_images[image_index];
     layout_transition.subresourceRange = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1};
 
-    VkDependencyInfo dep_info = vku::InitStructHelper();
-    dep_info.imageMemoryBarrierCount = 1;
-    dep_info.pImageMemoryBarriers = &layout_transition;
-
     m_command_buffer.Begin();
-    vk::CmdPipelineBarrier2(m_command_buffer, &dep_info);
+    m_command_buffer.Barrier(layout_transition);
     m_command_buffer.End();
 
     // The goal of this test is to use QueueSubmit API (not QueueSubmit2) to
@@ -214,7 +206,12 @@ TEST_F(PositiveSyncValWsi, ThreadedSubmitAndFenceWaitAndPresent) {
     // Main thread submits empty batches and presents images
     {
         vkt::Semaphore acquire_semaphore(*m_device);
-        vkt::Semaphore submit_semaphore(*m_device);
+
+        std::vector<vkt::Semaphore> submit_semaphores;
+        for (size_t i = 0; i < swapchain_images.size(); i++) {
+            submit_semaphores.emplace_back(*m_device);
+        }
+
         vkt::Fence fence(*m_device);
 
         for (int i = 0; i < N; i++) {
@@ -222,8 +219,8 @@ TEST_F(PositiveSyncValWsi, ThreadedSubmitAndFenceWaitAndPresent) {
             {
                 std::unique_lock<std::mutex> lock(queue_mutex);
                 m_default_queue->Submit(vkt::no_cmd, vkt::Wait(acquire_semaphore, VK_PIPELINE_STAGE_2_COLOR_ATTACHMENT_OUTPUT_BIT),
-                                        vkt::Signal(submit_semaphore), fence);
-                m_default_queue->Present(m_swapchain, image_index, submit_semaphore);
+                                        vkt::Signal(submit_semaphores[image_index]), fence);
+                m_default_queue->Present(m_swapchain, image_index, submit_semaphores[image_index]);
             }
             vk::WaitForFences(device(), 1, &fence.handle(), VK_TRUE, kWaitTimeout);
             vk::ResetFences(device(), 1, &fence.handle());
@@ -238,30 +235,6 @@ TEST_F(PositiveSyncValWsi, ThreadedSubmitAndFenceWaitAndPresent) {
     thread.join();
 }
 
-// TODO: make this a shared function, since WSI tests also use it
-static void SetImageLayoutPresentSrc(vkt::Queue& queue, vkt::Device& device, VkImage image) {
-    vkt::CommandPool pool(device, device.graphics_queue_node_index_);
-    vkt::CommandBuffer cmd_buf(device, pool);
-
-    cmd_buf.Begin();
-    VkImageMemoryBarrier layout_barrier{VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER,
-                                        nullptr,
-                                        0,
-                                        VK_ACCESS_MEMORY_READ_BIT,
-                                        VK_IMAGE_LAYOUT_UNDEFINED,
-                                        VK_IMAGE_LAYOUT_PRESENT_SRC_KHR,
-                                        VK_QUEUE_FAMILY_IGNORED,
-                                        VK_QUEUE_FAMILY_IGNORED,
-                                        image,
-                                        {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1}};
-
-    vk::CmdPipelineBarrier(cmd_buf.handle(), VK_PIPELINE_STAGE_ALL_COMMANDS_BIT, VK_PIPELINE_STAGE_ALL_COMMANDS_BIT, 0, 0, nullptr,
-                           0, nullptr, 1, &layout_barrier);
-    cmd_buf.End();
-    queue.Submit(cmd_buf);
-    queue.Wait();
-}
-
 TEST_F(PositiveSyncValWsi, WaitForFencesWithPresentBatches) {
     TEST_DESCRIPTION("Check that WaitForFences applies tagged waits to present batches");
     AddSurfaceExtension();
@@ -269,7 +242,7 @@ TEST_F(PositiveSyncValWsi, WaitForFencesWithPresentBatches) {
     RETURN_IF_SKIP(InitSwapchain());
     const auto swapchain_images = m_swapchain.GetImages();
     for (auto image : swapchain_images) {
-        SetImageLayoutPresentSrc(*m_default_queue, *m_device, image);
+        SetPresentImageLayout(image);
     }
 
     vkt::Semaphore acquire_semaphore(*m_device);
@@ -341,7 +314,7 @@ TEST_F(PositiveSyncValWsi, RecreateBuffer) {
     std::vector<vkt::Buffer> dst_buffers(swapchain_images.size());
 
     for (VkImage image : swapchain_images) {
-        SetImageLayoutPresentSrc(*m_default_queue, *m_device, image);
+        SetPresentImageLayout(image);
     }
     for (size_t i = 0; i < swapchain_images.size(); i++) {
         acquire_fences.emplace_back(*m_device);
@@ -403,7 +376,7 @@ TEST_F(PositiveSyncValWsi, RecreateImage) {
     std::vector<vkt::Image> dst_images(swapchain_images.size());
 
     for (auto image : swapchain_images) {
-        SetImageLayoutPresentSrc(*m_default_queue, *m_device, image);
+        SetPresentImageLayout(image);
     }
     for (size_t i = 0; i < swapchain_images.size(); i++) {
         acquire_fences.emplace_back(*m_device);
@@ -423,7 +396,7 @@ TEST_F(PositiveSyncValWsi, RecreateImage) {
 
         auto &dst_image = dst_images[image_index];
         dst_image.destroy();
-        dst_image = vkt::Image(*m_device, width, height, 1, format, VK_IMAGE_USAGE_TRANSFER_DST_BIT);
+        dst_image = vkt::Image(*m_device, width, height, format, VK_IMAGE_USAGE_TRANSFER_DST_BIT);
 
         VkBufferImageCopy region = {};
         region.imageSubresource = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 0, 1};
@@ -439,13 +412,9 @@ TEST_F(PositiveSyncValWsi, RecreateImage) {
         layout_transition.image = dst_image;
         layout_transition.subresourceRange = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1};
 
-        VkDependencyInfo dep_info = vku::InitStructHelper();
-        dep_info.imageMemoryBarrierCount = 1;
-        dep_info.pImageMemoryBarriers = &layout_transition;
-
         auto &command_buffer = command_buffers[image_index];
         command_buffer.Begin();
-        vk::CmdPipelineBarrier2(command_buffer, &dep_info);
+        command_buffer.Barrier(layout_transition);
         vk::CmdCopyBufferToImage(command_buffer, src_buffer, dst_image, VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL, 1, &region);
         command_buffer.End();
 
