@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2015-2024 The Khronos Group Inc.
- * Copyright (c) 2015-2024 Valve Corporation
- * Copyright (c) 2015-2024 LunarG, Inc.
- * Copyright (c) 2015-2024 Google, Inc.
+ * Copyright (c) 2015-2025 The Khronos Group Inc.
+ * Copyright (c) 2015-2025 Valve Corporation
+ * Copyright (c) 2015-2025 LunarG, Inc.
+ * Copyright (c) 2015-2025 Google, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -12,7 +12,6 @@
  */
 
 #include "../framework/layer_validation_tests.h"
-#include "generated/vk_extension_helper.h"
 
 class NegativeTooling : public VkLayerTest {};
 
@@ -28,11 +27,11 @@ TEST_F(NegativeTooling, PrivateDataFeature) {
     VkPrivateDataSlotCreateInfoEXT data_create_info = vku::InitStructHelper();
     data_create_info.flags = 0;
     m_errorMonitor->SetDesiredError("VUID-vkCreatePrivateDataSlot-privateData-04564");
-    vk::CreatePrivateDataSlotEXT(m_device->handle(), &data_create_info, NULL, &data_slot);
+    vk::CreatePrivateDataSlotEXT(*m_device, &data_create_info, NULL, &data_slot);
     m_errorMonitor->VerifyFound();
     if (vulkan_13) {
         m_errorMonitor->SetDesiredError("VUID-vkCreatePrivateDataSlot-privateData-04564");
-        vk::CreatePrivateDataSlot(m_device->handle(), &data_create_info, NULL, &data_slot);
+        vk::CreatePrivateDataSlot(*m_device, &data_create_info, NULL, &data_slot);
         m_errorMonitor->VerifyFound();
     }
 }
@@ -45,18 +44,18 @@ TEST_F(NegativeTooling, PrivateDataSetNonDevice) {
 
     VkPrivateDataSlot data_slot;
     VkPrivateDataSlotCreateInfo data_create_info = vku::InitStructHelper();
-    vk::CreatePrivateDataSlot(m_device->handle(), &data_create_info, NULL, &data_slot);
+    vk::CreatePrivateDataSlot(*m_device, &data_create_info, NULL, &data_slot);
 
     static const uint64_t data_value = 0x70AD;
     m_errorMonitor->SetDesiredError("VUID-vkSetPrivateData-objectHandle-04016");
-    vk::SetPrivateData(m_device->handle(), VK_OBJECT_TYPE_PHYSICAL_DEVICE, (uint64_t)gpu(), data_slot, data_value);
+    vk::SetPrivateData(*m_device, VK_OBJECT_TYPE_PHYSICAL_DEVICE, (uint64_t)Gpu(), data_slot, data_value);
     m_errorMonitor->VerifyFound();
 
     m_errorMonitor->SetDesiredError("VUID-vkSetPrivateData-objectHandle-04016");
-    vk::SetPrivateData(m_device->handle(), VK_OBJECT_TYPE_UNKNOWN, (uint64_t)gpu(), data_slot, data_value);
+    vk::SetPrivateData(*m_device, VK_OBJECT_TYPE_UNKNOWN, (uint64_t)Gpu(), data_slot, data_value);
     m_errorMonitor->VerifyFound();
 
-    vk::DestroyPrivateDataSlot(m_device->handle(), data_slot, nullptr);
+    vk::DestroyPrivateDataSlot(*m_device, data_slot, nullptr);
 }
 
 TEST_F(NegativeTooling, PrivateDataSetBadHandle) {
@@ -67,15 +66,15 @@ TEST_F(NegativeTooling, PrivateDataSetBadHandle) {
 
     VkPrivateDataSlot data_slot;
     VkPrivateDataSlotCreateInfo data_create_info = vku::InitStructHelper();
-    vk::CreatePrivateDataSlot(m_device->handle(), &data_create_info, NULL, &data_slot);
+    vk::CreatePrivateDataSlot(*m_device, &data_create_info, NULL, &data_slot);
 
     static const uint64_t data_value = 0x70AD;
     m_errorMonitor->SetDesiredError("VUID-vkSetPrivateData-objectHandle-04017");
     // valid handle, but not a vkSample
-    vk::SetPrivateData(m_device->handle(), VK_OBJECT_TYPE_SAMPLER, (uint64_t)device(), data_slot, data_value);
+    vk::SetPrivateData(*m_device, VK_OBJECT_TYPE_SAMPLER, (uint64_t)device(), data_slot, data_value);
     m_errorMonitor->VerifyFound();
 
-    vk::DestroyPrivateDataSlot(m_device->handle(), data_slot, nullptr);
+    vk::DestroyPrivateDataSlot(*m_device, data_slot, nullptr);
 }
 
 TEST_F(NegativeTooling, PrivateDataSetSecondDevice) {
@@ -84,24 +83,24 @@ TEST_F(NegativeTooling, PrivateDataSetSecondDevice) {
     AddRequiredFeature(vkt::Feature::privateData);
     RETURN_IF_SKIP(Init());
 
-    auto features = m_device->phy().features();
-    vkt::Device second_device(gpu_, m_device_extension_names, &features, nullptr);
+    auto features = m_device->Physical().Features();
+    vkt::Device second_device(gpu_, m_device_extension_names, &features);
 
     VkPrivateDataSlot data_slot;
     VkPrivateDataSlotCreateInfo data_create_info = vku::InitStructHelper();
-    vk::CreatePrivateDataSlot(m_device->handle(), &data_create_info, NULL, &data_slot);
+    vk::CreatePrivateDataSlot(*m_device, &data_create_info, NULL, &data_slot);
 
     static const uint64_t data_value = 0x70AD;
     m_errorMonitor->SetDesiredError("VUID-vkSetPrivateData-objectHandle-04016");
-    vk::SetPrivateData(m_device->handle(), VK_OBJECT_TYPE_DEVICE, (uint64_t)second_device.handle(), data_slot, data_value);
+    vk::SetPrivateData(*m_device, VK_OBJECT_TYPE_DEVICE, (uint64_t)second_device.handle(), data_slot, data_value);
     m_errorMonitor->VerifyFound();
 
     vkt::Sampler sampler(second_device, SafeSaneSamplerCreateInfo());
     m_errorMonitor->SetDesiredError("VUID-vkSetPrivateData-objectHandle-04016");
-    vk::SetPrivateData(m_device->handle(), VK_OBJECT_TYPE_SAMPLER, (uint64_t)sampler.handle(), data_slot, data_value);
+    vk::SetPrivateData(*m_device, VK_OBJECT_TYPE_SAMPLER, (uint64_t)sampler.handle(), data_slot, data_value);
     m_errorMonitor->VerifyFound();
 
-    vk::DestroyPrivateDataSlot(m_device->handle(), data_slot, nullptr);
+    vk::DestroyPrivateDataSlot(*m_device, data_slot, nullptr);
 }
 
 TEST_F(NegativeTooling, PrivateDataGetNonDevice) {
@@ -116,18 +115,18 @@ TEST_F(NegativeTooling, PrivateDataGetNonDevice) {
     VkPrivateDataSlot data_slot;
     VkPrivateDataSlotCreateInfo data_create_info = vku::InitStructHelper();
     data_create_info.flags = 0;
-    vk::CreatePrivateDataSlot(m_device->handle(), &data_create_info, nullptr, &data_slot);
+    vk::CreatePrivateDataSlot(*m_device, &data_create_info, nullptr, &data_slot);
 
     uint64_t data;
     m_errorMonitor->SetDesiredError("VUID-vkGetPrivateData-objectType-04018");
-    vk::GetPrivateData(m_device->handle(), VK_OBJECT_TYPE_PHYSICAL_DEVICE, (uint64_t)gpu(), data_slot, &data);
+    vk::GetPrivateData(*m_device, VK_OBJECT_TYPE_PHYSICAL_DEVICE, (uint64_t)Gpu(), data_slot, &data);
     m_errorMonitor->VerifyFound();
 
     m_errorMonitor->SetDesiredError("VUID-vkGetPrivateData-objectType-04018");
-    vk::GetPrivateData(m_device->handle(), VK_OBJECT_TYPE_UNKNOWN, (uint64_t)gpu(), data_slot, &data);
+    vk::GetPrivateData(*m_device, VK_OBJECT_TYPE_UNKNOWN, (uint64_t)Gpu(), data_slot, &data);
     m_errorMonitor->VerifyFound();
 
-    vk::DestroyPrivateDataSlot(m_device->handle(), data_slot, nullptr);
+    vk::DestroyPrivateDataSlot(*m_device, data_slot, nullptr);
 }
 
 TEST_F(NegativeTooling, PrivateDataGetBadHandle) {
@@ -137,15 +136,15 @@ TEST_F(NegativeTooling, PrivateDataGetBadHandle) {
 
     VkPrivateDataSlot data_slot;
     VkPrivateDataSlotCreateInfo data_create_info = vku::InitStructHelper();
-    vk::CreatePrivateDataSlot(m_device->handle(), &data_create_info, NULL, &data_slot);
+    vk::CreatePrivateDataSlot(*m_device, &data_create_info, NULL, &data_slot);
 
     uint64_t data;
     m_errorMonitor->SetDesiredError("VUID-vkGetPrivateData-objectHandle-09498");
     // valid handle, but not a vkSample
-    vk::GetPrivateData(m_device->handle(), VK_OBJECT_TYPE_SAMPLER, (uint64_t)device(), data_slot, &data);
+    vk::GetPrivateData(*m_device, VK_OBJECT_TYPE_SAMPLER, (uint64_t)device(), data_slot, &data);
     m_errorMonitor->VerifyFound();
 
-    vk::DestroyPrivateDataSlot(m_device->handle(), data_slot, nullptr);
+    vk::DestroyPrivateDataSlot(*m_device, data_slot, nullptr);
 }
 
 TEST_F(NegativeTooling, PrivateDataGetDestroyedHandle) {
@@ -155,7 +154,7 @@ TEST_F(NegativeTooling, PrivateDataGetDestroyedHandle) {
 
     VkPrivateDataSlot data_slot;
     VkPrivateDataSlotCreateInfo data_create_info = vku::InitStructHelper();
-    vk::CreatePrivateDataSlot(m_device->handle(), &data_create_info, NULL, &data_slot);
+    vk::CreatePrivateDataSlot(*m_device, &data_create_info, NULL, &data_slot);
 
     vkt::Sampler sampler(*m_device, SafeSaneSamplerCreateInfo());
     uint64_t bad_handle = (uint64_t)sampler.handle();
@@ -164,10 +163,10 @@ TEST_F(NegativeTooling, PrivateDataGetDestroyedHandle) {
     uint64_t data;
     m_errorMonitor->SetDesiredError("VUID-vkGetPrivateData-objectHandle-09498");
     // valid handle, but not a vkSample
-    vk::GetPrivateData(m_device->handle(), VK_OBJECT_TYPE_SAMPLER, bad_handle, data_slot, &data);
+    vk::GetPrivateData(*m_device, VK_OBJECT_TYPE_SAMPLER, bad_handle, data_slot, &data);
     m_errorMonitor->VerifyFound();
 
-    vk::DestroyPrivateDataSlot(m_device->handle(), data_slot, nullptr);
+    vk::DestroyPrivateDataSlot(*m_device, data_slot, nullptr);
 }
 
 TEST_F(NegativeTooling, ValidateNVDeviceDiagnosticCheckpoints) {
@@ -178,7 +177,7 @@ TEST_F(NegativeTooling, ValidateNVDeviceDiagnosticCheckpoints) {
 
     uint32_t data = 100;
     m_errorMonitor->SetDesiredError("VUID-vkCmdSetCheckpointNV-commandBuffer-recording");
-    vk::CmdSetCheckpointNV(m_commandBuffer->handle(), &data);
+    vk::CmdSetCheckpointNV(m_command_buffer, &data);
     m_errorMonitor->VerifyFound();
 }
 
@@ -194,21 +193,21 @@ TEST_F(NegativeTooling, PrivateDataDestroyHandle) {
     VkPrivateDataSlot data_slot;
     VkPrivateDataSlotCreateInfo data_create_info = vku::InitStructHelper();
     data_create_info.flags = 0;
-    vk::CreatePrivateDataSlot(m_device->handle(), &data_create_info, nullptr, &data_slot);
+    vk::CreatePrivateDataSlot(*m_device, &data_create_info, nullptr, &data_slot);
 
     vkt::Sampler sampler(*m_device, SafeSaneSamplerCreateInfo());
 
     static const uint64_t data_value = 0x70AD;
-    vk::SetPrivateData(m_device->handle(), VK_OBJECT_TYPE_SAMPLER, (uint64_t)sampler.handle(), data_slot, data_value);
+    vk::SetPrivateData(*m_device, VK_OBJECT_TYPE_SAMPLER, (uint64_t)sampler.handle(), data_slot, data_value);
 
-    vk::DestroyPrivateDataSlot(m_device->handle(), data_slot, nullptr);
+    vk::DestroyPrivateDataSlot(*m_device, data_slot, nullptr);
 
     uint64_t data;
     m_errorMonitor->SetDesiredError("VUID-vkGetPrivateData-privateDataSlot-parameter");
-    vk::GetPrivateData(m_device->handle(), VK_OBJECT_TYPE_SAMPLER, (uint64_t)sampler.handle(), data_slot, &data);
+    vk::GetPrivateData(*m_device, VK_OBJECT_TYPE_SAMPLER, (uint64_t)sampler.handle(), data_slot, &data);
     m_errorMonitor->VerifyFound();
 
     m_errorMonitor->SetDesiredError("VUID-vkSetPrivateData-privateDataSlot-parameter");
-    vk::SetPrivateData(m_device->handle(), VK_OBJECT_TYPE_SAMPLER, (uint64_t)sampler.handle(), data_slot, data_value);
+    vk::SetPrivateData(*m_device, VK_OBJECT_TYPE_SAMPLER, (uint64_t)sampler.handle(), data_slot, data_value);
     m_errorMonitor->VerifyFound();
-}
\ No newline at end of file
+}
